# Actionor 动作器

## 基本

为 GameObject 提供动画交互能力。

由 .fbx 中的动画序列和动作流程构成。

## 属性

- **actionInfo** : object

    当前动作，其结构如下：

    ```js
    {
        type:"entry|any|default|exit|normal",
        name:"xxx",
        animationClip:null,
        animationAction:null,
        nexts:[{
            name:"yyy",
            conditions:"ok==1"
        }],
        repeat:"once|repeat|reverseRepeat"
    }
    ```

    type 可以是 entry、any、default、exit、normal 中的一个。

    - entry，占位用（必须），就是起始
    - any，占位用，任何状态都算作 any（除开 entry/any/exit），比如即便在 walk 状态，也是一种 any 状态，所以 walk 运行中也可以转向到 move，就是把 walk 看作为 any，而 any 有一个触发条件直接转向 move 的。如果没有 any 这种占位符，那连线可能就是蜘蛛网了
    - default，默认状态（必须）
    - exit，出口，占位用，表明状态机逻辑不再生效
    - normal，正常动画播放状态
      

    其中 entry/any/exit 的名字固定为 _entry， _any，_exit

- **actionInfos** : array

    当前 .fbx 文件中所有动作

- **command** : object

    给动作器一个命令，其结构如下：

    ```js
    { 
        type:'action',
        conditions:{}
    }
    ```

    其中 conditions 是用来触发动作流程的。

## 方法

无。

## 事件

- **actionFinished** (et)
  
  一个动画序列完成后。et.name 是当前动画的名字。

  也会触发持有者的 onActionFinished 回调。

## 示例

首先 .fbx 包含多个动画序列，举例如下图：

![](../imgs/QQ截图20191211012900.png)

然后，再对应好名字，构建动作流程，下面是 game.json 文件的一个示例片段：

```json
"gameobjects": [{
    "uuid": "67084ab6-272a-066e-35c3-283486f419e1",
    "name": "myModel",
    "unit": "2bb9c83f-4ec9-72f7-16f1-c7a5327cebcd",
    "actionor":{
        "actionInfos":[
            {
                "type":"entry",
                "name":"_entry",
                "nexts":[{
                    "name":"diceStaticState"
                }]
            },
            {
                "type":"any",
                "name":"_any",
                "nexts":[{
                    "name":"diceCupOpen",
                    "conditions":"action=='open'"
                },{
                    "name":"diceCupInspectOpen",
                    "conditions":"action=='inspectOpen'"
                },{
                    "name":"diceShake",
                    "conditions":"action=='shake'"
                }]
            },
            {
                "type":"normal",
                "name":"diceCupOpen",
                "repeat":"once",
                "nexts":[{
                    "name":"diceCupClose",
                    "conditions":"action=='close'"
                }]
            },{
                "type":"normal",
                "name":"diceCupClose",
                "repeat":"once"
            },{
                "type":"default",
                "name":"diceStaticState",
                "repeat":"once"
            },{
                "type":"normal",
                "name":"diceShake",
                "repeat":"once"
            },{
                "type":"normal",
                "name":"diceCupInspectOpen",
                "repeat":"once",
                "nexts":[{
                    "name":"diceCupInspectClose",
                    "conditions":"action=='close'"
                }]
            },{
                "type":"normal",
                "name":"diceCupInspectClose",
                "repeat":"once"
            }
        ]
    }
}]
```

所表达的含义画出流程图就是：

![](../imgs/QQ截图20191211015220.png)

接下来是使用示例：

```js
var _t=this;
this.btnShake = new Button({
    left: 10,
    centerY: 0,
    text: '摇一摇',
    onClick: function (et) {
        _t.myModel.command = { type: 'action', conditions: { action: 'shake' } };
    }
});
this.btnSee = new Button({
    right: 10,
    centerY: 0,
    text: '看一看',
    onClick: function (et) {
        _t.myModel.command = { type: 'action', conditions: { action: 'inspectOpen' } };
    }
});
this.btnOpen = new Button({
    bottom: 80,
    right: 10,
    skin: 'firstButtonSkin',
    text: '开牌',
    onClick: function (et) {
        _t.myModel.command = { type: 'action', conditions: { action: 'open' } };
    }
});
this.btnClose = new Button({
    bottom: 80,
    left: 10,
    text: '盖牌',
    enabled: false,
    onClick: function (et) {
        _t.myModel.command = { type: 'action', conditions: { action: 'close' } };
    }
});
```