# GBeCool UI 主题机制

一个主题（theme）包含了多个皮肤项（skin）。

默认主题由 /UI/theme/Theme.js 来完成，/objects/Style.js 也做点配套。

/UI/theme 文件夹下面还有一堆 Skin 的类，表达了默认的皮肤样子。

用户撰写的主体保存在 game.json 文件中，而不需要撰写 js 文件。

每个继承于 UIObject 的元素都可以设置 theme 和 skin，也可以统一设置到 SM.theme 中，这样所有元素在没有设置 theme 的时候可以使用 SM.theme，相当于全局主题。

复合元素的皮肤是可以包含子皮肤的，比如弹出对话框里面的按钮就是子皮肤来设置。

皮肤不单单可以指定“样式”类的属性，也可以用来设置一些其他属性的值（不推荐）。

未来的插件中可能会包含有“主题插件”，可以提供一整套的皮肤，直接引用即可。

**注：theme 和 skin 即可以指定为字符串，也可以是对象。**

## 撰写皮肤项

为了降低代码编写量，在每个主题中的皮肤项均采用了继承方式：
```js
/**
 * 普通按钮
 */
get buttonSkin() {
    return Object.assign(this.boxSkin, {
        width: 120,
        height: 30,
        backgroundColor: '#ffffff',
        color: '#ffffff',
        font: '14px sans-serif',
        overflow: 'hidden',
        backgroundImage: '/libs/assets/button.png',
    });
}
```

最基础的样式是 Theme 中的 skin 成员，接下来就是 boxSkin，再接下来就有了 buttonskin 这些皮肤。

上述撰写皮肤项，作为游戏开发者并不需要这样写代码，而是在 GDeveloper 中进行可视化编辑，之后保存在 game.json 文件中。

## 在元素中使用

以 Label 举例，首先要设定默认皮肤项（在 super 之前）：

```js
constructor(options) {
    if (J.isN(options)) options = {};
    options.skin = J.def(options.skin, 'labelSkin');
    super(options);
    ...
```
接下来将皮肤项中的属性与用户传递的属性拼合：

```js
this._align = J.def(this.opts.align, this.skin.align);
```

这里要注意的是，如果是继承的属性就不要写了，父亲已经做过了。

而且 UIObject 将 options.theme / options.skin 已经转移为成员属性了，日后直接访问 xxx.theme / xxx.skin 即可拿到。

每个元素都有默认的皮肤，比如 Label 就是主题中的 labelSkin，但也可以自己来指定。

## 多级皮肤项

皮肤具有父子层级关系。
```js
get titleBarSkin() {
    return Object.assign(this.hboxSkin, {
        height: 40,
        backgroundColor: '#168dc5',
        labelSkin: Object.assign(this.labelSkin, {
            color: "#ffffff"
        })
    });
}
```
使用方法：
```js
this.titleBar = new HBox({
    left: 0,
    right: 0,
    top: 0,
    overflow: 'hidden',
    skin: 'titleBarSkin'
});
this.label = new Label({
    text: 'HELLO',
    centerX: 0,
    skin: this.titleBar.skin.labelSkin
});
```

## Style

配合主题机制，做了一些基础辅助，目前包含：颜色、透明度、渐变纹理，都是静态变量。

示例如下：
```js
var btn=new Button({
    //红色+50%透明度
    backgroundColor: Style.Red+Style.A50
    //蓝色渐变纹理
    backgroundImage: Style.steelBlueTextureOfDialog
});
```

## 事件

- onThemeInited

    当主题初始化完成后。

- onSkinInited

    当皮肤初始化完成后。

## 主题中的皮肤项

- skin

    基础皮肤

- boxSkin

    Box 皮肤

- hboxSkin

    HBox 皮肤

- vboxSkin

    VBox 皮肤

- frameSkin

    页框架皮肤

- pageSkin

    页皮肤。

    子皮肤为：

    - titleBarSkin

        标题栏皮肤。

        子皮肤为：

        - labelSkin
  
            标签皮肤

    - contentBarSkin

        内容栏皮肤

    - bottomBarSkin

        底部栏皮肤。

        子皮肤为：

        - labelSkin
  
            标签皮肤

- buttonSkin

    普通按钮皮肤

- firstButtonSkin

    主按钮皮肤

- secondButtonSkin

    次按钮皮肤

- onlyIconButtonSkin

    图标按钮皮肤

- labelSkin

    标签皮肤

- textinputSkin

    输入框皮肤

- dialogSkin

    对话框皮肤。

    子皮肤为：

    - titleBarSkin

        标题栏皮肤

        子皮肤为：

        - labelSkin
  
            标签皮肤

示例：

```js
var btn=new Button({
    text:"普通按钮",centerX:0,bottom:20,
});
var btn2=new Button({
    text:"主要按钮",centerX:0,bottom:60,
    skin:'firstButtonSkin'
});
var btn3=new Button({
    text:"次要按钮",centerX:0,bottom:100,
    skin:'secondButtonSkin'
});
SM.add(btn);
SM.add(btn2);
SM.add(btn3);
GM.start();
```

效果如下：

![](../imgs/QQ截图20191203014014.png)


