# GBeCool 事件机制

## 基本

GBeCool 事件机制很灵活。

触发事件有两种方法：

- fire
  
  ```js
  a.fire('added');
  ```

- 调用触发函数
  
  ```js
  a.click();
  ```

侦听有三种方法：

- 侦听事件的触发

```js
//侦听，撤销
a.on/off('added',function(){...});
```

- 在子类中重写事件回调

```js
class A extends GameObject{
    onAdded() {
        ...
    }
}
```

- 传递给构造函数

```js
var a=new A({
    onClick:function(){...}
});
```

某些事件，还有对应的状态变量，比如 inited 事件对应 isInited 状态，selected 对应 isSelected。

特别注意的是，**onAdded 会发生在 'added' 事件之前**。

为了改变 this 指向，在 on 时可以传递，如：

```js
a.on('added',function(){...},that);
```

## GameObject 事件流

一个物体从加入到舞台，到被移除，经历如下事件流：

- created
  
  创建。对应回调 onCreated。对应触发函数 create()。

  此回调中，通常处理创建几何体、加载纹理或其他异步事务。

- inited
  
  初始化。对应回调 onInited。对应触发函数 init()。对应标识 isInited。

  此回调中，物体已经创建完成，对物体再进行一些初始化事务。

  千万不要将物体的创建放在这个里面。

- added
  
  被添加到舞台。对应回调 onAdded。对应触发函数 add()。

  如果被添加到舞台是 GameObject，则会自动调用 attachToUnit 方法，以保障符合 THREE.js 的要求。

  args 为添加的对象。

- updated
  
  帧更新。对应回调 onUpdated。对应触发函数 update()。

- removed
  
  被移除。对应回调 onRemoved。对应触发函数 remove()。

  args 为移除的对象。

这个过程是全自动的，在调用 SM.add、SM.remove、gobj1.add、gobj1.remove 时会触发上述事件流。