# GBeCool 插件制作指南

1. [GBeCool 插件制作指南](#gbecool-插件制作指南)
   1. [概述](#概述)
   2. [文件组织](#文件组织)
      1. [config.json](#configjson)
      2. [readme.md](#readmemd)
   3. [写法](#写法)
      1. [习惯](#习惯)
   4. [使用](#使用)
   5. [示例](#示例)

## 概述

插件的作用是为了增补 GBeCool 的功能，但又不属于 GBeCool 核心功能。

在发布时，GBeCool 是采用 GBeCool.min.js 发行，而插件是独立发行，并且要在 plugins 目录下的 config.json 文件中注册。

## 文件组织

plugins 目录与 GBeCool.min.js 同级。

plugins 目录下有一个 config.json 文件，是系统已有插件的列表，形如：

```js
[
  {
    "uuid": "c1c50c35-98fa-4482-8ed3-6dd72c2c0d9a",
    "name": "轨道控制器",
    "varName": "orbitControls",
    "type": "control",
    "src": "OrbitControls/OrbitControls.js",
    "config": "OrbitControls/config.json"
  },
  ...
]
```
- uuid
  
  唯一编号，用户在使用时引用。

- varName
  
  默认变量名，全局唯一，注意取名。用户在使用插件的时候也可以修改。

- type
  
   三选一：object|class|control，分别是对象、类、控制器。

每个插件在 plugins 下面建立一个目录，目录中包含本插件的程序、资源、config.json、readme.md，缺一不可。

### config.json

对插件进行配置。

其内容是一个数组，每项都是 key/value 结构，如：

```js
[{
    "key": "属性名",
    "label": "标签",
    "desc": "描述",
    "value": "默认值",
    "type": "string|number|bool|array|date|Vector2|Vector3|Vector4"
}]
```

但除了上面 5 个必须字段以外，还可以有几个特殊字段：

| 字段    | 值                                                                               | 含义             | 示例                  |
| ------- | -------------------------------------------------------------------------------- | ---------------- | --------------------- |
| control | colorPicker<br />datePicker<br />dropdownlist<br />radioGroup<br />checkboxGroup | 特定控件         |                       |
| source  | 对象数组                                                                         | 对应特定控件的源 | [{value:xx,label:yy}] |

如果在一个目录下有多个插件，配置文件可以命名为：xxxConfig.json、yyyConfig.json ...

用户配置好的数据会保存到 game.json 中的 plugins 的 config 中。 

### readme.md

插件说明文档，里面需要包含基本、属性、方法、事件和示例。

在基本中，要说明插件功能，是类、对象还是控制器，继承谁，访问方式是什么。

在属性、方法和事件中，要对参数的数据类型、默认值、可填值有交代，难懂的要有小例子。

在示例中，举例要简明扼要，不要做一个综合的大例子，反而让读者蒙圈。

整个例子体系中，有属性、方法和事件的小例子，有示例中的小例子，还有教程中的大例子。

## 写法

插件的类必须进行 export default。

无界面呈现的，继承 EventDispatcher；有自定义的界面呈现的，继承 GameObject；完全是 UI 呈现的，继承 UIObject|Box|...

插件直接增加到舞台上的，用 object 类型；插件可以多实例的，用 class 类型；THREE.js 官方控制器的，用 control 类型。

插件的类的底部需附加到 G 上，比如：

```js
G.OutlineFilter = G;
```

G 表示全局对象，在网页中就是 window，在 node.js 中就是 global。

### 习惯

- 如果插件是多实例的，可以采用 attach 方法附着到对象上，也可以使用 target 属性来设置附着对象

## 使用

插件可以在舞台上面添加和配置，其数据保存在 game.json 文件中，格式如下：

```js
plugins:[
    {
        "uuid": "c1c50c35-98fa-4482-8ed3-6dd72c2c0d9b",
        "varName": "fpsControls",
        "config": {
           ...
        }
    }
]
```

- uuid
  
  对应插件唯一编号。

- varName 
  
  插件名，可选。不填则采用插件提供的默认名称。
  
- config
  
  给插件做配置，可选。

## 示例

请参见 libs/plugins 下面已有插件。