# GBeCool 架构

1. [GBeCool 架构](#gbecool-架构)
   1. [整体类图](#整体类图)
   2. [EventDispatcher](#eventdispatcher)
   3. [FrameObject](#frameobject)
   4. [GObject](#gobject)
   5. [GameObject](#gameobject)
   6. [UIObject](#uiobject)
   7. [Manager](#manager)
      1. [SceneManager](#scenemanager)
      2. [ResouceManager](#resoucemanager)
      3. [MapManager](#mapmanager)
      4. [GameManager](#gamemanager)
      5. [LogicManager](#logicmanager)
      6. [ConfigManager](#configmanager)
      7. [DataManager](#datamanager)

GBeCool 是基于 THREE.js 的一套游戏引擎。

## 整体类图

以 EventDispatcher 为顶级父类。

接下来是 FrameObject，所有 GObject 和 Manager 均继承于此类，具备了 EventDispatcher 功能，且与帧(frame)动画相关，拥有了基本的 create、init、update 方法 和 command 模式。

因 Object3D 有 type 成员，为了不冲突，采用了 gtype 和 isXXX 来表达子类，比如 isGObject、isGameObject、isComponent 等。

Object3D 继承 GObject，拥有了focus、draggable、opacity、鼠标事件相关等能力。

注：虽然 Object3D 继承了 GObject，但并不会在帧动画中做 FrameObject/GObject 的工作（比如重绘），只是为了整体框架的封装合理性。

GameObject 和 UIObject，前者具备了组件能力，是一个真正的游戏对象，后者用于 2D UI 交互。

GameObject 不是直接继承 Object3D，也不是 GObject，而是与 Object3D 混合（mixin），通过其 attachToUnit 来完成。

UIObject 继承 Sprite，Sprite 继承 Object3D。

整体类图如下：

![image-20201201165439453](../imgs/image-20201201165439453.png)

对于一个 Manager 或 GameObject，可挂接多个脚本，系统会自动将这些脚本拼合为一个脚本，这大大方便了脚本的协同撰写能力。

注：为了最大程度上不修改 three.min.js，但又遇到类声明的顺序性等问题，本架构会在生成 gbecool.min.js 的同时将一些必要的内容写入到 three.min.js 中。

本架构在对象实例化的时候，都采用传递一个 options 给构造函数，然后 options 里面的字段都会对应到对象的属性，比如：`var gobj=new GameObject({unit:xxx,visible:false})`，然后访问 gobj.unit，gobj.visible 都可。

## EventDispatcher

在 GBeCool 中，EventDispatcher 混合了 THREE.EventDispatcher 的函数。

## FrameObject

增补了与帧(frame)动画相关的能力。

## GObject

在 FrameObject 基础之上增补了父子关系和键鼠交互等能力。

在添加到舞台上之前会调用 init 方法，所有初始化动作都可以在这里完成。如果初始化是异步的，可以在 create 里面完成。

## GameObject

在 GObject 基础之上增补了组件等能力。

## UIObject

在 GObject 基础之上增补了 2D UI 相关能力。

## Manager

管理器基类。

继承链确定如下：

Manager -> FrameObject -> THREE.EventDispatcher 。

### SceneManager

全局简称：SM。

当前舞台管理器，包含 scene 和 uiScene 对象。

scene 包含灯光、物体等，对应摄像头为 camera。

uiScene 包含 2D UI 元素，对应摄像头为 uiCamera。

SM 还包含 children、uiChildren 两个数组，它们就是 SM.scene.children 和 SM.uiScene.children 的简写。

### ResouceManager
### MapManager
### GameManager
### LogicManager
### ConfigManager
### DataManager
