# GBeCool 模块机制

## 概述

微信小程序不支持动态添加组件， 只能先布局好组件，再往组件里面填充数据。使用模块功能可以动态布局界面， 并且模块机制以后在PC端和网页中也可以使用。 

## 配置

模块功能可以在游戏中添加和配置，其数据保存在 game.json 文件中。

格式如下：

```javascript
modules: [
  {
    "name": "留言查看",
    "src": "guestbook/config.json",
    "config": {}
  }
]
```

**name :** 模块功能的名称。

**src：**模块的配置文件的路径， 根据不同的模块路径， 显示不同的数据和排列方式。

**config：**模块配置信息， 该配置信息会与调用的模块config.json进行拼合。



## 文件组织

modules 目录与 plugins 同级。

modules 目录下有一个 conﬁg.json 文件，是已有模块的列表。

每个模块在 modules 下面建立一个目录，目录中只包含conﬁg.json(模块的配置文件)。

### conﬁg.json

对模块功能进行配置。

其内容是一个对象，包括名称， 可调用的api列表。 一个api中包括接口需要调用的底层函数，并传递 params 给该接口，params 中按需传递ownerId/gameId/pageIndex/playerId，这4个参数在调用接口时按需进行赋值。 

如：

```javascript
{
  "name": "留言簿",
  "apis": [
    {
      "name": "查看留言",
      "func": "BeCool.GuestBook.PostControl.GetListWithLinkId",
      "params": [
        {
          "ownerId": -1,
          "gameId": -1,
          "pageIndex": 1,
          "pageSize": 10,
          "orderbys": [
            {
              "fieldName": "PostId",
              "sortType": "Desc"
            }
          ],
          "fieldsAndExtends": "Content,AddTime",
          "template": "<div class='mt10 bdrb'><div class='desc'>{Content}</div> <div class='f12' style='text - align: right'>时间: {AddTime}</div> </div>"
        }
      ]
    }
  ]
}
```

**name:** 模块的名称。

**apis:** 是一个数组， 模块的接口 。

**name:**  接口名称。

**func:**  接口调用的底层函数。根据func发送服务器请求，反馈对应的数据，反馈数据分为两种，一种是 List<T>，一种是 IM，数据与 template 进行拼合。

**params:** 接口的参数， params 中按需传递 ownerId/gameId/pageIndex/playerId。

**template:** 数据显示模板， 在程序中用 J.us.format(template, 通过api获取到的数据) 进行数据与模板的融合，然后就可以在页面呈现出来。 

 

## **用于小程序**

将模块添加到游戏的game.json中， 点击“我的数据”时就会显示该游戏拥有的数据模块列表。比如“留言簿”，点击”留言簿“，列出该模块对应的接口，比如”查看留言“，然后执行该接口对应的 func 返回数据。 

![1](../imgs/GBeCool 模块机制/1.png)

![2](../imgs/GBeCool 模块机制/2.png)



在小程序端使用富文本组件 rich-text，该组件的nodes属性可以支持html节点。 调用模块的接口返回数据后，数据与模板拼合， 然后在wxml页面for循环将结合后的数据赋值给rich-text的nodes。
**显示效果如下：**

![3](../imgs/GBeCool 模块机制/3.png)