# GBeCool 鼠标键盘交互

## 概述

提供了一套 3D 视图中键鼠交互框架。

支持：

- 兼容 PC 和 移动

  比如 onClick 就是 tap/click 的合成

- 采用 Raycaster 方式进行目标对象捕捉

  射线碰撞检测技术，满足了 3D 的要求

- 更佳的封装

  比如为移动端提供了 longTap（长按）事件

- 为 GObject 直接提供事件回调

  每个舞台上 GObject 对象，通过重写自己的相关事件回调即可，如重写 onDown、onClick

- 事件参数更为详细

  如鼠标点击，不仅可以得到屏幕坐标，还可以得到视口坐标和世界坐标

## 鼠标

其中移动部分由 https://github.com/AlloyTeam/AlloyFinger 提供。

事件清单：

| 移动事件           | PC 事件   | 合成事件  | 生效于  | 回调        | 参数 | 说明             |
| ------------------ | --------- | --------- | ------- | ----------- | ---- | ---------------- |
| touchStart         | mousedown | down      | pc/mobi | onDown      |      | 按下             |
| touchMove          | mousemove | move      | pc/mobi | onMove      |      | 移动             |
| touchEnd           | mouseup   | up        | pc/mobi | onUp        |      | 松开             |
| multipointStart    |           | mpdown    | mobi    | onMpDown    |      | 多指按下         |
| multipointEnd      |           | mpup      | mobi    | onMpUp      |      | 多指松开         |
|                    | mouseover | over      | pc      | onOver      |      | 鼠标悬停         |
|                    | mouseout  | out       | pc      | onOut       |      | 鼠标移开         |
| tap                | click     | click     | pc/mobi | onClick     |      | 点击             |
| singleTap          | click     | click     | pc/mobi | onClick     |      | 点击             |
| doubleTap          | dbclick   | dbclick   | pc/mobi | onDbClick   |      | 双击             |
| longTap            |           | longClick | mobi    | onLongClick |      | 长按             |
| pressMove          |           | drag      | pc/mobi | onDrag      |      | 拖拽中(不停触发) |
|                    |           | drop      | pc/mobi | onDrop      |      | 放下             |
|                    |           | dragStart | pc/mobi | onDragStart |      | 拖拽开始         |
|                    |           | dragEnd   | pc/mobi | onDragEnd   |      | 拖拽结束         |
|                    |           | dragEnter | pc/mobi | onDragEnter |      | 拖拽进入         |
|                    |           | dragLeave | pc/mobi | onDragLeave |      | 拖拽离开         |
| twoFingerPressMove |           | tpdrag    | mobi    | onTpDrag    |      | 双指拖拽         |
| swipe              |           | swipe     | pc/mobi | onSwipe     |      | 滑动             |
| pinch              |           | pinch     | mobi    | onPinch     |      | 双指缩放         |
| rotate             |           | rotate    | mobi    | onRotate    |      | 双指旋转         |

其事件参数结构如下：

```json
{
	"positionOfScreen":{"x":0,"y":0},
	"positionOfViewPort":{"x":0,"y":0},
	"positionOfWorld":{"x":0,"y":0,"z":0},
	"gobject":gobj,
	"intersect":ist
}
  ```

其中 positionOfScreen 为屏幕坐标，像素单位。

positionOfViewPort 为视口坐标，从 (0,0) 到 (1,1)。

positionOfWorld 为世界坐标。

gobj 是当前对象，类型为 GObject。

ist 是当前交叠对象，结构为：

```json
{ distance, point, face, faceIndex, object }
```

注意，除了物体能够得到事件触发以外，舞台也会接受一次，比如：

```js
gobj.on('click', ,function(event){ ... })
SM.on('click',function(event){ ... })
```

## 键盘

参考了 https://github.com/jeromeetienne/threex.keyboardstate ，但为直接重写的。

这个处理仅适用于 PC 端。

在玩游戏时，处理方式是不同的。

比如用户是按着 w 不放，可持续性得到 w 的 press/down 事件，然后 w 不松开，再一直按着 d 键不松开，那后续就只能持续性得到 d 的 press/down 事件，w 的 press/down 事件就捕捉不到了。

而且 press 是没法侦测到 ctrl/shift/alt/esc 等按键的，且 press 区分大小写，而 down 不区分，所以 keycode 可能不同。

用法极为简单，可以在 onUpdated 中这样使用：

```js
if(GM.inputor.pressed('shift+a')) ...
```

用加号表示复合按键，不区分大小写。

一些特殊键的别名如下：

```js
{
    'left'		: 37,
	'up'		: 38,
	'right'		: 39,
	'down'		: 40,
	'space'		: 32,
	'pageup'	: 33,
	'pagedown'	: 34,
	'tab'		: 9,
	'escape'	: 27
}
```

比如：

```js
if(GM.inputor.pressed('ctrl+left')) ...
```

也可以自行监听键盘事件：

```js
GM.inputor.on('keypress',function(et){ ... });
```

注意：键盘事件仅适用于 PC 端。

## 虚拟摇杆

参考 https://github.com/jeromeetienne/virtualjoystick.js 。

但这个虚拟摇杆是新建了一个画布来做的，与 GBeCool 的 UI 体系不符，只能参考重写了。

虚拟摇杆暂时是采用插件形式构建，参见 [virtualJoystick 说明文档](../../GBeCool/plugins/virtualJoystick/readme.md)



