# GDeveloper 动画编辑器

本编辑器可以编辑几何物体的动画片段（animation clip）。

在 THREE.js 中，由 AnimationClip 表示动画片段（3dmax 导出的 .fbx 中就包含的是这个，以 Object3D.animations 表示），AnimationAction（动画行为）来控制动画片段的播放和停止等，而由 AnimationMixer 来控制逐帧更新。

注意，本编辑暂不支持骨骼动画，而且也不适用于过于复杂的几何体动画，这两个都可以采用 3dmax 来完成，然后配置好行为器组件即可，无需使用本动画编辑器。

编辑的结果是作为一个带动画的游戏物体存在（GameObject），接下来就可以采用“动作器”来完成动画的切换，与普通游戏物体无异。

## 基础

切换到“资源”中的“动画”选项卡，点击“新建”按钮新建一个动画片段。点击新建好的动画即可打开一个独立的动画编辑器，在“舞台”选项卡后边。

每个动画片段，会在预览中自动加入一个平面（Plane），平面的位移、旋转、缩放和 UV 的位移、旋转来构建所有动画。

动画编辑器的核心是“关键帧”，在时间轴上选择一个点，在右侧属性窗口中点击“插入关键帧”，即可在右侧设置此关键帧的动画。也可以点击“移除关键帧”来删除前面设置好的。

最终的结果都是 THREE.AnimationClip，后存入 game.json。

## 附着

所有特效都会附着在物体上，一个特效有可能从源物体开始一直到目标物体结束，比如法师投射一个火球到一个敌人身上，到达后敌人着火。

对于 GameObject 对象设定 useBox=true，就可以得到该对象的“占地立方体（boundingbox）”，就可以精准的附着在某个位置上。

## 跟踪

很多“子弹”是跟踪式的，比如 LOL 中的火男的大招，只要在施法范围放出来了，不管敌方到哪里，都会跟踪上，然后爆炸。

## 几何体动画

在预览窗口下面有四个图片按钮，分别是：选择（默认）、位移、旋转、缩放，点击不同按钮，几何体上会出现不同的指示器。

右边属性窗口会体现三个属性的值，既可以在预览窗口中用指示器来做，也可以手写属性值来做。

几何体默认为平面，可以更改为立方体、长方体、球体等，更为重要的是，可以更改为你选择一个游戏物体。

### 缩放

借助 Plane 的缩放（scale）实现。

### 位移

借助 Plane 的位置（position）实现。

### 偏移

## 材质动画

主要以透明度为主。采用 material.opacity 实现。

## UV 动画

右边属性窗口几个属性为：贴图（来自资源中的图片粘贴）、位置、旋转（含锚点和角度）。

### 旋转

```js
//默认旋转的锚点是 [0.5,0.5]，左下角是[0,0]，右上角是[1,1]
texture.center.set(0.5, 0);
texture.rotation = getRadian(-180);
```

采用 texture.offset 实现。

比如爆炸效果：

![](../imgs/sprite-explosion2.png)

比如2D任务走动效果：
![](../imgs/walkingManSpriteSheet.png)

### 裁剪

采用着色器或 texture.offset（纹理必须有一节空白）实现。

## 实现逻辑

预览窗口的实现逻辑是：

```js
//加入 Plane
var geometry = new THREE.PlaneGeometry(10, 10, 10);
var material = new THREE.MeshLambertMaterial({color: 0xff0000});
var plane = new THREE.Mesh(geometry, material);
plane.name = 'plane1';
scene.add(plane);

//时间轴上关键帧的时间点
var times = [0, 10];
//对应时间点的位移
var position_x = [0,10];
var position_y = [0,10];
var position_z = [0,10];

//形成位移关键帧
var pos1_Keyframe = new THREE.KeyframeTrack('plane1.position', times, position_x);
var pos2_Keyframe = new THREE.KeyframeTrack('plane1.position', times, position_y);
var pos3_Keyframe = new THREE.KeyframeTrack('plane1.position', times, position_z);

//构建动画片段
var duration = 10;
var clip = new THREE.AnimationClip('boxAnimation', duration, [
    pos1_Keyframe,
    pos2_Keyframe,
    pos3_Keyframe,
]);

//构建 mixer 和 action
var mixer = new THREE.AnimationMixer(group);
var action = mixer.clipAction(clip);
action.setLoop(THREE.LoopRepeat).play();

//在帧动画中调用，让动画动起来
mixer.update(SM.delta);
```