#  GDeveloper 粒子编辑器

粒子编辑器中粒子系统是采用 [SPE](http://squarefeet.github.io/ShaderParticleEngine) 引擎来实现。

编辑逻辑就是构建粒子分组（SPE.Group）和粒子发射器（SPE.Emitter），前者实为粒子的集合，后者来发射粒子，两者的关系就是在 SPE.Group.emitters 中加入 SPE.Emitter，意味着一个分组可以加入一个或多个发射器。

粒子编辑器的结果是给特效器使用。

## 基础

切换到 GDeveloper 的“资源”选项卡，选中粒子，点击“新建分组”按钮新建粒子分组，然后单击“粒子分组”，点击“新建发射器”。

点击分组时，就会打开一个独立的粒子编辑器，在“舞台”选项卡后面，呈现当前设置的默认粒子预览效果。

在右侧就是分组或发射器的属性，修改会及时刷新预览，参考如下：

## 粒子分组

表明一组粒子，多个粒子的集合。

| Name             | Type    | Description                                                                                                                                    |
| :--------------- | :------ | :--------------------------------------------------------------------------------------------------------------------------------------------- |
| `texture`        | Object  | An object describing the texture used by the group.                                                                                            |
| `hasPerspective` | Boolean | Whether the distance a particle is from the camera should affect the particle's size.                                                          |
| `colorize`       | Boolean | Whether the particles in this group should be rendered with color, or whether the only color of particles will come from the provided texture. |
| `blending`       | Number  | One of Three.js's blending modes to apply to this group's `ShaderMaterial`.                                                                    |
| `transparent`    | Boolean | Whether these particle's should be rendered with transparency.                                                                                 |
| `alphaTest`      | Number  | Sets the alpha value to be used when running an alpha test on the `texture.value` property. Value between 0 and 1.                             |
| `depthWrite`     | Boolean | Whether rendering the group has any effect on the depth buffer.                                                                                |
| `depthTest`      | Boolean | Whether to have depth test enabled when rendering this group.                                                                                  |
| `fog`            | Boolean | Whether this group's particles should be affected by their scene's fog.                                                                        |
| `scale`          | Number  | The scale factor to apply to this group's particle sizes. Useful for setting particle sizes to be relative to renderer size.                   |

其中 texture 对象如下：

| Name         | Type   | Description                                                                                                                                                                                              |
| :----------- | :----- | :------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------- |
| `value`      | Object | An instance of THREE.Texture.                                                                                                                                                                            |
| `frames`     | Object | A THREE.Vector2 instance describing the number of frames on the x- and y-axis of the given texture. If not provided, the texture will NOT be treated as a sprite-sheet and as such will NOT be animated. |
| `frameCount` | Number | The total number of frames in the sprite-sheet. Allows for sprite-sheets that don't fill the entire texture.                                                                                             |
| `loop`       | Number | The number of loops through the sprite-sheet that should be performed over the course of a single particle's lifetime.                                                                                   |

### blending model

其中 blending 几个取值如下：

- THREE.NoBlending

  不混合。透明度无效，变黑。

- THREE.NormalBlending

  普通混合。支持透明度，单独使用顶层，而不将其颜色与其下面的层混合。

- THREE.AdditiveBlending

  相加混合。将一个图层的像素值添加到另一个图层。如果值大于1（在RGB的情况下），则显示白色。线性减淡颜色值。由于它总是产生与输入相同或更浅的颜色，因此它也被称为“加亮”。

- THREE.SubtractiveBlending

  减去混合。可以理解为反色，白变黑，红偏蓝。

- THREE.MultiplyBlending

  乘积混合。透明度无效，变黑。源图像RGB分量与目标图像RGB分量的相乘。

示例如下：

![](../imgs/QQ截图20191226181414.png)

可见，Normal 最常用，Additive 用于高光。

### depthTest & depthWrite

![image-20191226213653829](../imgs/image-20191226213653829.png)

可见，要让深度和透明度同时生效，就采用中间的设置。

## 粒子发射器

发射器，用来发射粒子。

| Name               | Type                                                                                               | Description                                                                        |
| :----------------- | :------------------------------------------------------------------------------------------------- | :--------------------------------------------------------------------------------- |
| `type`             | [distribution](http://squarefeet.github.io/ShaderParticleEngine/docs/api/global.html#distribution) | 粒子类型，是立方体、球或2D碟子。                                                   |
| `particleCount`    | Number                                                                                             | 发射器持有多少个粒子。                                                             |
| `duration`         | Number \|null                                                                                      | 发射器持续时长，不指定就总是存在。                                                 |
| `isStatic`         | Boolean                                                                                            | 粒子静止不动。                                                                     |
| `activeMultiplier` | Boolean                                                                                            | 激活乘积，50% 表示只显示触发一半的粒子。                                           |
| `direction`        | Boolean                                                                                            | 从粒子的生命周期的“开始”或“结束”开始发射。                                         |
| `maxAge`           | Object                                                                                             | 指定粒子的最大生命周期。由 value/spread 构成，前者是具体周期，后者是左右偏移周期。 |
| `position`         | Object                                                                                             | 发射器位置。                                                                       |
| `velocity`         | Object                                                                                             | 粒子速度。如只有 z 有速度，且 z>0，则粒子从远处扑向你。                            |
| `acceleration`     | Object                                                                                             | 粒子加速度。如设定 Y=-10，就模拟了重力。                                           |
| `drag`             | Object                                                                                             | 对粒子的速度和加速度做拖拽。                                                       |
| `wiggle`           | Object                                                                                             | 粒子摇摆，与距离有关。                                                             |
| `rotation`         | Object                                                                                             | 发射器旋转。                                                                       |
| `color`            | Object                                                                                             | 粒子颜色变化（按生命周期）。                                                       |
| `opacity`          | Object                                                                                             | 粒子透明度变化（按生命周期）。                                                     |
| `size`             | Object                                                                                             | 粒子大小变化（按生命周期）。                                                       |
| `angle`            | Object                                                                                             | 粒子纹理旋转变化（按生命周期）。                                                   |

