# three.js 读书笔记

## 继承方法

先看下面的 Group 类的代码：

```js
function Group() {
	Object3D.call( this );
	this.type = 'Group';
}
Group.prototype = Object.assign( Object.create( Object3D.prototype ), {
	constructor: Group,
	isGroup: true
} );
```

three.js 将类的某些属性和方法放到了 xxx.prototype 里面，但因为采用 Object.create 方法，故日后再修改 Object3D.prototype 对 Group 是无效的。

当 `new Group()` 的时候会调用构造函数（constructor），里面又会调用 Object3D.call(this)，这句的作用就会将 Group 本身传递给 Object3D 这个函数，将 Object3D 的属性和方法填充给 Group。

因此，类的属性和方法，在两个地方都会给一些。

这种做法是经典的 ES5 之前的类与继承的逻辑，但这里致命缺点就是，如果修改了 Object3D 的 prototype 会对其子类毫无效果，好处就是保障了浏览器兼容性，当然，这种做法最终会被丢弃掉的。

如果要避免这种搞法的缺点，可以直接修改其构造函数，比如 `function Object3D()`，下面给一个例子：

```js
var __Object3D=Object3D;
Object3D=function(){
    __Object3D();
    //其他代码
    this.xxx=1;
    this.yyy=function(){};
};
```

在用户调用构造函数的时候，才会执行。

再者，从 `Object3D.call( this );` 可以看出一点是，任何传递给 Group 构造函数的参数都没有提交给 Object3D 构造函数，导致参数断链。