# 代码编写习惯

- 所有文件采用 UTF-8 编码模式。

- 在 js 回调函数中，用 _t 来代替 this，避免闭包问题。

- 插件的素材放置在插件的目录中，而 GBeCool 核心框架的素材放置在整体的 assets 中。

- 用 type/subType，command/subCommand 形式来表达主/子类型。

- 事件编写逻辑：

    在本类中，定义好事件的逻辑，按如下顺序：

    先注册事件名：`this.events(['started'])`，事件名用过去时。

    增补事件触发和回调的函数（可选）：

    ```js
    start(){

        //do something
        this.fire('started');
        this.onStarted();
    }

    onStarted(){

    }
    ```

    在本类代码中，调用它：`this.start()` 。

    在其他类中，使用这些事件的逻辑如下：

    在其子类中使用，重写：
    
    ```js
    onStarted(){ 
        super.onStarted(); 
        //do something    
    }
    ```

    在外部类使用时，监听：

    ```js
    a.on('started',function(){
        //do something
    });
    ```

- export 和 window 挂载：

    撰写的 .js 类、函数和变量需要同时支持两者，如：

    ```js
    if (typeof module !== 'undefined' && typeof exports === 'object'){
        module.exports.Async = Async;
    }
    if (window) {
        window.Async = Async;
    }
    ```

    小程序中没有 window，就可以采用 `import {Async} from 'xxx.js'` 。

    小游戏和网页中有 window 对象，就可以采用 `import from 'xxx.js'` 。

    导入后，就可以直接使用 Async 了。

    注意，不采用 export default 是因为一般在一个 .js 文件中有多个导出。