
import './three.min.js';
import './tween.min.js';
import './FBXLoader.js';
import './TGALoader.js';
import './astar.js';
import './alloy_finger.js';
import './MirrorShader.js';
import './inflate.min.js';
import './SkeletonUtils.js';
import './SPE.min.js';
/**
    带事件的数组。
    还需要增补 J.ua/ES6 里面所有的方法，使得这个类很强。
*/
export class Collection extends EventDispatcher {

    constructor(options) {
        super(options);
        this.items = [];
    }

    indexOf(key, value) {
        return J.ua.indexOf(this.items, key, value);
    }

    add(item) {
        this.items.push(item);
        this.change('add', item);
    }

    insert(idx, item) {
        J.ua.insert(this.items, idx, item);
        this.change('insert', item);
    }

    remove(item) {
        J.ua.remove(this.items, item);
        this.change('remove', item);
    }

    removeWith(key, value) {
        var item = this.indexOf(key, value);
        this.change('remove', item);
    }

    forEach(callback) {
        for (let i = 0; i < this.items.length; i++) {
            const item = this.items[i];
            callback(item, i);
        }
    }

    getItem(idx) {
        return this.items[idx];
    }

    setItem(idx, item) {
        this.items[idx] = item;
        this.change('update', item);
    }

}

G.Collection = Collection;
/**
 * 绑定相关工具
 */
G.bindingUtils = {

    /**
     * 填充某个成员的值
     * @param {EventDispatcher} source
     * @param {string} name
     * @param {BindingInfo} prop
     */
    fillProp: function (source, name, prop) {

        if (prop.type != 'prop')
            return;

        if (name != prop.sourcePath)
            return;

        var src = source[name];

        if (prop.converter != undefined)
            prop.target[prop.targetProp] = prop.converter(src, prop.converterParams);
        else
            prop.target[prop.targetProp] = src;
    },

    /**
     * 填充所有成员
     * @param {BindingInfo} bindingInfo 
     */
    fillProps(bindingInfo) {

        bindingInfo.children.forEach(function (item) {
            bindingUtils.fillProp(bindingInfo.source, item.sourcePath, item);
        });

    }

};
/**
 * 绑定信息
 */
export class BindingInfo extends EventDispatcher {
    constructor(options) {
        super(options);

        this.source = this.options.source;
        this.sourcePath = this.options.sourcePath;
        this.targetId = this.options.targetId;
        this.target = this.options.target;
        this.subTarget = this.options.subTarget;
        this.targetProp = this.options.targetProp;
        this.converter = this.options.converter;
        this.converterParams = this.options.converterParams;
        this.type = this.options.type;
        this.children = [];

    }
    clone() {
        return J.uo.deepClone(this);
    }
}

G.BindingInfo = BindingInfo;
/**
 * 绑定器
 * @class
 */
export class Binding extends EventDispatcher {

    constructor(options) {
        super(options);

        /**
         * 上次绑定信息\根级 with 绑定\根级 foreach 绑定
         */
        this.lastBindInfo = null;
        this.rootWith = null;
        this.rootForeach = null;
    }

    /**
    * 增加子绑定
    * @param {string} sourcePath 数据源路径
    * @param {object} targetId 目标对象Id
    * @param {string} targetProp 目标对象属性
    * @param {function} converter 转换器
    * @param {*} converterParams 传递给转换器的额外参数
    */
    prop(sourcePath, targetId, targetProp, converter, converterParams) {

        var bd = new BindingInfo({
            sourcePath: sourcePath,
            targetId: targetId,
            targetProp: targetProp,
            converter: converter,
            converterParams: converterParams,
            type: 'prop'
        });

        this.lastBindInfo.children.push(bd);

        return this;
    }

    /**
     * 对象绑定
     * @param {EventDispatcher} source  数据源或路径
     */
    with(source, targetId) {

        var sourcePath = "";
        var target = null;

        //证明是次级绑定
        if (typeof (source) == 'string') {
            sourcePath = source;
            source = null;
        } else {
            target = document.body.find('id', targetId)[0];
            this._fixId(target);
        }

        var bd = new BindingInfo({
            source: source,
            sourcePath: sourcePath,
            target: target,
            targetId: targetId,
            type: 'with',
        });

        if (source != null)
            this.rootWith = bd;

        if (this.lastBindInfo)
            this.lastBindInfo.children.push(bd);

        this.lastBindInfo = bd;

        return this;
    }

    /**
     * 数组绑定。
     * @param {Collection} source 数据源或路径
     * @param {UIObject} target 父容器
     */
    foreach(source, targetId) {

        var target = null;
        var subTarget = null;
        var sourcePath = "";

        //证明是次级绑定
        if (typeof (source) == 'string') {
            sourcePath = source;
            source = null;
        } else {
            target = this._findChild(document.body, targetId);
            this._fixId(target);
            subTarget = target.children[0].clone();
            this._fixId(subTarget);
            //这里将容器中的孩子容器删除，避免后续增删混乱
            target.remove(target.children[0]);
        }

        var bd = new BindingInfo({
            source: source,
            sourcePath: sourcePath,
            target: target,
            targetId: targetId,
            subTarget: subTarget,
            type: 'foreach',
        });

        if (source != null)
            this.rootForeach = bd;

        if (this.lastBindInfo)
            this.lastBindInfo.children.push(bd);

        this.lastBindInfo = bd;

        return this;

    }

    /**
     * 进行根绑定（递归）
     */
    bind() {

        var _t = this;
        if (_t.rootWith)
            _t._bindWith(null, _t.rootWith);
        if (_t.rootForeach)
            _t._bindForeach(null, _t.rootForeach);
    }


    _findChild(parent, id) {
        var r = parent.find('oid', id)[0];
        if (!r)
            r = parent.find('id', id)[0];

        return r;
    }

    _fixId(target) {

        if (target.id.indexOf('nid') == -1) {
            //同时更改对象属性和 DOM 
            target.oid = target.id;
            target.setAttribute('oid', target.id);
        }

        target.id = 'nid_' + J.us.guid4();
    }


    _bindWith(parent, child) {

        var _t = this;
        if (parent != null) {

            child.source = parent.source[child.sourcePath];
            child.target = _t._findChild(parent.target, child.targetId);
            this._fixId(child.target);

        }

        child.source.bindingInfo = child;

        child.children.forEach(function (item) {
            item.target = _t._findChild(child.target, item.targetId);
            if (item.target == undefined)
                throw ("为[" + item.targetId + "]的元素未找到");
            _t._fixId(item.target);
        });

        child.source.on('changed', function (name) {
            var _s = this;
            _s.bindingInfo.children.forEach(function (item) {
                if (item.type == 'prop')
                    bindingUtils.fillProp(_s, name, item);
            });
        });

        bindingUtils.fillProps(child);

        _t._subBindWith(child);
        _t._subBindForeach(child);

    }

    /**
     * 绑定集合项。
     * @param {BindingInfo} bindingInfo 
     * @param {EventDispatcher} item 
     */
    _bindForeachItem(bindingInfo, item) {

        var _t = this;

        var target = bindingInfo.subTarget.clone();
        _t._fixId(target);

        bindingInfo.target.add(target);

        var bd = new BindingInfo({
            source: item,
            target: target,
            type: 'with'
        });

        bindingInfo.children.forEach(function (bi) {
            //不克隆就可能导致 bi 的 target 后者覆盖前者
            bd.children.push(bi.clone());
        });

        _t._bindWith(null, bd);
    }

    _bindForeach(parent, child) {

        var _t = this;
        if (parent != null) {

            child.source = parent.source[child.sourcePath];
            child.target = _t._findChild(parent.target, child.targetId);
            this._fixId(child.target);

        }

        child.source.bindingInfo = child;

        //已有数据：只有数组项才有绑定的真实含义
        child.source.forEach(function (item) {
            _t._bindForeachItem(child, item);
        });

        //新的数据：数组的变更只是增删改查项，与项的属性变化无关
        child.source.on('changed', function (action, item) {

            switch (action) {
                case "add":
                    //为新孩子增补绑定行为
                    _t._bindForeachItem(child, item);
                    break;
                case "insert":
                    break;
                case "update":
                    break;
                case "remove":
                    break;
            }

        });

        /* 接下来无需执行 _subBindWith、_subBindForeach，
         * 因为已经转化为 _bindForeachItem 里面去做了，它后续可以继续执行这两个
        */

    }

    _subBindWith(parent) {
        var _t = this;
        parent.children.forEach(function (item) {
            if (item.type == 'with')
                _t._bindWith(parent, item);
        });
    }

    _subBindForeach(parent) {
        var _t = this;
        parent.children.forEach(function (item) {
            if (item.type == 'foreach')
                _t._bindForeach(parent, item);
        });
    }

}

G.Binding = Binding;
/**
 * 游戏对象。
 * 游戏中的静物也包含在内，但可能缺乏 skiller 等特性。
 * 游戏物体不一定等同于3D，即便是2D的游戏，游戏物体也是从这个类开始，
 * 只有2D UI才是从 uiobject 类开始。
 */
export class GameObject extends GObject {

    constructor(options) {

        super(options);

        this.isGameObject = true;
        this.gtype = 'GameObject';

        this.components = [];

        this.uuid = this.options.uuid;

        /**
         * 对应的模型
         */
        this.unit = this.options.unit;

        /**
         * 角色信息
         */
        this.attachComponent('actor', Actor, this.options.actor);

        /**
         * 移动器
         */
        this.attachComponent('mover', Mover, this.options.mover);

        /**
         * 动作器
         */
        this.attachComponent('actionor', Actionor, this.options.actionor);

        /**
         * 粒子器
         */
        this.attachComponent('psys', ParticleSystem, this.options.psys);

        /**
         * 音效器
         */
        this.attachComponent('sounder', Sounder, this.options.sounder);

        /**
         * 升级器
         */
        this.attachComponent('upgrader', Upgrader, this.options.upgrader);

        /**
         * 碰撞器。仅为战斗中的碰撞，与寻路的碰撞无关。
         */
        this.attachComponent('collider', Collider, this.options.collider);

        /**
         * 攻击器
         */
        this.attachComponent('attacker', Attacker, this.options.attacker);

        /**
        * 背包器
        */
        this.attachComponent('bagger', Bagger, this.options.bagger);

        /**
        * 刚体
        */
        this.attachComponent('rigidbody', Rigidbody, this.options.rigidbody);


    }

    /**
     * 附加组件，如果 options 为空，则不产生组件，
     * 避免内存的浪费
     * @param  {string} name
     * @param  {Class} clazz
     * @param  {object} options
     */
    attachComponent(name, clazz, options) {

        if (options) {
            if (options instanceof Array) {
                var cpts = [];
                for (let i = 0; i < options.length; i++) {

                    var options3 = options[i];
                    var cpt = new clazz(options3);
                    cpt.owner = this;
                    this.components.push(cpt);
                    cpts.push(cpt);
                }
                this[name] = cpts;
            } else {
                var cpt = null;
                cpt = new clazz(options);
                if (cpt) {
                    cpt.owner = this;
                    this.components.push(cpt);
                }
                this[name] = cpt;
            }
        }
    }


    get command() {
        return this._command;
    }

    /**
    * 指令
    * @param {object} cmd  指令
    */
    set command(cmd) {
        this._command = cmd;

        switch (cmd.type) {
            //移动
            case "move":
                this.mover.command = cmd;
                break;
            //攻击
            case "attack":
                this.attacker.command = cmd;
                break;
            //被攻击
            case "attacked":
                this.actor.command = cmd;
                break;
            //碰撞
            case "collid":
                this.collider.command = cmd;
                break;
            //角色
            case "actor":
                this.actor.command = cmd;
                break;
            //行动
            case "action":
                this.actionor.command = cmd;
                break;

            //粒子系统
            case "psys":
                this.psys.command = cmd;
                break;

            //声效
            case "sound":
                this.sounder.command = cmd;
                break;
        }

        this.change('command');

    }

    onInited(args) {

        super.onInited(args);

        for (var i = 0; i < this.components.length; i++) {
            this.components[i].init();
        }

    }

    onUpdated(args) {

        super.onUpdated(args);

        /**
         * 让组件进行一次重绘
         */
        for (var i = 0; i < this.components.length; i++) {
            this.components[i].update();
        }
    }

    /**
     * 将自己附加 unit 对象上，并返回 unit
     */
    attachToUnit(unit) {
        if (unit)
            this.unit = unit;

        return threejsUtils.mixin(this.unit, this);
    }

}

G.GameObject = GameObject;
/**
 * 依赖属性。
 * 与 wpf 概念类似，功能弱一些。
 * 未来再有需求后，要完成：资源、数据绑定、样式、动画、元数据重写、属性值继承、WPF 设计器集成 这些功能。
 * https://docs.microsoft.com/zh-cn/dotnet/framework/wpf/advanced/dependency-properties-overview?view=netframework-4.8
 */
class DependencyProperty extends EventDispatcher {
    constructor(options) {
        super(options);

        this.isDependencyProperty = true;
        this.type = 'DependencyProperty';

        var _t = this;

        /**
         * 该属性的名字
         */
        this.name = J.def(this.options.name, null);

        /**
         * 该属性的持有对象
         */
        this.owner = J.def(this.options.owner, null);

        /**
         * 原始数据
         */
        this.initValue = J.def(this.options.value, null);

        /**
         * 变更前的老值
         */
        this.oldValue = this.initValue;

        /**
         * 新数据
         */
        this.newValue = this.initValue;

        /**
         * 校验模式。
         * 如果为 custom 则 validateCallback 生效，否则按设定的规矩办。
         * 目前支持 int/float/number/string/percent/undefined。
         * 多个请用逗号分隔，表示或关系。
         * 未来还需要支持更多。
         */
        this.validateType = J.def(this.options.validateType, 'custom');

        /**
         * 值校验回调。
         * 进行赋值时会调用，返回 IM，isError为真则不赋值。
         */
        this.validateCallback = J.def(this.options.validateCallback, function () {
            return new IM({ isError: false });
        });

        /**
         * 新值的处理回调。
         * 当值校验成功后，会进行，以便进行默认值处理或数据类型的处理。
         * 需返回你确定好的值。
         */
        this.handleCallback = J.def(this.options.handleCallback, null);

        /**
         * 赋值后回调。
         * 调用顺序：validateCallback(返回真继续) -> handleCallback -> changeCallback
         */
        this.changeCallback = function () {
            _t.owner.change(_t.name, _t.newValue);
        };
    }

    get value() {
        return this.newValue;
    }

    /**
     * 设置值
     * @param  {any} v
     */
    set value(v) {

        if (this.newValue == v)
            return;

        var im = new IM();

        if (this.validateType.indexOf('custom') != -1) {
            im = this.validateCallback(v);
        }

        if (im.isError && this.validateType.indexOf('number') != -1) {
            im.isSuccess = typeof (v) == 'number';
        }

        if (im.isError && this.validateType.indexOf('percent') != -1) {
            im.isSuccess = J.un.isPercent(v);
        }

        if (im.isError && this.validateType.indexOf('undefined') != -1) {
            im.isSuccess = v == undefined;
        }

        //...

        if (im.isSuccess) {

            if (this.handleCallback) {
                this.oldValue = this.newValue;
                this.newValue = this.handleCallback(v);
            }

            this.changeCallback();
        }

    }

    set allValue(v) {
        this.value = v;
        this.initValue = this.value;
    }
}

G.DependencyProperty = DependencyProperty;
/**
 * 输入处理器。含鼠标、触摸、键盘等。
 * 移动部分来自 https://github.com/AlloyTeam/AlloyFinger
 * 键盘部分来自 https://github.com/jeromeetienne/threex.keyboardstate
 * 虚拟摇杆采用插件完成，不属于本类功能。
 * 还存在一个问题：点击的时候有点不灵敏，猜测是某个事件触发了后没有冒泡，导致 click 事件没有触发（特别是在手机上）
 */
export class Inputor extends FrameObject {

	constructor(options) {
		super(options);

		this.raycaster = new THREE.Raycaster();

		this.isInputor = true;
		this.gtype = 'Inputor';


		/**
		 * 与 gameObject 交叠
		 */
		this.intersects = [];
		/**
		 * 与 UI 界面的交叠
		 * @type {Array}
		 */
		this.uiIntersects = [];

		this.lastIntersects = [];
		this.lastUIIntersects = [];

		this.keyCodes = {};
		this.modifiers = {};
		this.MODIFIERS = ['shift', 'ctrl', 'alt', 'meta'];
		this.ALIAS = {
			'left': 37,
			'up': 38,
			'right': 39,
			'down': 40,
			'space': 32,
			'pageup': 33,
			'pagedown': 34,
			'tab': 9,
			'escape': 27
		};

		this.isStartDrag = false;
		//当前正在拖动的物体
		this.dragObject = null;

	}

	onInited(args) {

		super.onInited(args);

		var _t = this;

		var cvs = document.getElementsByTagName('canvas')[0];
		if (J.udt.isMobile) {

			new AlloyFinger(cvs, {

				touchStart: function (et) {
					et.preventDefault();
					_t._mousedownHandle(et);
					_t.isStartDrag = true;
					_t._dragStartHandle(et);
				},

				touchMove: function (et) {
					et.preventDefault();
					_t._mousemoveHandle(et);
				},

				touchEnd: function (et) {
					et.preventDefault();
					_t._mouseupHandle(et);
					_t.isStartDrag = false;
					_t._dragEndHandle(et);
				},

				touchCancel: function (et) {
					et.preventDefault();
					_t._cancelHandle(et);
				},

				multipointStart: function (et) {
					et.preventDefault();
					_t._multipointStartHandle(et);
				},

				multipointEnd: function (et) {
					et.preventDefault();
					_t._multipointEndHandle(et);
				},

				tap: function (et) {
					et.preventDefault();
					_t._clickHandle(et);
				},

				doubleTap: function (et) {
					et.preventDefault();
					_t._dbclickHandle(et);
				},

				longTap: function (et) {
					et.preventDefault();
					_t._longTapHandle(et);
				},

				rotate: function (et) {
					et.preventDefault();
					_t._rotateHandle(et);
				},

				pinch: function (et) {
					et.preventDefault();
					_t._pinchHandle(et);
				},

				swipe: function (et) {
					et.preventDefault();
					_t._swipeHandle(et);
				},

				pressMove: function (et) {
					et.preventDefault();
					_t._dragHandle(et);
				}

			});

		} else {

			cvs.removeEventListener('click', _t.clickHandle);
			cvs.addEventListener('click', function (et) {
				et.preventDefault();
				_t._clickHandle(et);
			}, false);
			cvs.removeEventListener('dblclick', _t.dbclickHandle);
			cvs.addEventListener('dblclick', function (et) {
				et.preventDefault();
				_t._dbclickHandle(et);
			}, false);

			cvs.removeEventListener('mousemove', _t.mousemoveHandle);
			cvs.addEventListener('mousemove', function (et) {
				et.preventDefault();
				_t._mousemoveHandle(et);
				if (_t.isStartDrag)
					_t._dragHandle(et);
			}, false);
			cvs.removeEventListener('mousedown', _t.mousedownHandle);
			cvs.addEventListener('mousedown', function (et) {
				et.preventDefault();
				_t._mousedownHandle(et);
				_t.isStartDrag = true;
				_t._dragStartHandle(et);
			}, false);
			cvs.removeEventListener('mouseup', _t.mouseupHandle);
			cvs.addEventListener('mouseup', function (et) {
				et.preventDefault();
				_t._mouseupHandle(et);
				_t.isStartDrag = false;
				_t._dragEndHandle(et);
			}, false);

			cvs.removeEventListener('keydown', _t._onKeyHandle);
			cvs.addEventListener('keydown', function (et) {
				et.preventDefault();
				_t._onKeyHandle(et);
			}, false);
			cvs.removeEventListener('keyup', _t._onKeyHandle);
			cvs.addEventListener('keyup', function (et) {
				et.preventDefault();
				_t._onKeyHandle(et);
			}, false);

			cvs.removeEventListener("focus", _t._onBlur);
			cvs.addEventListener('focus', function (et) {
				et.preventDefault();
				_t._onFocus(et);
			}, false);

			cvs.removeEventListener("blur", _t._onBlur);
			cvs.addEventListener('blur', function (et) {
				et.preventDefault();
				_t._onBlur(et);
			}, false);
		}

		super.onInited(args);
	}

	/**
	 * 设置一些额外数据
	 */
	_setExtraData(et) {

		var x = !J.isNull(et.touches) ? et.touches[0].clientX : et.clientX;
		var y = !J.isNull(et.touches) ? et.touches[0].clientY : et.clientY;

		//touchend 特殊点
		if (et.type == 'touchend') {
			x = et.changedTouches[0].clientX;
			y = et.changedTouches[0].clientY;
		}

		var pos = {
			x: x,
			y: y
		};

		et.positionOfScreen = new THREE.Vector2(pos.x, pos.y);
		et.positionOfViewPort = measureUtils.toViewPort(pos.x, pos.y);
		et.positionOfWorld = measureUtils.toWorld(et.positionOfViewPort.x, et.positionOfViewPort.y);
	}

	_doIntersect(et) {
		var _t = this;

		var x = et.positionOfScreen.x;
		var y = et.positionOfScreen.y;

		//与 gameobject 碰撞
		_t.intersects = [];
		_t.raycaster.setFromCamera(et.positionOfViewPort, SM.camera);
		var ists = _t.raycaster.intersectObjects(SM.children, true);
		if (ists.length > 0) {
			for (var i = 0; i < ists.length; i++) {
				var ist = ists[i];

				//有时候并不是与物体碰撞
				if (!ist.object) continue;

				//有时候碰撞的不是 unit，而是 unit 中的某个 Mesh
				var gobj = SM.getChild('uuid', ist.object.uuid);
				if (gobj != null) {
					//记录交叠信息
					et.intersect = gobj.intersect = ist;
					_t.intersects.push(gobj);
				}
			}
		}

		//与 uiobject 碰撞
		_t.uiIntersects = [];
		SM.eachUIChild((child) => {
			if (child.containsPoint(x, y))
				_t.uiIntersects.push(child);
		});
	}

	_callback(obj, type, et) {
		var r = true;

		//增补对象到 et 中
		et.gobject = obj;

		switch (type) {

			case 'click':
				r = obj.click(et);
				break;

			case 'dbclick':
				r = obj.dbclick(et);
				break;

			case 'mousedown':
				r = obj.mousedown(et);
				break;

			case 'mouseup':
				r = obj.mouseup(et);
				break;

			case 'mouseover':
				r = obj.mouseover(et);
				break;

			case 'mouseout':
				r = obj.mouseout(et);
				break;

			case 'mousemove':
				r = obj.mousemove(et);
				break;

			case 'mpdown':
				r = obj.mpdown(et);
				break;

			case 'mpup':
				r = obj.mpup(et);
				break;

			case 'swipe':
				r = obj.swipe(et);
				break;

			case 'drag':
				r = obj.drag(et);
				break;

			case 'dragstart':
				r = obj.dragStart(et);
				break;

			case 'dragend':
				r = obj.dragEnd(et);
				break;
		}

		return r;
	}

	_myHandle(type, et) {
		var _t = this;

		if (GM.isInited) {

			_t._setExtraData(et);
			_t._doIntersect(et);

			var isBreak = false;

			//优先 UI 的事件响应，UI 返回 false，那游戏物体也就不再接受事件了
			if (_t.uiIntersects.length > 0) {

				for (var i = 0; i < _t.uiIntersects.length; i++) {
					var ui = _t.uiIntersects[i];
					if (ui.visible == false || ui.enabled == false)
						continue;

					if (!ui.draggable && type.indexOf('drag') != -1)
						continue;

					var r = _t._callback(ui, type, et);
					if (r == false) {
						isBreak = true;
						break;
					}
				}
			}

			//触发对应事件。返回false就不再冒泡
			if (isBreak == false && _t.intersects.length > 0) {

				for (var i = 0; i < _t.intersects.length; i++) {
					var gobj = _t.intersects[i];
					if (gobj.visible == false || gobj.enabled == false)
						continue;

					if (!gobj.draggable && type.indexOf('drag') != -1)
						continue;

					var r = _t._callback(gobj, type, et);
					if (r == false) {
						break;
					}
				}
			}

		};
	}

	/**
	 * 缓冲按键。
	 * 键盘的按下是一瞬间的，但是在按下的瞬间可能有些招数放不出来。
	 * 所以，在某些情况下是需要对按键进行缓冲的。
	 */
	onUpdated(args) {

		super.onUpdated(args);

	}

	_mousemoveHandle(et) {

		var _t = this;
		if (GM.isInited) {

			_t._myHandle('mousemove', et);

			//再处理 mouseout/mouseover 事件（只能通过 mousemove 才能模拟出来）
			var arr = J.ua.except(_t.intersects, _t.lastIntersects);
			for (var i = 0; i < arr.length; i++) {
				_t._callback(arr[i], 'mouseover', et);
			}
			var arr = J.ua.except(_t.uiIntersects, _t.lastUIIntersects);
			for (var i = 0; i < arr.length; i++) {
				_t._callback(arr[i], 'mouseover', et);
			}

			var arr = J.ua.except(_t.lastIntersects, _t.intersects);
			for (var i = 0; i < arr.length; i++) {
				_t._callback(arr[i], 'mouseout', et);
			}
			var arr = J.ua.except(_t.lastUIIntersects, _t.uiIntersects);
			for (var i = 0; i < arr.length; i++) {
				_t._callback(arr[i], 'mouseout', et);
			}

			_t.lastIntersects = _t.intersects;
			_t.lastUIIntersects = _t.uiIntersects;
		}

		this.fire('mousemove', et);
	}

	_clickHandle(et) {
		this._myHandle('click', et);
		this.fire('click', et);
	}

	_dbclickHandle(et) {
		this._myHandle('dbclick', et);
		this.fire('dbclick', et);
	}

	_longTapHandle(et) {

	}

	_cancelHandle(et) {

	}

	_mousedownHandle(et) {
		this._myHandle('mousedown', et);
		this.fire('mousedown', et);
	}

	_mouseupHandle(et) {
		this._myHandle('mouseup', et);
		this.fire('mouseup', et);
	}

	_dragHandle(et) {
		this._myHandle('drag', et);
		//拖动的时候，有时候程序会限制物体的位置，而此时鼠标已经与物体无射线交叠了，就无法触发 drag，这里补齐
		if (this.dragObject)
			this._callback(this.dragObject, 'drag', et);
		this.fire('drag', et);
	}

	_dragStartHandle(et) {
		this._myHandle('dragstart', et);
		if (this.dragObject)
			this._callback(this.dragObject, 'dragstart', et);
		this.fire('dragstart', et);
	}

	_dragEndHandle(et) {
		this._myHandle('dragend', et);
		//这里需要将拖动中的物体停下，毕竟舞台上无法一次拖动多个物体
		if (this.dragObject)
			this._callback(this.dragObject, 'dragend', et);
		this.fire('dragend', et);
	}

	_swipeHandle(et) {
		this._myHandle('swipe', et);
		this.fire('swipe', et);
	}

	_multipointStartHandle(et) {
		this._myHandle('mpup', et);
		this.fire('mpup', et);
	}

	_multipointEndHandle(et) {
		this._myHandle('mpdown', et);
		this.fire('mpdown', et);
	}

	/**
	 * 当失去焦点时，所有按键都设置为没有按下
	 */
	_onBlur() {

		for (var prop in this.keyCodes)
			this.keyCodes[prop] = false;
		for (var prop in this.modifiers)
			this.modifiers[prop] = false;
	}

	_onFocus() {

	}

	_onKeyHandle(et) {

		var keyCode = et.keyCode;
		var pressed = et.type === 'keydown' ? true : false;

		this.keyCodes[keyCode] = pressed;
		this.modifiers['shift'] = et.shiftKey;
		this.modifiers['ctrl'] = et.ctrlKey;
		this.modifiers['alt'] = et.altKey;
		this.modifiers['meta'] = et.metaKey;

		this.fire('keypress', et);
	}

	pressed(keyDesc) {

		var keys = keyDesc.split("+");
		for (var i = 0; i < keys.length; i++) {
			var key = keys[i];
			var pressed = false;

			if (this.MODIFIERS.indexOf(key) !== -1) {
				pressed = this.modifiers[key];
			} else if (Object.keys(this.ALIAS).indexOf(key) != -1) {
				pressed = this.keyCodes[this.ALIAS[key]];
			} else {
				pressed = this.keyCodes[key.toUpperCase().charCodeAt(0)];
			}

			if (!pressed)
				return false;
		}
		return true;
	}
}

G.Inputor = Inputor;
/**
 * 资源加载。
 * 支持对图片、模型、音乐、视频、JSON 等加载。
 * 同时支持事件机制和回调机制。
 */
export class Loader extends EventDispatcher {

    constructor(options) {
        super(options);

        this.loader = null;
    }
    /**
     * 加载多个资源
     * @param  {Array} datas 由 {url,texture} 对象组成的数组，texture 可以是单个或多个，也可以是 string 组成的数组
     * @param  {Function} loadedCallback 可选，装载后的回调
     * @param  {Function} progressCallback 可选，装载进度的回调
     * @param  {Function} failedCallback 可选，装载失败的回调
     */
    loads(datas, loadedCallback, progressCallback, failedCallback) {
        var _t = this;
        var len = datas.length;
        var i = 0;
        var maps = [];
        var nl = new Loader();

        nl.on("loaded", function (map) {
            maps.push(map);
            if (i < len - 1) {
                i++;
                if (typeof (datas[i]) == 'string')
                    nl.load(datas[i]);
                else
                    nl.load(datas[i].url, datas[i].texture);
            } else {
                if (loadedCallback) loadedCallback(maps);
                _t.fire("loaded", maps);
            }
            if (progressCallback) progressCallback(i / len);
            _t.fire("progress", {
                loaded: i,
                total: len
            });
        });

        nl.on("failed", function (err) {
            if (failedCallback) failedCallback(err);
            _t.fire("failed", err);
        });

        if (typeof (datas[i]) == 'string')
            nl.load(datas[i]);
        else
            nl.load(datas[i].url, datas[i].texture);
    }

    /**
     * 加载单个资源。
     * texture 参数可不需要，后续三个参数向前移。
     * @param  {string} url     文件地址
     * @param  {[string|Array]} texture 纹理地址，单个或数组，仅在文件为模型时才需要传递
     * @param  {Function} loadedCallback 可选，装载后的回调
     * @param  {Function} progressCallback 可选，装载进度的回调
     * @param  {Function} failedCallback 可选，装载失败的回调
     * @return {Texture|DataTexture|null}         依据加载器类型返回
     */
    load(url, texture, loadedCallback, progressCallback, failedCallback) {
        var _t = this;

        var ext = J.us.getExtension(url);
        var type = "image";

        //参数迁移
        if (typeof (texture) == "function") {
            var lc = texture;
            var pc = loadedCallback;
            var fc = progressCallback;
            loadedCallback = lc;
            progressCallback = pc;
            failedCallback = fc;
            texture = undefined;
        }

        switch (ext.toLowerCase()) {
            case ".png":
            case ".jpg":
            case ".gif":
            case ".tga":
                type = "image";
                break;
            case ".mp4":
                type = "video";
                break;
            case ".mp3":
            case ".ogg":
                type = "audio";
                break;
            case ".js":
            case ".json":
                type = "json";
                break;
            case ".fbx":
            case ".glb":
            case ".gltf":
                type = "model";
                break;
        }
        switch (type) {
            case "image":
                _t.loadTexture(url, loadedCallback, progressCallback, failedCallback);
                break;
            case "video":
                _t.loadVideo(url, loadedCallback, progressCallback, failedCallback);
                break;
            case "audio":
                _t.loadAudio(url, loadedCallback, progressCallback, failedCallback);
                break;
            case "json":
                _t.loadJSON(url, loadedCallback, progressCallback, failedCallback);
                break;
            case "model":
                _t.loadModel(url, texture, loadedCallback, progressCallback, failedCallback);
                break;
        }
        return _t.loader;
    }

    loadTexture(url, loadedCallback, progressCallback, failedCallback) {
        var _t = this;

        var ext = J.us.getExtension(url);
        var ld = null;
        if (ext == '.tga')
            ld = new THREE.TGALoader();
        else
            ld = new THREE.TextureLoader();

        _t.loader = ld.load(url,
            function (map) {
                if (loadedCallback) loadedCallback(map);
                _t.fire("loaded", map);
            },
            function (xhr) {
                if (progressCallback) progressCallback(xhr);
                _t.fire("progress", xhr);
            },
            function (err) {
                if (failedCallback) failedCallback(err);
                _t.fire("failed", err);
            });
        return _t.loader;
    }

    loadVideo(url, loadedCallback, progressCallback, failedCallback) {

    }

    loadAudio(url, loadedCallback, progressCallback, failedCallback) {

    }

    /**
     * 加载 JSON 文件
     * @param  {[type]} url                 文件地址
     * @param  {Function} loadedCallback   加载完成回调
     * @param  {Function} progressCallback 进度回调
     * @param  {Function} failedCallback   失败回调
     */
    loadJSON(url, loadedCallback, progressCallback, failedCallback) {
        var _t = this;

        _t.loader = J.getJSON(url, null,
            function (r) {
                if (loadedCallback)
                    loadedCallback(r);
                _t.fire("loaded", r);
            },
            function (xhr, ts, et) {
                console.log(xhr, ts, et)
            });

        return _t.loader;
    }

    /**
     * 加载模型
     * @param  {strnig} url                     模型地址，仅支持 .fbx
     * @param  {string|array} texture           纹理，单个或多个
     * @param  {Function} loadedCallback        加载完成回调
     * @param  {Function} progressCallback      进度回调
     * @param  {Function} failedCallback        失败回调
     * @return {Mesh}                           模型
     */
    loadModel(url, texture, loadedCallback, progressCallback, failedCallback) {
        var _t = this;

        //参数迁移
        if (typeof (texture) == "function") {
            var lc = texture;
            var pc = loadedCallback;
            var fc = progressCallback;
            loadedCallback = lc;
            progressCallback = pc;
            failedCallback = fc;
            texture = undefined;
        }
        var loadMesh = function (u, t) {

            var ext = J.us.getExtension(url);
            var ld = null;
            switch (ext) {
                case ".fbx":
                    ld = new THREE.FBXLoader();
                    break;
                case ".glb":
                case ".gltf":
                    ld = new THREE.GLTFLoader();
                    break;
            }
            _t.loader = ld.load(u, function (object) {
                var i = 0;
                //object.scene 有值表明是 gltf 格式
                var obj = object;
                if (object.scene) {
                    obj = object.scene;
                    obj.animations = object.animations;
                }
                obj.traverse(function (child) {
                    if (child.isMesh) {
                        if (t) {
                            if (J.isArray(t)) {
                                child.material = t[i];
                                i++;
                            } else {
                                child.material = t;
                            }
                        }
                    }
                });
                if (loadedCallback) loadedCallback(obj);
                _t.fire("loaded", obj);
            }, function (xhr) {
                if (progressCallback) progressCallback(xhr);
                _t.fire("progress", xhr);
            },
                function (err) {
                    if (failedCallback) failedCallback(err);
                    _t.fire("failed", err);
                }
            );
        };
        if (texture) {
            //一个模型多个纹理
            if (J.isArray(texture)) {
                var arr = [];
                for (var i = 0; i < texture.length; i++) {
                    arr.push(texture[i]);
                }
                new Loader().loads(arr, function (ts) {
                    loadMesh(url, ts);
                });
            } else {
                new Loader().load(texture, function (t) {
                    loadMesh(url, t);
                });
            }
        } else {
            loadMesh(url);
        }
        return _t.loader;
    }

};

G.Loader = Loader;
/**
 * UI 元素基类。
 * 采用 uicamera 摄像头，区别于 camera 场景摄像头，以便保持 UI 总是面对镜头的 2D 画面。
 * 注意：某项数据为空("")表示不设置。
 * 
 * @param {object} options 选项
 */
export class UIObject extends THREE.Sprite {

	constructor(options) {

		/*
		* 默认构造一个空材质的 THREE.Sprite。
		* options 不能传递进去，这样会导致 this.material 变为 options 了
		*/
		super();

		//因为上面没有传递进去，只能自己设定了
		this.options = J.def(options, {});

		this.isUIObject = true;
		this.gtype = 'UIObject';

		//因本类最终与 Sprite 拼合，所以如果没有设置，就按 Sprite 的来。
		if (this.options.uuid)
			this.uuid = this.options.uuid;

		/**
		 * 主题
		 */
		var th = '';
		if (this.options.theme) {
			if (typeof (this.options.theme) == 'string')
				th = new window[this.options.theme];
			else
				th = this.options.theme;
		} else {
			th = SM.theme;
		}
		this.theme = th;
		this.onThemeInited();

		/**
		 * 皮肤
		 */
		var sk = '';
		if (this.options.skin) {
			if (typeof (this.options.skin) == 'string')
				sk = this.theme[this.options.skin];
			else
				sk = this.options.skin;
		} else {
			sk = this.theme['skin'];
		}
		this.skin = sk;
		this.onSkinInited();

		this._width = J.def(this.options.width, this.skin.width);

		this._height = J.def(this.options.height, this.skin.height);

		this._anchorX = J.def(this.options.anchorX, this.skin.anchorX);
		this._anchorY = J.def(this.options.anchorY, this.skin.anchorY);

		this._margin = J.def(this.options.margin, this.skin.margin);

		this._padding = J.def(this.options.padding, this.skin.padding);

		var v = this._padding;
		if (typeof (v) == 'number')
			this._padding = {
				left: v,
				right: v,
				top: v,
				bottom: v
			};
		else
			this._padding = Object.assign({
				left: 0,
				right: 0,
				top: 0,
				bottom: 0,
			}, v);

		v = this._margin;
		if (typeof (v) == 'number')
			this._margin = {
				left: v,
				right: v,
				top: v,
				bottom: v
			};
		else
			this._margin = Object.assign({
				left: 0,
				right: 0,
				top: 0,
				bottom: 0,
			}, v);

		this._x = 0;
		this._y = 0;
		this._z = 1;

		/**
		 * 真实 x/y，做过物体锚点转换和舞台转换之后的，与世界坐标一致
		 */
		this.rx = 0;
		this.ry = 0;

		this._left = J.def(this.options.left, this.skin.left);
		this._top = J.def(this.options.top, this.skin.top);
		this._right = J.def(this.options.right, this.skin.right);
		this._bottom = J.def(this.options.bottom, this.skin.bottom);

		this._centerX = J.def(this.options.centerX, this.skin.centerX);
		this._centerY = J.def(this.options.centerY, this.skin.centerY);

		this._enabled = J.def(this.options.enabled, true);

	}

	onInited(args) {
		super.onInited(args);
	}

	get enabled() { return this._enabled; }
	set enabled(v) {
		if (this._enabled == v)
			return;

		this._enabled = v;

		this.children.forEach(child => {
			child.enabled = v;
		})

		this.change('enabled');
	}

	get left() { return this._left; }
	set left(v) {
		if (this._left == v)
			return;
		this._left = v;

		this.change('left');
	}
	get right() { return this._right; }
	set right(v) {
		if (this._right == v)
			return;
		this._right = v;

		this.change('right');
	}
	get top() { return this._top; }
	set top(v) {
		if (this._top == v)
			return;
		this._top = v;

		this.change('top');
	}
	get bottom() { return this._bottom; }
	set bottom(v) {
		if (this._bottom == v)
			return;
		this._bottom = v;

		this.change('bottom');
	}
	/**
	 * 水平居中偏移量
	 * @type {number}
	 */
	get centerX() { return this._centerX; }
	set centerX(v) {
		if (this._centerX == v)
			return;
		this._centerX = v;

		this.change('centerX');
	}
	/**
	 * 垂直居中偏移量
	 * @type {number}
	 */
	get centerY() { return this._centerY; }
	set centerY(v) {
		if (this._centerY == v)
			return;
		this._centerY = v;

		this.change('centerY');
	}
	/**
	 * 锚点偏移。
	 * 默认情况下，sprite 是以自己的中心点作为本物体的坐标，
	 * 这不符合习惯，应该以它的左上角作为坐标，比如设为0,0，
	 * 意思是某个形状的左上角在0,0点，而不是这个形状的中心点在0,0。
	 * @type {number}
	 */
	get anchorX() {
		if (this._anchorX == undefined)
			return this.width / 2;
		return this._anchorX;
	}
	set anchorX(v) {
		if (this._anchorX == v)
			return;
		this._anchorX = v;

		this.change('anchorX');
	}

	get anchorY() {
		if (this._anchorY == undefined)
			return this.height / 2;
		return this._anchorY;
	}
	set anchorY(v) {
		if (this._anchorY == v)
			return;
		this._anchorY = v;

		this.change('anchorY');
	}

	/**
	 * 外边距
	 * @type {number|object} 数字或 {left:0,right:0,top:0,bottom:0} 模式
	 */
	get margin() {
		return this._margin;
	}
	set margin(v) {
		if (this._margin == v) return;
		this.change('margin');
	}

	/**
	 * 内边距。与 border-box 一致的度量方式，width 包含 padding，但不包含 margin。
	 * @type {number|object} 数字或 {left:0,right:0,top:0,bottom:0} 模式
	 */
	get padding() {
		return this._padding;
	}
	set padding(v) {
		if (this._padding == v) return;
		this.change('padding');
	}

	/**
	 * 父亲宽
	 */
	get parentWidth() {
		return this.parent == SM.uiScene ? SM.wWidth : this.parent.width;
	}
	get parentHeight() {
		return this.parent == SM.uiScene ? SM.wHeight : this.parent.height;
	}
	/**
	 * 宽
	 * @type {number|string}	可以是数字或百分比
	 */
	get width() {
		return this._width;
	}
	set width(v) {
		this._width = v;
	}

	get height() {
		return this._height;
	}
	set height(v) {
		this._height = v;
	}
	/**
	 * x 坐标，是物体的左上角为依据，与 web 体系一致
	 * @type {number}
	 */
	get x() {
		return this._x;
	}
	/**
	 * y 坐标，是物体的左上角为依据，与 web 体系一致
	 * @type {number}
	 */
	get y() {
		return this._y;
	}
	/**
	 * z 坐标，表明深度，越大越在前，范围是 1-99
	 * @type {number}
	 */
	get z() {
		return this._z;
	}
	set z(v) {
		if (this._z == v)
			return;
		this._z = v;
		if (this._z > 99) this._z = 99;

		//因为 add 有顺序问题，所以要递归让孩子再做一次 z 的增加。
		if (!J.isN(this.children)) {
			this.children.forEach((child) => {
				child.z += v;
				if (child.z > 99) child.z = 99;
			});
		}
		this.change('z');
	}

	/**
	 * UI 对应的矩形结构
	 */
	get rect() {

		return {
			x: this.x,
			y: this.y,
			x1: this.x,
			y1: this.y,
			x2: this.x + this.width,
			y2: this.y + this.height,
			width: this.width,
			height: this.height
		};
	}

	/**
	 * 坐标是否在本元素内
	 * @param  {int} x 屏幕坐标
	 * @param  {int} y 屏幕坐标
	 * @return {bool}
	 */
	containsPoint(x, y) {

		var _t = this;

		if (x >= _t.x && x <= _t.x + _t.width &&
			y >= _t.y && y <= _t.y + _t.height)
			return true;

		return false;
	}

	/**
	 * 将宽高依据规则设定好。
	 * 1. 不设置则为父亲高宽
	 * 2. 设置了左右，则用左右度量宽
	 * 3. 设置了上下，则用上下度量高
	 */
	fixWH() {
		var _t = this;

		var ne = J.NE(_t.left, _t.right);
		var ne2 = J.NE(_t.top, _t.bottom);

		if (J.isN(_t.width))
			_t.width = _t.parentWidth;

		if (J.isN(_t.height))
			_t.height = _t.parentHeight;

		if (ne == '11')
			_t.width = _t.parentWidth - _t.left - _t.right;

		if (ne2 == '11')
			_t.height = _t.parentHeight - _t.top - _t.bottom;
	}

	/**
	 * 依据 left/right/top/bottom/centerX/centerY 来修正 xy，
	 * 他们的优先级高于 xy。
	 * 将修正好的 xy 转换为以中心点为 0,0 的坐标模式，
	 * 这个模式是正交相机确立的坐标模式，
	 * 正交相机与透视相机不同，它是直接采用屏幕像素为单位来构建的。
	 * 屏幕左上角为 0,0，右下角为屏幕宽高。
	 * x/y 都假定以物体左上角为参考，但 sprite 本身设置位置的时候，
	 * 是以物体的中心点为参考的，故需要加上 anchorX 和 anchorY。
	 */
	fixXY() {

		var _t = this;

		var ne = J.NE(_t.left, _t.right);
		var ne2 = J.NE(_t.top, _t.bottom);

		//考虑 centerX/centerY，优先级小于 left/right/top/bottom
		if (!J.isN(_t.centerX)) {
			_t._x = (_t.parentWidth - _t.width) / 2 + _t.centerX;
		}
		if (!J.isN(_t.centerY)) {
			_t._y = (_t.parentHeight - _t.height) / 2 + _t.centerY;
		}

		//考虑 left/right/top/bottom
		switch (ne) {
			case '10':
			case '11':
				_t._x = 0 + _t.left;
				break;
			case '01':
				_t._x = _t.parentWidth - _t.width - _t.right;
				break;
		}

		switch (ne2) {
			case '10':
			case '11':
				_t._y = 0 + _t.top;
				break;
			case '01':
				_t._y = _t.parentHeight - _t.height - _t.bottom;
				break;
		}

	}

	/**
	 * 相对于舞台进行平移，
	 * uiScene 是以舞台中心作为原点的
	 * 但这个不能直接给 x/y，不然孩子节点就没法用父亲的 x/y 做参考了
	 */
	transXY() {
		var _t = this;

		_t.rx = _t.x;
		_t.ry = _t.y;

		// fixXY() 的 x/y 均是以物体左上角为 0,0 的，接下来要转换为 Sprite（以中心为 0,0）的 x/y，进行偏移
		_t.rx += _t.anchorX;
		_t.ry += _t.anchorY;

		// 舞台中心是 0,0，因此要转换一下
		_t.rx = _t.rx - _t.parentWidth / 2;
		_t.ry = _t.parentHeight / 2 - _t.ry;
	}

	stretch() {
	}

	/**
	 * 布局自己。
	 * 容器才会布局自己和孩子。
	 */
	layout() {
		var _t = this;

		_t.fixWH();
		_t.fixXY();
		_t.transXY();

		//修改高宽
		//sprite 大小 是不能为0的
		//特别注意：如果一个元素有父亲，所有孩子都是按父亲缩放的，所以指定自己缩放的时候要除以父亲的缩放才是真正的缩放值，比如父亲宽100，实际上是x缩放了100倍（默认缩放是1），那孩子如果宽50，实际上是x缩放100倍后再变成50，所以，x缩放值应该为 50/100=0.5
		if (_t.parent != SM.uiScene)
			_t.scale.set(
				_t.width ? _t.width / _t.parentWidth : 0.1,
				_t.height ? _t.height / _t.parentHeight : 0.1,
				1
			);
		else
			_t.scale.set(
				_t.width ? _t.width : 0.1,
				_t.height ? _t.height : 0.1,
				1
			);

		//修改位置
		//位置的大小与缩放有关系，在默认缩放为1的情况下，xyz是正常的，但如果缩放不为1，则要按比例缩小xyz
		_t.position.set(
			_t.rx * _t.scale.x / _t.width,
			_t.ry * _t.scale.y / _t.height,
			_t.z * _t.scale.z
		);

		var ro = this.onLayout();
		var rf = this.fire('layout');
		return (ro == false || rf == false) ? false : true;
	}

	onLayout() {
		if (this.options.onLayout)
			return this.options.onLayout.apply(this);
	}

	onThemeInited() {
		if (this.options.onThemeInited)
			return this.options.onThemeInited.apply(this);

	}
	onSkinInited() {
		if (this.options.onSkinInited)
			return this.options.onSkinInited.apply(this);
	}

}

G.UIObject = UIObject;



/**
 * 扩展 three.js 相关功能
 */
G.threejsUtils = {};

G.threejsUtils.LEFT = new THREE.Vector3(-1, 0, 0);
G.threejsUtils.RIGHT = new THREE.Vector3(1, 0, 0);
G.threejsUtils.BACK = new THREE.Vector3(0, 0, -1);
G.threejsUtils.FORWARD = new THREE.Vector3(0, 0, 1);

/**
 * 因为three.js解析后的多个对象是以对象存储的，
 * 所以要把数组转成对象，其中uuid作为对象的唯一属性，可以看做是key
 * @param  { [object] } list three3D对象集
 */
G.threejsUtils.arrayToHash = function (list) {

    var obj = {};
    list.map(function (o) {
        obj[o.uuid] = o;
    });
    return obj;

};

/**
 * 设置纹理偏移值
 * @param  {texture} texture 纹理对象
 * @param  {number} offsetX x轴偏移量，通常在 [0-1] 之间
 * @param  {number} offsetY y轴偏移量，通常在 [0-1] 之间
 * @param  {number} repeatX x轴平铺量，大于 1 时，wrapS = RepeatWrapping，以达成理想效果
 * @param  {number} repeatY y轴平铺量，大于 1 时，wrapT = RepeatWrapping，以达成理想效果
 */
G.threejsUtils.uvOffset = function (texture, offsetX, offsetY, repeatX, repeatY, wrapS, wrapT) {

    if (!texture)
        return;

    if (offsetX != undefined)
        texture.offset.set(offsetX, offsetY);
    if (repeatX != undefined)
        texture.repeat.set(repeatX, repeatY);

    texture.wrapS = (!wrapS || repeatX > 1) ? THREE.RepeatWrapping : wrapS;
    texture.wrapT = (!wrapT || repeatY > 1) ? THREE.RepeatWrapping : wrapT;

};

//遍历每个顶点
G.threejsUtils.eachRowAndColumn = function (geometry, callback) {
    //遍历每一行，遍历每一列
    for (var i = 0; i <= geometry.parameters.heightSegments; i++) {
        for (var j = 0; j <= geometry.parameters.widthSegments; j++) {
            callback(i, j);
        }
    }
};

//遍历每个顶点并回调
G.threejsUtils.eachVertex = function (geometry, callback) {
    var vertices = geometry.attributes.position.array;
    for (var i = 0, j = 0, len = vertices.length; j < len; i++, j += 3) {
        var r = callback(vertices[j], vertices[j + 1], vertices[j + 2], i, j);
        vertices[j] = r[0];
        vertices[j + 1] = r[1];
        vertices[j + 2] = r[2];
    }
};

//遍历每个UV并回调
G.threejsUtils.eachUV = function (geometry, callback) {
    var uvs = geometry.attributes.uv.array;
    for (var i = 0, j = 0, len = uvs.length; j < len; i++, j += 2) {
        var r = callback(uvs[j], uvs[j + 1], i, j);
        uvs[j] = r[0];
        uvs[j + 1] = r[1];
    }
};

/*
    * 获取某行某列的顶点（左上为0行0列）并回调
    * geometry：平面体
    * row：行数
    * col：列数
    * callback：回调
*/
G.threejsUtils.vertexAt = function (geometry, row, col, callback) {
    var j = ((geometry.parameters.widthSegments + 1) * row + col) * 3;
    var vertices = geometry.attributes.position.array;
    var r = callback(vertices[j], vertices[j + 1], vertices[j + 2]);
    vertices[j] = r[0];
    vertices[j + 1] = r[1];
    vertices[j + 2] = r[2];
};

//获取某行某列的UV值（左上为0行0列）并回调
G.threejsUtils.uvAt = function (geometry, row, col, callback) {
    var j = ((geometry.parameters.widthSegments + 1) * row + col) * 2;
    var uvs = geometry.attributes.uv.array;
    var r = callback(uvs[j], uvs[j + 1]);
    uvs[j] = r[0];
    uvs[j + 1] = r[1];
};

/**
 * 修正物体接收和投射阴影
 */
G.threejsUtils.fixShadow = function (obj, enabled) {
    obj.traverse(function (child) {
        if (child.isMesh) {
            child.castShadow = enabled;
            child.receiveShadow = enabled;
        }
    });
};

/**
 * 寻找物体的满足条件的孩子
 */
G.threejsUtils.find = function (obj, key, value) {
    var arr = [];
    obj.traverse(function (child) {
        if (child[key] === value) {
            arr.push(child);
        }
    });
    if (arr.length == 1) {
        return arr[0];
    }
    return arr;
};

/**
 * 物体是否在容器内部。
 * @param  {GObject} container
 * @param  {GObject} obj
 * @param  {boolean} refresh 刷新否，当物体大小在变化时使用
 */
G.threejsUtils.contains = function (container, obj, refresh) {

    var ct = container;
    var obj1 = obj;
    if (container instanceof GObject)
        ct = container.unit;
    if (obj instanceof GObject)
        obj1 = obj.unit;

    var box = new THREE.Box3();
    var pos = new THREE.Vector3();

    //防止重复计算
    if (!ct._box3 || refresh) {
        box.setFromObject(ct);
        ct._box3 = box;
    } else
        box = ct._box3;

    if (!ct._wpos || refresh) {
        obj1.getWorldPosition(pos);
        obj1._wpos = pos;
    } else
        pos = obj1._wpos;

    return box.containsPoint(pos);
}

/**
 * 加载纹理。
 * 图片可以是 url | uuid | THREE.Texture | GradientTexture
 */
G.threejsUtils.loadTexture = function (url, onLoaded) {

    if (J.isN(url)) {
        return;
    }

    if (J.us.isUUID(url)) {
        var map = RM.getItem('texture', 'uuid', url);
        onLoaded(map);
    } else if (url.isGradientTexture) {
        onLoaded(url.texture);
    } else if (url.isTexture) {
        onLoaded(url);
    }
    else {
        new Loader().load(url, function (map) {
            onLoaded(map);
        });
    }
}

/**
 * 加载多个纹理。
 */
G.threejsUtils.loadTextures = function (urls, onLoaded) {

    if (J.isN(urls)) {
        return;
    }

    var maps = [];
    var idx = 0;

    var _load = function () {
        threejsUtils.loadTexture(urls[idx], function (map) {
            maps.push(map);
            idx++;
            if (idx < urls.length)
                _load();
            else
                onLoaded(maps);
        });
    }

    _load();
}

/**
 * 支持对 object3d/model 的解析
 */
THREE.ObjectLoader.prototype.parseObjectEx = function (gobj, geometries, materials, callback) {

    //这里面的代码有大部分与 assignWith 有重复，要想办法复用
    if (gobj.type == "Model") {

        new Loader().load(gobj.url, gobj.materials, (obj) => {

            //保持着两个属性不能动
            obj.uuid = gobj.uuid;
            obj.name = gobj.name;

            if (gobj.scale)
                obj.scale.set(gobj.scale.x, gobj.scale.y, gobj.scale.z);

            if (gobj.position)
                obj.position.set(gobj.position.x, gobj.position.y, gobj.position.z);

            if (gobj.rotation)
                obj.rotation.set(gobj.rotation.x, gobj.rotation.y, gobj.rotation.z);

            if (gobj.castShadow && gobj.receiveShadow)
                threejsUtils.fixShadow(obj, true);

            callback(obj);
        },
            function (xhr) {
                console.log(xhr);
            },
            function (err) {
                console.log(err);
            }
        );

    } else {

        //在 objectloader.js 源码中，对于 position/rotation/scale 等都是采用 fromArray 的方式，故先做转换
        if (gobj.position)
            gobj.position = [gobj.position.x, gobj.position.y, gobj.position.z];

        if (gobj.rotation)
            gobj.rotation = [gobj.rotation.x, gobj.rotation.y, gobj.rotation.z];

        if (gobj.scale)
            gobj.scale = [gobj.scale.x, gobj.scale.y, gobj.scale.z];

        var obj = this.parseObject(gobj, geometries, materials);

        //在 objectloader.js 源码中，对于阴影的处理不到位，再次处理下
        if (gobj.castShadow && gobj.receiveShadow)
            threejsUtils.fixShadow(obj, true);

        callback(obj);
    }

};

/**
 * 用目标 dest 混合到 src 对象中，返回混合后的 src 对象。
 * @param {*} src 
 * @param {*} dest 
 */
G.threejsUtils.mixin = function (src, dest) {

    //先给属性
    Object.assign(src, dest);

    //再给原型中的属性或方法
    //getOwnPropertyNames 可以获取枚举或不可枚举的属性 
    var obj2 = Object.getOwnPropertyNames(Object.getPrototypeOf(dest));
    for (var i = 0; i < obj2.length; i++) {

        var key = obj2[i];

        if (key == 'constructor') continue;

        if (key == 'main')
            src.main = dest.main;
        else if (key == 'onInited')
            src.onIniteds.push(dest.onInited);
        else if (key == 'onUpdated')
            src.onUpdateds.push(dest.onUpdated);
        else
            src[key] = dest[key];
    }
    return src;
}



//小游戏自带 window.canvas，但网页不带，保持统一
if (document.getElementsByTagName) {
	window.canvas = document.getElementsByTagName("canvas")[0];
}

/**
 * 画布工具
 */
G.canvasUtils = {
	/**
	 * 创建画布
	 * 如要背景透明，fillStyle 可以是 'rgba(255, 255, 255, 0)'
	 * @param  {int} width  宽
	 * @param  {int} height 高
	 * @param  {color|gradient|pattern} fullfillStyle 填充样式
	 * @return {HTMLCanvasElement}        画布对象，并且置入了 ctx 成员
	 */
	createCanvas: function (width, height, fullfillStyle) {
		var cv = document.createElement('canvas');
		cv.width = width;
		cv.height = height;
		cv.ctx = cv.getContext('2d');
		if (fullfillStyle) {
			//暂不支持带透明度的颜色
			if (fullfillStyle[0] == '#' && fullfillStyle.length == 9)
				return cv;
			cv.ctx.fillStyle = fullfillStyle;
			cv.ctx.fullfill();
		}
		return cv;
	},
	/**
	 * 以图像大小创建画布，并渲染图像在里面
	 * @param  {Image|Canvas|Video} image 图像、画布或视频
	 * @return {HTMLCanvasElement}       画布对象，并且置入了 ctx 成员
	 */
	createImageCanvas: function (image) {
		var cv = canvasUtils.createCanvas(image.width, image.height);
		cv.ctx.drawImage(image, 0, 0);
		return cv;
	},

	/**
	 * 创建填满纹理的画布
	 * @param  {int} width        宽
	 * @param  {int} height       高
	 * @param  {Image} textureImage 纹理图像
	 * @param  {string} repeat       重复模式，repeat|repeat-x|repeat-y|no-repeat
	 * @return {HTMLCanvasElement}              画布对象，并且置入了 ctx 成员
	 */
	createTextureCanvas: function (width, height, textureImage, repeat) {
		var cv = canvasUtils.createCanvas(width, height);
		cv.ctx.fillStyle = cv.ctx.createPattern(textureImage, repeat);
		cv.ctx.fullfill();
		return cv;
	},

	/**
	 * 创建文字画布。
	 * 可以依据文字自身大小创建对应大小的画布（在没有传递高宽参数时）。
	 * 使用时应该要把字体放大一倍，再去贴图，这样会比较清晰。
	 * 还没有支持行高，要继续...
	 * @param  {string} text 文字，支持 \n 进行换行
	 * @param  {[string]} font 字体，参考：http://www.w3school.com.cn/tags/canvas_font.asp
	 * @param  {[string]} style 样式，color|gradient|pattern
	 * @param  {[string]} backgroundStyle 背景样式，color|gradient|pattern
	 * @param  {[string]} type 类型，fill|stroke
	 * @param  {[int]} width 宽，undefined 或 -1为不限制
	 * @param  {[int]} height 高，undefined 或 -1为不限制
	 * @param  {[string]} align 对齐，默认 center
	 * @param  {[bool]} hasShadow 是否有阴影，默认 true
	 */
	createTextCanvas: function (text, font, style, backgroundStyle, type, width, height, align, hasShadow) {

		if (J.isNull(font))
			font = '14px sans-serif';

		if (J.isNull(align))
			align = 'center';

		if (J.isNull(style))
			style = '#000000';

		if (J.isNull(hasShadow))
			hasShadow = true;

		if (J.isNull(backgroundStyle))
			backgroundStyle = '#00000000';

		if (J.isNull(type) || type == 'fill')
			type = 'fill';
		else
			type = 'stroke';

		text = text.replace(/\r/g, '');
		var texts = text.split('\n');
		var lines = texts.length;

		if (width == undefined || width == -1) {
			var tmpcv = canvasUtils.createCanvas(100, 100);
			tmpcv.ctx.font = font;

			//得到最宽的那一行作为宽度
			width = 1;
			texts.forEach((t) => {
				var w = tmpcv.ctx.measureText(t).width;
				if (w > width) width = w;
			});
		}

		var font2 = font.match(/(\d+)px/ig)[0]
		var size = parseInt(font2);

		if (height == undefined || height == -1) {
			//乘以1.4再乘以行数就可以得到最适合文字的高度
			height = size * 1.4 * lines;
		}

		//把宽高、文字大小都放大一倍，这样做出来的纹理就比较清晰
		width *= 2;
		height *= 2;
		font = font.replace(size, size * 2);

		var cv = canvasUtils.createCanvas(width, height, backgroundStyle);
		cv.ctx.textAlign = align;
		cv.ctx.font = font;

		cv.ctx[type + 'Style'] = style;

		cv.ctx.textBaseline = 'middle';

		for (var i = 0; i < texts.length; i++) {
			var t = texts[i];
			if (hasShadow) {
				cv.ctx[type + 'Style'] = J.uc.isLight(style) ? "#000" : "#fff";
				cv.ctx[type + 'Text'](t, width / 2 + 1, (i + 1) * height / (lines + 1) + 1);
				cv.ctx[type + 'Style'] = style;
			}
			cv.ctx[type + 'Text'](t, width / 2, (i + 1) * height / (lines + 1));
		};

		//在外面如果要获取真正的高宽，要除以 2
		return cv;
	},


	/**
	 * 整个画布填充。
	 * 如要背景透明，fillStyle 可以是 'rgba(255, 255, 255, 0)'
	 */
	fullfill: function (ctx, fillStyle) {
		ctx.fillStyle = fillStyle;
		ctx.fillRect(0, 0, ctx.canvas.width, ctx.canvas.height);
	},

	/**
	 * 清除画布
	 */
	clear: function (ctx) {
		ctx.clearRect(0, 0, ctx.canvas.width, ctx.canvas.height);
	},

	/**
	 * 图片
	 */
	image: function (ctx, src, sx, sy, swidth, sheight, x, y, width, height) {
		var img = new Image();
		img.load = function () {
			ctx.drawImage(this, sx, sy, swidth, sheight, x, y, width, height);
		};
		img.src = src;
	},

	/**
	 * 圆
	 */
	circle: function (ctx, x, y, r) {
		ctx.beginPath();
		ctx.arc(x, y, r, 0, 2 * Math.PI);
	},

	/**
	 * 圆角矩形。
	 * x/y 是矩形的左上角，不是中心。
	 */
	roundRect: function (ctx, x, y, w, h, r) {
		if (w < 2 * r) { r = w / 2; }
		if (h < 2 * r) { r = h / 2; }
		ctx.beginPath();
		ctx.moveTo(x + r, y);
		ctx.arcTo(x + w, y, x + w, y + h, r);
		ctx.arcTo(x + w, y + h, x, y + h, r);
		ctx.arcTo(x, y + h, x, y, r);
		ctx.arcTo(x, y, x + w, y, r);
		ctx.closePath();
	}
};

/**
 * context 扩展
 */
if (typeof (window.CanvasRenderingContext2D) != undefined) {

	CanvasRenderingContext2D.prototype.fullfill = function (fillStyle) {
		canvasUtils.fullfill(this, fillStyle);
	};

	CanvasRenderingContext2D.prototype.clear = function () {
		canvasUtils.clear(this);
	};

	CanvasRenderingContext2D.prototype.image = function (src, sx, sy, swidth, sheight, x, y, width, height) {
		canvasUtils.image(this, src, sx, sy, swidth, sheight, x, y, width, height);
	};

	CanvasRenderingContext2D.prototype.circle = function (x, y, r) {
		canvasUtils.circle(this, x, y, r);
	};

	CanvasRenderingContext2D.prototype.roundRect = function (x, y, w, h, r) {
		canvasUtils.roundRect(this, x, y, w, h, r);
	};

}
/**
 * 与量度相关工具，包含矢量、距离、大小、坐标等
 */
G.measureUtils = {

	getWorldPosition: function (gobj) {
		var v3 = new THREE.Vector3();
		gobj.getWorldPosition(v3);
		return v3;
	},

	getWorldDirection: function (gobj) {
		var v3 = new THREE.Vector3();
		gobj.getWorldDirection(v3);
		return v3;
	},

	/**
	 * 将对象转换为 THREE 矢量
	 * @param  {object} obj
	 * @return {THREE.Vector2|THREE.Vector3|THREE.Vector4}
	 */
	toVector: function (obj) {

		if (obj.isVector2 || obj.isVector3 || obj.isVector4)
			return obj;

		if (obj.hasOwnProperty('w')) {
			return new THREE.Vector4(obj.x, obj.y, obj.z, obj.w);
		} else if (obj.hasOwnProperty('z')) {
			return new THREE.Vector3(obj.x, obj.y, obj.z);
		} else
			return new THREE.Vector2(obj.x, obj.y);
	},

	/**
	 * 世界坐标转屏幕坐标
	 * @param  {Vector3} v3     世界坐标
	 * @return {Vector2}        屏幕坐标
	 */
	toScreen: function (v3) {

		//防止摄像头已经移位了
		SM.camera.updateMatrixWorld();

		//三维投射为二维视口坐标
		var vport = new THREE.Vector3(v3.x, v3.y, v3.z);
		vport.project(SM.camera);

		var hw = window.innerWidth / 2;
		var hh = window.innerHeight / 2;

		//二维视口坐标转换为屏幕坐标
		var vscreen = new THREE.Vector2(
			Math.round(vport.x * hw + hw),
			Math.round(-vport.y * hh + hh)
		);

		return vscreen;
	},

	/**
	 * 屏幕坐标转视口坐标
	 * @param  {float} x X
	 * @param  {float} y Y
	 * @return {Vector2}   视口坐标 x(-1,1),y(1,-1)，z为0
	 */
	toViewPort: function (x, y) {
		return new THREE.Vector2(
			(x / window.innerWidth) * 2 - 1,
			-(y / window.innerHeight) * 2 + 1
		);
	},

	/**
	 * 视口坐标转世界坐标
	 * @param {float} x
	 * @param {float} y
	 */
	toWorld: function (x, y, targetZ) {
		if (targetZ == undefined) targetZ = 0;

		//防止摄像头已经移位了
		SM.camera.updateMatrixWorld();

		var pos = new THREE.Vector3();
		var v3 = new THREE.Vector3(x, y, 0.5);
		v3.unproject(SM.camera);
		v3.sub(SM.camera.position).normalize();
		var distance = targetZ - SM.camera.position.z / v3.z;
		pos.copy(SM.camera.position).add(v3.multiplyScalar(distance));
		return pos;
	},

	/**
	 * 获取摄像头视口宽高
	 * @param  {camera} camera 摄像头
	 * @param  {[int]} dist   离摄像头的距离（按 three.js 世界坐标系中的尺寸单位，并非像素），在 near 和 far 之间。
	 * 如未传递，则取原点到摄像头的距离，也就是当前视口大小。
	 * @return {Vector2}        高宽
	 */
	getViewSize: function (dist) {
		if (!dist)
			dist = new THREE.Vector3(0, 0, 0).distanceTo(G.GM.SM.scene.camera.position);
		var fov = THREE.Math.degToRad(G.GM.SM.scene.camera.fov); // convert vertical fov to radians
		var height = 2 * Math.tan(fov / 2) * dist; // visible height
		var width = height * G.GM.SM.scene.camera.aspect; // visible width
		return {
			width: width,
			height: height
		};
	},

	/**
	 * 如要某个高度或宽度的物体能够显示在屏幕中填满，就需要调整摄像头 Z 大小。
	 * @param  {int} size 高度或宽度（按 three.js 世界坐标系中的尺寸单位，并非像素）
	 * @param  {string} type 类型，默认为 height，表示 size 是高度
	 * @return {int}      Z 值
	 */
	getFullViewZ: function (size, type) {
		if (type == undefined)
			type = "height";
		var fov = THREE.Math.degToRad(G.GM.SM.scene.camera.fov);
		var z = 0;
		if (type == 'height')
			z = size / 2 * Math.tan(fov / 2);
		else
			z = size / G.GM.SM.scene.camera.aspect / 2 * Math.tan(fov / 2);
		return z;
	},
	/**
	 * 距离
	 * @param  {THREE.Vector3|GameObject} src 源位置或对象
	 * @param  {THREE.Vector3|GameObject} dest 目标位置或对象
	 * @return {number}     距离
	 */
	distanceTo(src, dest) {
		var sv = src;
		var dv = dest;
		if (dest instanceof GObject)
			dv = dest.unit.position;
		if (src instanceof GObject)
			sv = src.unit.position;

		return sv.distanceTo(dv);
	}
};

/**
 * 组件父类
 */
export class Component extends FrameObject {

	constructor(options) {

		super(options);

		this.isComponent = true;
		this.gtype = 'Component';

		this.owner = this.options.owner;

	}
}

G.Component = Component;
/**
 * 动作器
 */
export class Actionor extends Component {

	constructor(options) {
		super(options);

		this.isActionor = true;
		this.gtype = "Actionor";

		this.animationMixer = null;

		/**
		 * 当前动作
		 */
		this._actionInfo = null;

		/**
		 * 所有动作。
		 * 从 GDeveloper 中采用动画器设计好的结果。
		 * 里面可能包含：entry、any、default、exit、normal 几种类型的动作。
		 * 完整属性如下：
		 * {
		 * 	type:"entry|any|default|exit|normal",
		 * 	name:"xxx",
		 *  animationClip:null,
		 * 	animationAction:null,
		 * 	nexts:[{
		 * 		name:"yyy",
		 * 		conditions:"ok==1"
		 * 	}],
		 *	repeat:"once|repeat|reverseRepeat"
		 * }
		 * 其中 entry/any/exit 的名字固定为 _entry， _any，_exit
		 * 功能的实现几乎都是依赖于 animationAction
		 */
		this.actionInfos = J.def(this.options.actionInfos, []);

	}

	get actionInfo() {
		return this._actionInfo;
	}

	set actionInfo(v) {
		var _t = this;
		_t._actionInfo = v;

		//播动画
		if (_t.actionInfo != null &&
			_t.actionInfo.type != 'exit' &&
			_t.actionInfo.type != 'entry' &&
			_t.actionInfo.type != 'any') {

			if (_t.actionInfo.animationAction != null) {

				_t.actionInfos.forEach((ai) => {
					if (ai.animationAction)
						ai.animationAction.stop();
				});
				_t.actionInfo.animationAction.play();
			}
		}

		_t.change('actionInfo');
	}

	_getLoopType(t) {
		switch (t) {
			case 'once':
				return THREE.LoopOnce;
			case 'repeat':
				return THREE.LoopRepeat;
			case 'reverseRepeat':
				return THREE.LoopPingPong;
		}
	}

	actionFinish(et) {

		this.onActionFinished(et);
		this.fire('actionFinished', et);
	}

	onActionFinished(et) {
		if (this.owner.onActionFinished) {
			this.owner.onActionFinished(et);
		}
	}

	onInited(args) {

		super.onInited(args);

		var _t = this;

		if (!J.isNull(_t.owner.unit.animations)) {

			_t.animationMixer = new THREE.AnimationMixer(_t.owner.unit);
			//一个动画播放完成之后，继续
			_t.animationMixer.addEventListener('finished', function (e) {
				e.name = e.action._clip.name;
				_t.actionFinish(e);
				//触发空条件就是看后面有没有直接接的动画
				_t._trigger();
			});

			//补齐 actionInfos 的 clip 和 loop 属性
			for (var i = 0; i < _t.owner.unit.animations.length; i++) {
				//是 AnimationClip 对象
				var ani = _t.owner.unit.animations[i];

				//修正导出 fbx 时，动画延迟开始的问题，将每个 track 的 times 都从 0 秒开始
				var minDelay = ani.duration;
				for (var j = 0; j < ani.tracks.length; j++) {
					var track = ani.tracks[j];
					var delay = track.times[0];
					if (delay < minDelay) {
						minDelay = delay;
					}
				}
				if (minDelay > 0) {
					ani.duration -= minDelay;
					for (var j = 0; j < ani.tracks.length; j++) {
						var track = ani.tracks[j];
						for (var k = 0; k < track.times.length; k++) {
							track.times[k] -= minDelay;
						}
					}
				}

				var act = _t.animationMixer.clipAction(ani);

				for (var j = 0; j < _t.actionInfos.length; j++) {
					var act2 = _t.actionInfos[j];

					if (act2.type != "any" && act2.type != "exit" && act2.name == ani.name) {

						act2.animationClip = ani;
						act2.animationAction = act;
						act2.animationAction.clampWhenFinished = true;
						act2.animationAction.loop = _t._getLoopType(act2.repeat);
						break;

					}
				}
			}
		}

		//直接进入 entry
		_t.actionInfo = this.actionInfos.find(item => item.type == 'entry');
		//entry 后进入无条件的动画播放
		_t._trigger();
	}

	get command() {
		return this._command;
	}
	set command(cmd) {

		super.command = cmd;
		/**
		 * conditions 的格式是 key/value 模式。
		 */
		this._trigger(cmd.conditions);
	}

	_canTrigger(srcConditions, destConditions) {

		//没有条件就直接为真了
		if (J.isNull(destConditions)) return true;

		//有目标条件，但没有源条件，肯定不行
		if (J.isNull(srcConditions)) return false;

		//将给进来的数据替换设定的条件
		for (const key in srcConditions) {
			if (typeof (srcConditions[key]) == 'string')
				destConditions = destConditions.replace(key, "'" + srcConditions[key] + "'");
			else
				destConditions = destConditions.replace(key, srcConditions[key]);
		}

		//然后计算是否为真
		return eval(destConditions);
	}

	_next(ai, srcConditions) {

		var _t = this;

		if (ai.type == 'exit') {
			_t.actionInfo = null;
			return false;
		}

		if (!J.isN(ai.nexts)) {
			for (let i = 0; i < ai.nexts.length; i++) {
				const next = ai.nexts[i];
				var r = _t._canTrigger(srcConditions, next.conditions);
				if (r == true) {
					this.actionInfo = _t.actionInfos.find(item => item.name == next.name);
					return true;
				}
			}
		}

		return false;
	}

	/**
	 * 触发条件，进行状态切换。
	 * 不能直接设置 actionInfo，应该都采用条件触发。
	 */
	_trigger(conditions) {

		var _t = this;

		var r = false;

		//首先对所有 any 状态的触发器进行处理（所有状态都是 any 状态）
		var cur = _t.actionInfos.find(item => item.type == 'any');
		if (cur != null)
			r = _t._next(cur, conditions);

		//然后再看当前状态的触发器
		if (r == false) {
			cur = _t.actionInfo;
			_t._next(cur, conditions);
		}
	}

	onUpdated(args) {

		var _t = this;

		if (_t.subType == 'cancel') return;

		if (_t.animationMixer != null) {
			_t.animationMixer.update(SM.delta);
		}

	}
}

G.Actionor = Actionor;
/**
 * 角色。
 * 这里的生命值、攻击力等数据是总体数据，
 * 在装备、攻击器、升级器中还会有这些数据，
 * 在游戏运行过程中不停的由各种因素导致它们的变化。
 */
export class Actor extends Component {

	constructor(options) {
		super(options);

		this.isActor = true;
		this.gtype = "Actor";

		/**
		 * 是否可移动
		 */
		this.canMove = J.def(this.options.canMove, false);
		/**
		 * 是否可转向
		 */
		this.canLootAt = J.def(this.options.canLootAt, false);

		/**
		 * 是否可以攻击
		 */
		this.canAttack = J.def(this.options.canAttack, false);

		/**
		 * 是否可被攻击
		 */
		this.canAttacked = J.def(this.options.canAttacked, false);

		/**
		 * 有些物体有攻击系统，但不显示血条，比如子弹
		 */
		this.hasHPBar = J.def(this.options.hasHPBar, true);
		this.hasMPBar = J.def(this.options.hasMPBar, false);
		this.hasEPBar = J.def(this.options.hasEPBar, false);
		this.hasUserNameBar = J.def(this.options.hasUserNameBar, false);
		this.hasAvatarBar = J.def(this.options.hasAvatarBar, false);

		this.username = J.def(this.options.username, '未命名');
		/**
		 * 头像
		 */
		this.avatar = J.def(this.options.avatar, '/libs/assets/user.png');

		/**
		 * bar总容器
		 */
		this.bar = null;
		this.hpBar = null;
		this.mpBar = null;
		this.epBar = null;
		this.usernameBar = null;
		this.avatarBar = null;

		/**
		 * 当前攻击力等信息。
		 * 称为面板数据，由武器、等级、当前攻击方式等决定。
		 * 对这个面板数据进行增减。
		 * 这个数据也作为起始角色数据。
		 */
		this._attackerInfo = new AttackerInfo(this.options.attackerInfo);

		/**
		 * 摧毁获得金币。
		 * 这里还缺失一些，比如摧毁可以得到木头
		 * @type {Number}
		 */
		this.coin = J.def(this.options.coin, 1);
		/**
		 * 建造所需金币
		 * 这里还缺失一些，比如建造需要木头
		 * @type {Number}
		 */
		this.needCoin = J.def(this.options.needCoin, 1);


		//哪边的队伍，pros 正方、cons 反方、neutral 中立、static 静物
		this.side = J.def(this.options.side, "static");

	}

	get attackerInfo() {
		//由各个组件进行拼合，最终得到增减的面板数据
		this.owner.components.forEach((cpt) => {
			if (cpt != this && cpt.attackerInfo && cpt.attackerInfo.isInherit == false) {
				//这里进行数据的增减，反馈给 this._attackerInfo
				//...
			}
		});

		return this._attackerInfo;
	}

	set command(cmd) {

		switch (cmd.subType) {
			//被伤害
			case 'attacked':
				this._attacked(cmd.sender.actor.attackerInfo);
				break;

		}
	}

	die() {

		this.onDied();
		this.fire('died');
	}

	onDied() {

		if (this.owner.onDied) {
			this.owner.onDied();
		}
	}

	/**
	 * 被攻击的伤害计算。
	 * 这个计算还差得远，啥也没有考虑。
	 */
	_attacked(attackerInfo) {

		this.attackerInfo.hp -= attackerInfo.physicalAttack;
		if (this.attackerInfo.hp <= 0) {
			this.die();
		}
	}

	onInited(args) {

		super.onInited(args);
		var _t = this;
		_t.bar = new VBox({ width: 50, height: 110, overflow: 'hidden' });

		if (_t.hasAvatarBar) {
			_t.avatarBar = new Picture({
				url: _t.avatar,
				width: 60,
				height: 60,
				centerX: 0,
			});
			_t.bar.add(_t.avatarBar);
		}

		if (_t.hasUserNameBar) {
			_t.usernameBar = new Label({
				text: _t.name,
				color: Style.White,
				centerX: 0,
			});
			_t.bar.add(_t.usernameBar);
		}

		if (_t.hasHPBar) {
			_t.hpBar = new ProgressBar({
				barBackgroundColor: 0x006600,
				max: _t.attackerInfo.maxhp,
				min: 0,
				value: _t.attackerInfo.hp,
				width: 50,
				height: 5
			});
			_t.bar.add(_t.hpBar);

		}

		if (_t.hasMPBar) {
			_t.mpBar = new ProgressBar({
				barBackgroundColor: 0x000066,
				max: _t.attackerInfo.maxmp,
				min: 0,
				value: _t.attackerInfo.mp,
				width: 50,
				height: 5
			});

			_t.bar.add(_t.mpBar);
		}

		if (_t.hasEPBar) {
			_t.epBar = new ProgressBar({
				barBackgroundColor: 0x666600,
				max: _t.attackerInfo.maxep,
				min: 0,
				value: _t.attackerInfo.ep,
				width: 50,
				height: 5
			});

			_t.bar.add(_t.mpBar);
		}

		SM.add(_t.bar);

		_t.attackerInfo.on('changed', (name) => {
			if (_t.hasHPBar && (name == 'hp' || name == 'maxhp'))
				_t.hpBar.value = 100 * _t.attackerInfo.hp / _t.attackerInfo.maxhp;

			if (_t.hasMPBar && (name == 'mp' || name == 'maxmp'))
				_t.mpBar.value = 100 * _t.attackerInfo.mp / _t.attackerInfo.maxmp;

			if (_t.hasEPBar && (name == 'ep' || name == 'maxep'))
				_t.epBar.value = 100 * _t.attackerInfo.ep / _t.attackerInfo.maxep;
		});

	}

	onUpdated(args) {

		super.onUpdated(args);
		var _t = this;

		if (_t.subType == 'cancel') return;

		if (this.options.attackerInfo && this.hasBar) {

			if (_t.owner.unit) {

				if (_t.owner.lastPosition && _t.owner.lastPosition.equals(_t.owner.unit.position))
					return;

				//跟随物体位置
				//这里应该采用 boundingBox 来实现（未作）
				_t.owner.lastPosition = _t.owner.unit.position.clone();
				var pos = measureUtils.toScreen(_t.owner.unit.position);
				_t.bar.left = pos.x;
				_t.bar.top = pos.y - 80;
				_t.bar.layout();
			}

		}
	}

	onRemoved() {

		super.onRemoved();
		SM.remove(this.bar);

	}

}

G.Actor = Actor;

/**
 * 攻击器。
 * 魔法攻击和普通攻击都是本类。
 * 普攻也可能造成魔法伤害。
 * 给友军加血等操作也属于本类范畴。
 */
export class Attacker extends Component {

	constructor(options) {
		super(options);

		this.isAttacker = true;
		this.gtype = 'Attacker';

		this._targets = [];

		/**
		 * 攻击器里面的多种攻击，比如物理攻击、多个技能（魔法）攻击等，还可以是被动技能
		 */
		this.attackerInfos = [];
		if (this.options.attackerInfos) {
			this.options.attackerInfos.forEach((info) => {
				this.attackerInfos.push(new AttackerInfo(info));
			});
		}

		/**
		 * 当前攻击信息
		 */
		this.attackerInfo = null;

	}

	/**
	 * 攻击目标
	 * @type {Array}
	 */
	get targets() {
		return this._targets;
	}

	set targets(v) {
		this._targets = v;
		this.change('targets');
	}

	set command(cmd) {

		super.command = cmd;

		if (cmd.type != 'attack') return;

		//通过攻击信息的名字来设定当前攻击信息
		this.attackerInfos.forEach((info) => {
			if (info.name === cmd.current)
				this.attackerInfo = info;
		});

		this.targets = cmd.targets;
		this.shoot();
	}

	/**
	 * 找寻攻击范围内的目标（按距离大小排序）。
	 * 包含敌人、中立单位（可攻击的）。
	 * 不同的攻击（普攻和魔法）攻击距离是不同的，
	 * 所以只能放置到这里。
	 * @param  {array} filters 过滤，暂不实现
	 */
	findTargets(filters) {

		var ts = [];

		//可以用网格分区来提升性能（还未实现）
		for (let j = 0; j < SM.children.length; j++) {
			const childj = SM.children[j];

			//不打自己
			if (childj.uuid == this.owner.uuid) continue;
			//不打无角色
			if (!childj.actor) continue;
			//不打不能被攻击
			if (childj.actor.canAttacked == false) continue;
			//不打静物
			if (childj.actor.side == 'static') continue;
			//不打同边（这个地方是有问题的，补血的操作就是给友军的）
			if (this.owner.actor.side == childj.actor.side) continue;

			var dis = measureUtils.distanceTo(this.owner, childj);
			if (dis <= this.attackerInfo.attackRange) {
				childj.distance = dis;
				ts.push(childj);
			}
		}

		//按距离大小排序
		ts.sort(function (a, b) {
			return a.distance - b.distance;
		});

		return ts;
	}

	/**
	 * 发射子弹 或 释放魔法。
	 * 选中目标攻击 或者 自动选择范围里第一个敌人攻击。
	 * 这里没有参考 Actor 的 attackerInfo，这可能不准确。
	 */
	shoot() {

		var _t = this;

		if (_t.attackerInfo && _t.owner.actor.canAttack) {

			//逝去的时间超过攻击速度，则发射新子弹
			if (_t.attackerInfo.shotElapsed > 1 / _t.attackerInfo.attackSpeed) {
				_t.attackerInfo.shotElapsed = 0;

				//指向型攻击
				if (_t.attackerInfo.isDirected) {
					//如果没有指定目标，就找寻最近的在攻击范围的目标攻击
					//暂时只攻击最近的一个单位
					if (J.isN(_t.targets)) {
						var ts = _t.findTargets();
						if (ts.length > 0) {
							_t.targets = [ts[0]];
						}
					}

					if (!J.isN(_t.targets)) {
						var b = _t.attackerInfo.bullet.clone();
						b.owner = _t.owner;
						//子弹的mover设置是被攻击方式覆盖的
						b.mover.subType = 'target';
						//子弹撞上人了，移除
						b.onCollided = function (t, i) {
							J.ua.remove(_t.attackerInfo.bullets, this);
							SM.remove(this);
						};

						//子弹设置的位置要采用相对定位，由 game.json 文件中写好
						b.unit.position.copy(_t.owner.unit.position.clone().add(b.unit.position));
						SM.add(b);
						b.targets = _t.targets;
						_t.attackerInfo.bullets.push(b);
					}
				}
				//非指向型攻击
				else {
					var b = _t.attackerInfo.bullet.clone();
					b.owner = _t.owner;
					b.mover.subType = 'direction';
					//子弹撞上人了，移除
					b.onCollided = function (t, i) {
						J.ua.remove(_t.attackerInfo.bullets, this);
						SM.remove(this);
					};

					//子弹设置的位置要采用相对定位，由 game.json 文件中写好
					b.unit.position.copy(_t.owner.unit.position.clone().add(b.unit.position));
					SM.add(b);
					b.mover.direction = measureUtils.getWorldDirection(_t.owner.unit);
					_t.attackerInfo.bullets.push(b);
				}
			}

		}
	}

	onUpdated(args) {

		super.onUpdated(args);

		if (this.subType == 'cancel') return;

		if (this.attackerInfo) {

			//记录逝去时间，从上次攻击开始
			this.attackerInfo.shotElapsed += SM.delta;

			//自动攻击
			if (this.attackerInfo.isAuto) {
				this.shoot();
			}

			//子弹移动已经超过了攻击距离，移除
			this.attackerInfo.bullets.forEach((b) => {

				//如果是非指向型的子弹，则要时时刻刻给出他可以碰撞的目标
				if (!this.attackerInfo.isDirected) {
					var ts = this.findTargets();
					if (ts.length > 0) {
						b.collider.target = ts[0];
					}
				}

				if (b.mover.distance >= this.attackerInfo.attackRange) {
					J.ua.remove(this.attackerInfo.bullets, b);
					SM.remove(b);
				}

			});
		}
	}

}

G.Attacker = Attacker;
/**
	背包系统
*/
export class Bagger extends Component {

	constructor(options) {
		super(options);

		this.isBagger = true;
		this.gtype = 'Bagger';

		/**
		 * 背包中的物体
		 */
		this.baggerInfos = J.def(this.options.baggerInfos, []);

		/**
		 * 当前背包
		 */
		this.baggerInfo = null;

	}

	onUpdated(args) {

		super.onUpdated(args);
		if (this.subType == 'cancel') return;
	}

}

G.Bagger = Bagger;
/**
 * 碰撞器。
 * 碰撞器是一定需要目标的，请在使用时传递。
 */
export class Collider extends Component {
    constructor(options) {
        super(options);

        this.isCollider = true;
        this.gtype = 'Collider';

        /**
         * 碰撞方式。
         * distance：距离
         * directionAndDistance：方向和距离
         * direction：方向
         */
        this.subType = J.def(this.options.type, 'distance');

        /**
         * 碰撞类型。
         * box：盒子
         */
        this.method = J.def(this.options.method, 'box');

        /**
         * 碰撞距离阈值。
         * 如果是球体的话，应该是球的半径。
         * 因为射线是从球体中心发出，碰上的话，两者的距离也应该是中心+半径的距离。
         */
        this._distance = 0.5;

        /**
         * 如果要按方向碰撞，要给出方向
         */
        this._direction = null;


        this.isCollided = false;

        /**
         * 用来做碰撞的射线。
         * 虽然只用距离也可以判断出是否碰撞，
         * 但没法获知碰撞的面(face)，
         * 射线碰撞可以获知更多的信息。
         */
        this.raycaster = new THREE.Raycaster();

        /**
         * 碰撞对象
         */
        this._target = null;
    }

    /**
     * 目标
     */
    get target() {
        return this._target;
    }

    set target(v) {
        this._target = v;
        this.change('target');
    }

    get distance() { return this._distance; }
    set distance(v) {
        if (v < 0.05) v = 0.1;
        this._distance = v;
        this.change('distance');
    }

    get direction() {
        return this._direction;
    }

    set direction(v) {
        this._direction = v;
        this.change('direction');
    }

    set command(cmd) {

        if (cmd.type != 'collid') return;

        //这种写法是防止冲掉初始化时的数据
        if (cmd.subType)
            this.subType = cmd.subType;
        if (cmd.target)
            this.target = cmd.target;
        if (cmd.distance)
            this.distance = cmd.distance;
        if (cmd.direction)
            this.direction = cmd.direction;

        //如果是方向相关，也可以设置 target 来表示目的方向
        if (cmd.subType == 'direction' || cmd.subType == 'directionAndDistance') {
            if (!cmd.direction && cmd.target) {
                this.direction = this.owner.unit.worldToLocal(cmd.target.unit).clone().normalize();
            }
        }

        super.command = cmd;

    }

    /**
     * 除了给出 target 以外，还要给出 interset 相关信息，以便做更详细的判断
     */
    collid(target, intersect) {

        this.isCollided = true;
        this.onCollided(target, intersect);
        this.fire('collided', target, intersect);
    }

    onCollided(target, intersect) {

        //直接调用 owner 回调，这样，父对象就用不着侦听 collided 事件，直接写上回调即可
        if (this.owner.onCollided) {
            this.owner.onCollided(target, intersect);
        }
    }

    onUpdated(args) {

        super.onUpdated(args);

        if (this.subType == 'cancel') return;
        if (this.isCollided) return;

        switch (this.subType) {

            case "direction":

                if (this.direction && this.target) {
                    this.raycaster.set(this.owner.unit.position, this.direction);

                    var intersections = this.raycaster.intersectObject(this.target.unit);
                    if (intersections.length > 0) {
                        var itst = intersections[0];
                        this.collid(this.target, itst);
                    }
                }
                break;

            case "distance":

                if (this.target) {
                    var dist = measureUtils.distanceTo(this.owner, this.target);
                    if (dist < this.distance) {
                        this.collid(this.target, null);
                    }
                }
                break;

            case "directionAndDistance":

                if (this.direction && this.target) {
                    this.raycaster.set(this.owner.unit.position, this.direction);

                    var intersections = this.raycaster.intersectObject(this.target.unit);
                    if (intersections.length > 0) {
                        var itst = intersections[0];
                        if (itst.distance < this.distance) {
                            this.collid(this.target, itst);
                        }
                    }
                }
                break;

        }
    }

    /**
     * 增补长方体做盒子碰撞。给物体加一个孩子即可。
     */
    _makeBox() {

        var boundingBox = new THREE.Box3().setFromObject(this.owner);
        var v3 = new THREE.Vector3();
        boundingBox.getSize(v3);

        var geo = new THREE.BoxBufferGeometry(v3.x, v3.y, v3.z);
        var mt = new THREE.MeshBasicMaterial({
            color: 0xffff00,
            wireframe: true
        });
        var cb = new THREE.Mesh(geo, mt);
        //盒子本身不可见，但可以用作碰撞检测
        cb.material.visible = false;

        //geo的坐标是以立方体中心点为原点的，但有些模型本身的原点并不一定在中心
        //所以要将 geo 的坐标移至到模型的中心点，才能包裹好模型
        var pos = this.owner.position.clone();
        pos.set(boundingBox.min.x + v3.x / 2,
            boundingBox.min.y + v3.y / 2,
            boundingBox.min.z + v3.z / 2);
        cb.position.copy(pos);

        //将盒子置于自己的孩子，就可以做好碰撞检测
        this.owner.add(cb);
    }
}

G.Collider = Collider;
/**
 * 移动器
 */
export class Mover extends Component {

    constructor(options) {
        super(options);

        this.isMover = true;
        this.gtype = "Mover";

        this._target = null;

        this._position = null;

        this._direction = null;

        this._angle = null;

        /**
         * 可行走路径
         * @type {Array}
         */
        this.pathPoints = [];

        this.wayPoints = [];
        this.patrolPoints = [];

        /**
        * 移动方式：
        * target 接近目标，跟踪式，
        * position 走到某个目的点，
        * angle 按某个角度走（无目标），
        * direction 只按某个方向走(无目标)，
        * path 按寻路算法的路径移动，
        * waypoint 路径点模式，
        * patrol 巡逻，在巡逻点中随机走来走去，
        * interaction 交互，使用键鼠或摇杆或VR来完成移动
        * @type {string}
        */
        this.subType = J.def(this.options.subType, "target");

        /**
         * 已经移动的距离
         */
        this._distance = 0;


    }

    get distance() {
        return this._distance;
    }

    /**
     * 目标
     */
    get target() {
        return this._target;
    }

    set target(v) {
        this._target = v;
        this.change('target');
    }

    get position() {
        return this._position;
    }

    set position(v) {
        this._position = v;
        this.change('position');
    }

    get angle() {
        return this._angle;
    }
    set angle(v) {
        this._angle = v;
        this.change('angle');
    }

    get direction() {
        return this._direction;
    }

    set direction(v) {
        this._direction = v;
        this.change('direction');
    }

    set command(cmd) {

        if (cmd.type != 'move') return;

        if (cmd.subType)
            this.subType = cmd.subType;
        if (cmd.angle)
            this.angle = cmd.angle;
        if (cmd.direction)
            this.direction = cmd.direction;
        if (cmd.target)
            this.target = cmd.target;
        if (cmd.pathPoints)
            this.pathPoints = cmd.pathPoints;
        if (cmd.wayPoints)
            this.wayPoints = cmd.wayPoints;
        if (cmd.patrolPoints)
            this.patrolPoints = cmd.patrolPoints;

        //如果是方向相关，也可以设置 target 来表示目的方向
        if (cmd.subType == 'direction') {
            if (!cmd.direction && cmd.target) {
                this.direction = this.owner.unit.worldToLocal(cmd.target.unit).clone().normalize();
            }
        }

        //如果是目的坐标相关，也可以设置 target 来表示目的坐标
        if (cmd.subType == 'position') {
            if (!cmd.position && cmd.target) {
                this.position = cmd.target.unit.position.clone();
            }
        }

        //侦测鼠标和键盘和摇杆和VR的响应
        if (cmd.subType == 'interaction') {

        }

        super.command = cmd;
    }

    onInited(args) {

        super.onInited(args);
        var _t = this;
        //第一次计算自己的行走路径（不包含自己的位置，pathPoints[0] 就是下一个位置）
        // if (_t.owner.actor.canMove && _t.born && _t.target) {
        // 	//寻路器中只认x/y平面坐标
        // 	var bp = _t.getBornPosition();
        // 	var tp = _t.getTargetPosition();

        // 	_t.pathPoints = SM.scene.pathor.search({
        // 		x: bp.x,
        // 		y: bp.z
        // 	}, {
        // 		x: tp.x,
        // 		y: tp.z
        // 	});
        // }

    }


    /**
     * 接近目标
     */
    toTarget() {
        var _t = this;
        _t.owner.unit.lookAt(_t.target.unit.position);

        //无法靠近到 0，所以取 0.05
        if (measureUtils.distanceTo(_t.owner, _t.target) < 0.05) return;

        var dis = _t.owner.actor.attackerInfo.speed * SM.delta;
        _t.owner.unit.translateOnAxis(
            _t.owner.unit.worldToLocal(_t.target.unit).normalize(),
            dis
        );

        _t._distance += dis;
    }

    toPosition() {
        var _t = this;
        _t.owner.unit.lookAt(_t.position);

        //无法靠近到 0，所以取 0.05
        if (measureUtils.distanceTo(_t.owner, _t.position) < 0.05) return;

        var dis = _t.owner.actor.attackerInfo.speed * SM.delta;
        _t.owner.unit.translateOnAxis(
            _t.owner.unit.worldToLocal(_t.position).normalize(),
            dis
        );

        _t._distance += dis;
    }

    toAngle() {
        var _t = this;

        // 由一个半径为1的圆+夹角，可以求得交点
        var x = Math.sqrt(1 / (1 + Math.pow(Math.tan(_t.angle), 2)));
        // z 的正反与坐标系相反
        var z = -x * Math.tan(_t.angle);
        // tan 值不考虑钝角，只能自己转换
        if (_t.angle > Math.PI / 2 || _t.angle < -Math.PI / 2) {
            x = -x;
            z = -z;
        }

        var v31 = new THREE.Vector3(x, 0, z);
        var v3 = measureUtils.getWorldPosition(_t.owner.unit);
        var v32 = v3.add(v31);

        // lookAt 需要世界坐标
        _t.owner.unit.lookAt(v32);

        var dis = _t.owner.actor.attackerInfo.speed * SM.delta;
        /* 
        * translateOnAxis 需要本地坐标，要用 worldToLocal，
        * 是因为模型本身朝向是 3dmax 中已经确定好的，没有必要去纠结，
        * 不管是朝向哪里，worldToLocal 都可以正确的移动到 v32
        */
        _t.owner.unit.translateOnAxis(
            _t.owner.unit.worldToLocal(v32).normalize(),
            dis
        );
        _t._distance += dis;
    }

    toDirection() {

        var _t = this;

        var v3 = measureUtils.getWorldPosition(_t.owner.unit);
        _t.owner.unit.lookAt(v3.add(_t.direction));

        var dis = _t.owner.actor.attackerInfo.speed * SM.delta;
        //这里使用 threejsUtils.FORWARD 是因为前面做了 lookAtDirection，
        //既然朝向了要求的方向之后，接下来当然就是往前走了
        _t.owner.unit.translateOnAxis(
            threejsUtils.FORWARD,
            dis
        );

        _t._distance += dis;

    }

    toPath() {

        var _t = this;
        //按寻路逻辑走路，pathPoints[0] 就是下一步要走的
        if (_t.pathPoints.length >= 1) {

            var next = new THREE.Vector3(
                _t.pathPoints[0].x,
                _t.unit.position.y,
                _t.pathPoints[0].y
            );

            _t.unit.lookAtTarget(next);
            _t.toTarget(next);

            if (_t.distanceTo(next) < 20) {
                //每次计算自己的路径
                var tp = _t.getTargetPosition();

                _t.pathPoints = SM.scene.pathor.search({
                    x: _t.unit.position.x,
                    y: _t.unit.position.z
                }, {
                    x: tp.x,
                    y: tp.z
                });
            }
        }
        //没有网格需要寻路了，但还没有到达最终目的地
        else if (!_t.isCollided) {
            //已经到达目的地
            // var end = G.Utils.toVector(_t.endPosition);
            // if (Math.round(_t.unit.position.distanceTo(end)) < 5) {
            // 	_t.isCollided = true;
            // 	_t.trigger("collided");
            // }
            _t.toTarget();
            _t.hit();

            //虽然到了最后网格，但并没有达到精准目的地（一个网格宽高是比较大的）
            //else {
            //}
        }
    }

    onUpdated(args) {

        super.onUpdated(args);

        var _t = this;

        if (_t.subType == 'cancel') return;

        if (_t.owner.actor && _t.owner.actor.canMove == false) return;

        switch (_t.subType) {
            case "path":
                _t.toPath();
                break;

            case "target":
                if (_t.owner.actor.canLootAt && _t.target) {
                    _t.toTarget();
                }
                break;

            case "position":
                if (_t.owner.actor.canLootAt && _t.position) {
                    _t.toPosition();
                }
                break;


            case 'angle':
                if (_t.owner.actor.canLootAt && _t.angle) {
                    _t.toAngle();
                }
                break;

            case "direction":
                if (_t.owner.actor.canLootAt && _t.direction) {
                    _t.toDirection();
                }
                break;
        }
    }
}

G.Mover = Mover;
/**
 * 粒子系统
 */
export class ParticleSystem extends Component {

    constructor(options) {
        super(options);

        this.isParticleSystem = true;
        this.gtype = "ParticleSystem";

        this.particles = [];

        if (this.options.particles) {
            var _particles = [];
            this.options.particles.forEach((p) => {
                var _emitters = p.emitters;
                p.emitters = [];
                var g = new SPE.Group(p);
                _particles.push(g);
                _emitters.forEach((e) => {
                    g.addPool(1, e);
                });

            });
            this.particles = _particles;
        }


    }

    onInited(args) {

        super.onInited(args);

        this.particles.forEach(group => {
            this.owner.unit.add(group.mesh);
            group.triggerPoolEmitter(1, this.owner.unit.position);
        });

    }
    onUpdated(args) {

        super.onUpdated(args);

        if (this.subType == 'cancel') return;

        this.particles.forEach(group => {
            group.tick(SM.delta);
        });
    }
}

G.ParticleSystem = ParticleSystem;
/**
 * 刚体系统
 */
export class Rigidbody extends Component {

    constructor(options) {
        super(options);

        this.isRigidbody = true;
        this.gtype = "Rigidbody";

        /**
         * 质量
         */
        this.mass = J.def(this.options.mass, 1);
        /**
         * 重力加速度
         */
        this.gravity = J.def(this.options.gravity, 9.8);

    }

    onUpdated(args) {
        super.onUpdated(args);

        if (this.subType == 'cancel') return;
    }

}

G.Rigidbody = Rigidbody;
/**
	音效器
*/
export class Sounder extends Component {

	constructor(options) {

		super(options);

		this.isSounder = true;
		this.gtype = 'Sounder';

		/**
		 * 所有声效
		 */
		this.audioInfos = J.def(this.options.audioInfos, []);

		/**
		 * 当前声效
		 */
		this.audioInfo = null;

	}

	set command(cmd) {

		super.command = cmd;

		this.audioInfo = this.audioInfos.find((v) => v.name == cmd.name);

		if (this.audioInfo) {
			switch (cmd.action) {

				case 'play':
					this.audioInfo.play();
					break;

				case 'replay':
					this.audioInfo.replay();
					break;

				case 'pause':
					this.audioInfo.pause();
					break;

				case 'stop':
					this.audioInfo.stop();
					break;

			}
		}
	}

	onInited(args) {

		super.onInited(args);

		var _t = this;

		for (var i = 0; i < this.audioInfos.length; i++) {

			var info = this.audioInfos[i];

			//将 uuid 变为真正的声音
			if (J.us.isUUID(info)) {
				info = RM.getItem('audio', 'uuid', info);
				this.audioInfos[i] = info;
			}
		}

	}

	onUpdated(args) {
		super.onUpdated(args);

		if (this.subType == 'cancel') return;
	}

}

G.Sounder = Sounder;
/**
	升级器
*/
export class Upgrader extends Component {

	constructor(options) {
		super(options);

		this.isUpgrader = true;
		this.gtype = 'Upgrader';

		/**
		 * 当前等级
		 */
		this.upgraderInfo = null;

		/**
		 * 当前等级数字
		 */
		this._level = this.options.level;

		//默认前十级的等级和经验
		var _infos = [
			[0, 0],
			[1, 3],
			[2, 5],
			[3, 8],
			[4, 12],
			[5, 17],
			[6, 23],
			[7, 30],
			[8, 38],
			[9, 47],
			[10, 71]
		];

		var _upgraderInfos = [];
		for (var i = 0; i < _infos.length; i++) {
			_upgraderInfos.push(
				new UpgraderInfo({
					level: _infos[i][0],
					exp: _infos[i][1],
					attackerInfo: new AttackerInfo()
				})
			);
		}

		/**
		 * 本角色对应的升级体系
		 */
		this.upgraderInfos = J.def(this.options.upgraderInfos, _upgraderInfos);

	}

	get level() {
		return this._level;
	}

	set level(v) {
		this._level = v;
		for (var i = 0; i < this.upgraderInfos.length; i++) {
			if (this.upgraderInfos[i].level == v)
				this.upgraderInfo = this.upgraderInfos[i];
		}
		this.change('level');
	}

	onUpdated(args) {
		super.onUpdated(args);

		if (this.subType == 'cancel') return;
	}

}

G.Upgrader = Upgrader;
/**
	子弹基类。
	模拟了普通子弹。
	子弹就是一个物体，因此在 game.json 中是做到 gameobjects 中的，
	相关属性也是按照物体的设定方式，比如 mover，unit.position 等
*/
export class Bullet extends GameObject {

	constructor(options) {
		super(options);

		this.isBullet = true;
		this.gtype = 'Bullet';

		this.owner = this.options.owner;

		//自己加上，不接受外部配置
		this.attachComponent('actor', Actor, {
			canMove: true,
			canLootAt: true,
			//子弹是不显示血条的，没这个概念
			hasHPBar: false,
			hasMPBar: false,
			hasEPBar: false,
			hasNameBar: false,
			hasAvatarBar: false,
		});

		this.attachComponent('mover', Mover, this.options.mover);

		this.attachComponent('collider', Collider, this.options.collider);

		this._targets = [];
	}

	/**
	 * 攻击目标，第一个
	 */
	get target() {
		return this._targets.length > 0 ? this._targets[0] : null;
	}

	/**
	 * 攻击目标，多个
	 * @type {Array}
	 */
	get targets() {
		return this._targets;
	}

	set targets(v) {

		this._targets = v;
		this.mover.target = this.target;
		this.collider.target = this.target;
		this.change('targets');
	}

	_colliderHandler() {

		var _t = this;
		_t.collider.off(_t._colliderHandler);
		_t.target.command = {
			type: 'actor',
			subType: 'attacked',
			sender: _t.owner
		};
	}

	onInited(args) {

		var _t = this;
		_t.collider.method = 'box';
		_t.collider.on('collided', _t._colliderHandler, this);
		super.onInited(args);
	}

}

G.Bullet = Bullet;

/**
	射线类子弹。
	激光、雷电等。
	几何体最好采用平面(Plane)且重复贴图。
*/
export class Ray extends Bullet {
	constructor(options) {
		super(options);

		this.isRay = true;
		this.type = 'Ray';
	}

	onInited(args) {

		super.onInited(args);

	}

	onChanged(name) {
		var _t = this;

		//将几何体拉伸长度，纹理进行重复，即可完成激光类效果
		switch (name) {

			case 'targets':
				var w = _t.distanceTo(_t.target);
				_t.unit.scale.x = w;
				break;
		}
	}

	onUpdated(args) {

		super.onUpdated(args);

	}
}
G.Ray = Ray;
/**
	导弹。
	平抛运动。
*/
export class Rocket extends Bullet {
	constructor(options) {
		super(options);

		this.isRocket = true;
		this.type = 'Rocket';

		this.moveType = 'custom';

		//原始位置
		this.pos = null;

		//假定都以45度抛出
		this.angle = 45;
		this.radian = J.um.getRadian(this.angle);
		//重力加速度
		this.g = 9.8;

		this.sin90 = Math.sin(J.um.getRadian(this.angle * 2));
		this.cos45 = Math.cos(this.radian);
		this.sin45 = Math.cos(this.radian);

		this.dx = 0;
		this.dy = 0;
		//初速度
		this.v0 = 0;

		this.x1 = 0;
		this.y1 = 0;
		this.x2 = 0;
		this.y2 = 0;

	}

	onInited(args) {

		super.onInited(args);

		var _t = this;
		_t.pos = _t.unit.position.clone();
		_t.elapsed = 0;
	}

	onChanged(name) {
		var _t = this;
		switch (name) {
			case 'targets':
				_t.x1 = _t.pos.x;
				_t.y1 = _t.pos.y;
				_t.x2 = _t.target.unit.position.x;
				_t.y2 = _t.target.unit.position.y;
				_t.dx = _t.x2 - _t.x1;
				_t.dy = _t.y2 - _t.y1;
				_t.v0 = Math.pow(_t.dx * _t.dx * _t.g / (_t.dx * _t.sin90 - 2 * _t.dy * _t.cos45 * _t.cos45), 1 / 2);
				_t.speed = _t.distanceTo(_t.target);
				break;
		}
	}

	onUpdated(args) {

		super.onUpdated(args);

		var _t = this;

		var x = _t.x1 + _t.v0 * _t.elapsed * _t.cos45;
		var y = _t.y1 + _t.v0 * _t.elapsed * _t.sin45 - _t.g * Math.pow(_t.elapsed, 2) / 2;

		_t.unit.position.setY(y);
		_t.approach(_t.target);

	}

}
G.Rocket = Rocket;
/**
 * 攻击信息
 */
export class AttackerInfo extends EventDispatcher {
	constructor(options) {
		super(options);

		this.isAttackerInfo = true;
		this.type = 'AttackerInfo';

		/**
		 * 本信息的名字
		 */
		this.name = J.def(this.options.name, "");

		/**
		 * 当前生命值。有些技能可以增加生命值，下面都是。
		 * @type {Number}
		 */
		this._hp = J.def(this.options.hp, 100);

		/**
		 * 回血速度， 每秒多少
		 */
		this._hpRecover = J.def(this.options.hpRecover, 1);
		/**
		 * 最大生命值
		 */
		this._maxhp = J.def(this.options.maxhp, 100);
		/**
		 * 当前魔法值
		 * @type {Number}
		 */
		this.mp = J.def(this.options.mp, 100);
		/**
		 * 回蓝速度， 每秒多少
		 */
		this.mpRecover = J.def(this.options.mpRecover, 1);
		/**
		 * 最大魔法值
		 */
		this.maxmp = J.def(this.options.maxmp, 100);

		/**
		 * 当前能量值
		 * @type {Number}
		 */
		this.ep = J.def(this.options.ep, 100);
		/**
		 * 回蓝速度， 每秒多少
		 */
		this.epRecover = J.def(this.options.epRecover, 1);
		/**
		 * 最大能量值
		 */
		this.maxep = J.def(this.options.maxep, 100);

		//移动速度（每秒走多远）
		this.speed = J.def(this.options.speed, 10);

		/**
		 * 物理攻击力
		 * @type {Number}
		 */
		this.physicalAttack = J.def(this.options.physicalAttack, 10);

		/**
		 * 物理穿透
		 */
		this.physicalPenetration = J.def(this.options.physicalPenetration, 0);

		/**
		 * 暴击率（百分比）
		 */
		this.criticalStrike = J.def(this.options.criticalStrike, 0);

		/**
		 * 暴击伤害（比例，默认一倍）
		 */
		this.criticalStrikeDamage = J.def(this.options.criticalStrikeDamage, 1);

		/**
		 * 攻击速度（每秒攻击多少次）
		 * @type {Number}
		 */
		this.attackSpeed = J.def(this.options.attackSpeed, 2);

		/**
		 * 魔法攻击力
		 */
		this.magicAttack = J.def(this.options.magicAttack, 10);

		/**
		 * 法术穿透
		 */
		this.magicPenetration = J.def(this.options.magicAttack, 0);

		/**
		 攻击范围
		 */
		this.attackRange = J.def(this.options.attackRange, 100);

		/**
		 * 冷却时间（秒）
		 */
		this.coolDown = J.def(this.options.ma, 10);

		/**
		 * 护甲（有些技能是可以提升护甲的）
		 */
		this.armor = J.def(this.options.armor, 10);

		/**
		 * 魔抗
		 */
		this.resistance = J.def(this.options.resistance, 10);

		/**
		 * 韧性
		 */
		this.tenacity = J.def(this.options.tenacity, 10);

		/**
		 * 子弹
		 */
		this.bullet = null;
		if (this.options.bullet) {
			this.bullet = RM.getItem('gameobject', 'uuid', this.options.bullet);
		}

		/**
		 * 当前已经发射出去的子弹
		 */
		this.bullets = [];

		/**
		 * 上次发射子弹经历时长。
		 * 给定这个初始值是防止第一枪打不出（参阅 Attacker.shoot()）。
		 */
		this.shotElapsed = 1 / this.attackSpeed + 0.1;

		/**
		 * 攻击类型：melee 近战，ranged 远程
		 */
		this.mode = J.def(this.options.mode, 'ranged');

		/**
		 * 指向型攻击，一定要有 target
		 */
		this.isDirected = J.def(this.options.isDirected, false);

		/**
		 * 是否自动攻击
		 */
		this.isAuto = J.def(this.options.isAuto, false);

		/**
		 * 攻击数量，默认只能攻击一个对象
		 */
		this.attackCount = J.def(this.options.attackCount, 1);

		/**
		 * 攻击范围指示器
		 */
		this.arBar = null;

		/**
		 * 是否继承 Actor 的面板数据。
		 * 是则不对面板数据做任何更改，否就是增减它们。
		 */
		this.isInherit = J.def(this.options.isInherit, true);
	}

	get hp() { return this._hp; }
	set hp(v) {
		if (this._hp == v) return;
		this._hp = v;
		this.change('hp')
	}

	get hpRecover() { return this._hpRecover; }
	set hpRecover(v) {
		if (this._hpRecover == v) return;
		this._hpRecover = v;
		this.change('hpRecover');
	}

	get maxhp() { return this._maxhp; }
	set maxhp(v) {
		if (this._maxhp == v) return;
		this._maxhp = v;
		this.change('maxhp');
	}

	//未完待续，其他属性一一填上...

}

G.AttackerInfo = AttackerInfo;
/**
 * 音频信息类，包裹了 THREE.Audio 对象，也是 RM 加载的对象。
 */
export class AudioInfo extends EventDispatcher {

    constructor(options) {
        super(options);

        /**
         * 名称
         */
        this._name = J.def(this.options.name, "");
        /**
         * 是否循环播放
         */
        this._loop = J.def(this.options.loop, false);
        /**
         * 是否自动播放
         */
        this._autoplay = J.def(this.options.autoplay, false);
        /**
         * 默认播放速度
         * 1.0 正常速度
         * 0.5 半速（更慢）
         * 2.0 倍速（更快）
         * -1.0 向后，正常速度
         * -0.5 向后，半速
         */
        this._playbackRate = J.def(this.options.playbackRate, 1);
        /**
         * 音频地址
         */
        this._url = J.def(this.options.url, '');
        /**
         * 声音大小
         */
        this._volume = J.def(this.options.volume, 1);
        /**
        * 声音类型。
        * 普通声音 normal，定位声音 positional
        */
        this._subType = J.def(this.options.subType, 'normal');

        /**
         * 音频对象，THREE.Audio 对象
         */
        this.audio = null;

    }

    /**
    * 名称
    */
    get name() {
        return this._name;
    }
    set name(v) {
        this._name = v;
        this.change('name');
    }
    /**
    * 自动播放
    */
    get autoplay() {
        return this._autoplay;
    }
    set autoplay(v) {
        this._autoplay = v;
        if (this.audio)
            this.audio.autoplay = v;
        this.change('autoplay');
    }

    /**
    * 循环播放否
    */
    get loop() {
        return this._loop;
    }
    set loop(v) {
        this._loop = v;
        if (this.audio)
            this.audio.setLoop(v);
        this.change('loop');
    }

    /**
    * 播放速度
    */
    get playbackRate() {
        return this._playbackRate;
    }
    set playbackRate(v) {
        this._playbackRate = v;
        if (this.audio)
            this.audio.setPlaybackRate(v);
        this.change('playbackRate');
    }

    /**
    * 视频来源
    */
    get url() {
        return this._url;
    }
    set url(v) {

        var _t = this;

        _t._url = v;

        _t.load(v);
        this.change('url');
    }

    /**
    * 音量
    */
    get volume() {
        return this._volume;
    }
    set volume(v) {
        this._volume = v;
        if (this.audio)
            this.audio.setVolume(v);
        this.change('volume')
    }

    /**
    * 加载
    * @url {string}	地址
    * @callback {function}	加载完后回调
    */
    load(url, callback) {

        var _t = this;
        _t._url = url;

        new THREE.AudioLoader().load(
            url,
            function (buffer) {

                _t.audio = _t._subType == 'normal' ?
                    new THREE.Audio(SM.audioListener) :
                    new THREE.PositionalAudio(SM.audioListener);

                _t.audio.autoplay = _t.autoplay;
                _t.audio.setBuffer(buffer);
                _t.audio.setLoop(_t.loop);
                _t.audio.setVolume(_t.volume);

                //播放完成
                _t.audio.onEnded = () => {
                    _t.audio.isPlaying = false;
                    _t.fire("played");
                }

                if (callback)
                    callback();
            }
        );
    }


    /**
   * 正在播放否
   */
    get isPlaying() {
        return this.audio && this.audio.isPlaying;
    }

    /**
    * 播放
    */
    play() {
        if (this.audio) {
            this.audio.play();
            this.fire("play");
        }
    }

    /**
    * 重播
    */
    replay() {

        if (this.audio) {
            if (this.isPlaying) {
                this.audio.stop();
                this.audio.play();
            } else {
                this.audio.play();
            }
        }
        this.fire("replay");
    }

    /**
    * 暂停
    */
    pause() {
        if (this.audio) {
            this.audio.pause();
            this.fire("pause");
        }
    }

    /**
    * 停止
    */
    stop() {
        if (this.isPlaying) {
            this.audio.stop();
            this.fire("stop");
        }
    }

}

G.AudioInfo = AudioInfo;
export class BaggerInfo extends EventDispatcher {
    constructor(options) {
        super(options);

        this.isBagger = true;
        this.type = 'Bagger';

        /**
         * 缩略图
         */
        this.thumb = J.def(this.options.thumb, '/libs/assets/placeholder.png');

        this.name = J.def(this.options.name, 'MyBaggerInfo');

        this.attackerInfo = J.def(this.options.attackerInfo, new AttackerInfo());
    }
}

G.BaggerInfo = BaggerInfo;
/**
 * 渐变纹理
 */
export class GradientTexture extends EventDispatcher {

    constructor(options) {

        super(options);

        this.isGradientTexture = true;
        this.type = 'GradientTexture';

        this.texture = new THREE.Texture(

            this.options.image, this.options.mapping,
            this.options.wrapS, this.options.wrapT,
            this.options.magFilter, this.options.minFilter,
            this.options.format, this.options.type,
            this.options.anisotropy, this.options.encoding

        );

        /**
         * 模式，支持 linear 和 radial
         */
        this.mode = J.def(this.options.mode, 'linear');

        this.width = J.def(this.options.width, 128);
        this.height = J.def(this.options.height, 128);

        this.x1 = J.def(this.options.x1, 64);
        this.y1 = J.def(this.options.x1, 0);
        this.x2 = J.def(this.options.x1, 64);
        this.y2 = J.def(this.options.x1, 128);
        this.r1 = J.def(this.options.x1, 8);
        this.r2 = J.def(this.options.x1, 64);

        this.colorStops = J.def(this.options.colorStops, [
            { position: 0, color: '#900' },
            { position: 1, color: '#000' }
        ]);

        this.canvas = canvasUtils.createCanvas(this.width, this.height);

        var grd = {};
        if (this.mode == 'linear') {
            grd = this.canvas.ctx.createLinearGradient(this.x1, this.y1, this.x2, this.y2);
        } else {
            grd = this.canvas.ctx.createRadialGradient(this.x1, this.y1, this.r1, this.x2, this.y2, this.r2);
        }
        this.colorStops.forEach((item) => {
            grd.addColorStop(item.position, item.color);
        });

        canvasUtils.fullfill(this.canvas.ctx, grd);

        this.texture.image = this.canvas;
        this.texture.needsUpdate = true;
    }


};

G.GradientTexture = GradientTexture;
/**
 * 3D 文本。
 * 在设置位置的时候要注意，x=-0.5 * this.textWidth，x 是几何体的左边
 */
export class Label3D extends GObject {

    constructor(options) {
        super(options);

        this.isLabel3D = true;
        this.type = 'Label3D';

        this._text = J.def(this.options.text, 'HELLO');

        this._textWidth = 10;

        this.fontFamily = '方正黑体';

        this._texture = J.def(this.options.texture, {
            face: "/libs/assets/steel2.jpg",
            faceTexture: null,
            side: "/libs/assets/steel2.jpg",
            sideTexture: null,
        });

        this._parameters = J.def(this.options.parameters, {
            size: 12,
            height: 2,
            curveSegments: 3,
            bevelThickness: 2,
            bevelSize: 1,
            bevelEnabled: true,
            bevelSegments: 2
        });

    }

    create(args) {

        super.onCreated(args);
        this._createText();
    }

    _createText() {

        var _t = this;

        var async = new Async(
            [

                (url) => {
                    window.threejsUtils.loadTexture(url, (map) => {
                        _t._texture.faceTexture = map;
                        _t._texture.faceTexture.needsUpdate = true;

                        async.next();
                    });
                },

                (url) => {
                    window.threejsUtils.loadTexture(url, (map) => {
                        _t._texture.sideTexture = map;
                        _t._texture.sideTexture.needsUpdate = true;

                        async.next();
                    });
                },

                () => {

                    Object.assign(_t._parameters, {
                        material: 0,
                        extrudeMaterial: 1
                    });

                    var geo = new THREE.TextGeometry(
                        _t._text,
                        _t._parameters
                    );

                    var marr = [
                        new THREE.MeshPhongMaterial({ color: 0xffffff, flatShading: true, map: _t._texture.faceTexture }),
                        new THREE.MeshPhongMaterial({ color: 0xffffff, map: _t._texture.sideTexture })
                    ];

                    _t.unit = new THREE.Mesh(geo, marr);

                    geo.computeBoundingBox();
                    geo.computeVertexNormals();

                    _t._textWidth = geo.boundingBox.max.x - geo.boundingBox.min.x;

                    _t.fire('created');
                }

            ],
            [
                _t._texture.face,
                _t._texture.side
            ],
            _t
        );

        async.first();

    }

    get textWidth() { return this._textWidth; }

    get text() { return this._text; }

    set text(v) {
        this._text = v;
        this._createText();
        this.change('text');
    }

    get texture() { return this._texture; }

    set texture(v) {
        this._texture = v;
        this._createText();
        this.change('texture');
    }

    get parameters() { return this._parameters; }

    set parameters(v) {
        this._parameters = v;
        this._createText();
        this.change('parameters');
    }

}

G.Label3D = Label3D;
/**
 * 寻路器，一个舞台对应一个寻路器。
 * 每个物体自己的移动采用 Mover 来实现。
 * @class
 * @param  {Object} options
 */
export class Pathor extends FrameObject {

	constructor(options) {
		super(options);

		this.isPathor = true;
		this.gtype = 'Pathor';
		this.type = 'Mesh';

		/**
		 * 网格模型。
		 * 包含 grid 和 nodes 两个对象。
		 * grid 二维数组，如 20*20，
		 * nodes 一维数组，如 400，
		 * 但里面都是存储 GridNode 对象，为了方便计算。
		 * @type {Graph}
		 */
		this.graph = {};

		/**
		 * 地图大小
		 * @type {Object}
		 */
		this.mapSize = J.val(this.opts, 'mapSize', {
			width: 1000,
			height: 1000
		});
		/**
		 * 网格分片
		 * @type {Object}
		 */
		this.gridSize = J.val(this.opts, 'gridSize', {
			rows: 20,
			cols: 20
		});
		/**
		 * A* 算法选项
		 * @type {Object}
		 */
		this.searchOptions = J.val(this.opts, 'searchOptions', {});

		this.mw = this.mapSize.width;
		this.mh = this.mapSize.height;

		this.gw = this.gridSize.rows;
		this.gh = this.gridSize.cols;

		/**
		 * 障碍物，二维数组，0 为不可通过，1 为可通过
		 */
		this._barriers = new Array(this.gw);
		for (let i = 0; i < this.gw; i++) {
			this._barriers[i] = new Array(this.gh);
			for (let j = 0; j < this.gh; j++) {
				this._barriers[i][j] = 0;
			}
		}

		//每片大小
		this.dw = this.mw / this.gw;
		this.dh = this.mh / this.gh;

		//地图一半大小
		this.mhw = this.mw / 2;
		this.mhh = this.mh / 2;

	}

	get barriers() {
		return this._barriers;
	}

	set barriers(v) {
		this._barriers = v;
		this.graph = new Graph(this._barriers);

		var geometry = new THREE.PlaneGeometry(this.mw, this.mh, this.gw, this.gh);

		geometry.rotateX(-Math.PI / 2);

		//如果是20*20，则 faces 有 800 个，一行行排列
		for (var i = 0, j = 0; i < geometry.faces.length; i += 2, j++) {
			var row = ~~(j / this.gw);
			var col = j - row * this.gw;
			var color = this._barriers[row][col] == 1 ? 0x999900 : 0x000000;
			geometry.faces[i].color.setHex(color);
			geometry.faces[i + 1].color.setHex(color);
		}

		var material = new THREE.MeshLambertMaterial({
			vertexColors: THREE.FaceColors,
			wireframe: true
		});

		this.unit = new THREE.Mesh(geometry, material);
		this.geometry = geometry;
		this.material = material;
		this.position.y = 10;
		this.change('barriers');
	}

	/**
	 * 创建寻路所需网格模型
	 */
	onInited(args) {

		super.onInited(args);

	}

	/**
	 * 搜索路径。
	 * 注意三种坐标系：
	 * 1.寻路器网格坐标（行列坐标），一个平面分为 gridSize*gridSize 分片，形成的行列，左上角00
	 * 2.寻路器世界坐标，与网格坐标一致，也是左上角00，但大小变为了世界坐标的大小单位，不再是行列
	 * 3.世界坐标，THREE.js世界坐标，舞台中心为00
	 * @param  {Vector2|Vector3} start 开始点（世界坐标，坐标中心为0,0）
	 * @param  {Vector2|Vector3} end   结束点（世界坐标）
	 * @return {Array[GridNode]}       可行走路径（世界坐标）
	 */
	search(start, end) {

		var _t = this;

		//先将坐标转换为寻路器世界坐标
		start.x += _t.mhw;
		start.y += _t.mhh;
		end.x += _t.mhw;
		end.y += _t.mhh;

		//归一为寻路器网格坐标
		var sx = parseInt(start.x / _t.dw);
		if (sx >= _t.gw) sx = gw - 1;
		var sy = parseInt(start.y / _t.dh);
		if (sy >= _t.gh) sy = gh - 1;
		var ex = parseInt(end.x / _t.dw);
		if (ex >= _t.gw) ex = gw = 1;
		var ey = parseInt(end.y / _t.dh);
		if (ey >= _t.gh) ey = gh - 1;

		var s = _t.graph.grid[sx][sy];
		var e = _t.graph.grid[ex][ey];

		//用网格坐标寻路，得出结果也是网格坐标
		var paths = astar.search(_t.graph, s, e, _t.searchOptions);

		//绘制辅助
		if (_t.gizmos) {
			var color = 0x009900;
			for (var i = 0; i < paths.length; i++) {
				var j = paths[i].x * _t.gw * 2 + paths[i].y * 2;
				_t.unit.geometry.faces[j].color.setHex(color);
				_t.unit.geometry.faces[j + 1].color.setHex(color);
			}
		}

		for (var i = 0; i < paths.length; i++) {
			//克隆一份，不然有可能修改 grid 里面的 GridNode
			paths[i] = {
				x: paths[i].x,
				y: paths[i].y
			};

			//网格坐标转为寻路器世界坐标
			paths[i].x = paths[i].x * _t.dw + _t.dw / 2;
			paths[i].y = paths[i].y * _t.dh + _t.dh / 2;

			//寻路器世界坐标转化为真正世界坐标
			paths[i].x -= _t.mhw;
			paths[i].y -= _t.mhh;
		}
		return paths;
	}

	onUpdated(args) {

		super.onUpdated(args);

	}

}

G.Pathor = Pathor;
/**
 * 占位器。
 * 仅在设计视图显示，程序运行时不显示。
 * 在 map.json 中，放在 decorations 中。
 */
export class PlaceHolder extends FrameObject {

	constructor(options) {

		super(options);
		this.isPlaceHolder = true;
		this.gtype = 'PlaceHolder';
		this.type = 'Mesh';

	}

	onInited(args) {

		var _t = this;

		var geometry = new THREE.PlaneGeometry(10, 10, 1, 1);
		geometry.rotateX(-Math.PI / 2);

		new Loader().load('/libs/assets/placeholder.png', (map) => {

			var material = new THREE.MeshPhongMaterial({
				map: map
			});

			this.geometry = geometry;
			this.material = material;

			super.onInited(args);

		});


	}

	onUpdated(args) {

		super.onUpdated(args);

	}

}

G.PlaceHolder = PlaceHolder;
/**
 * 样式相关
 * 
 * @link https://www.runoob.com/tags/html-colorname.html
 * @class Style
 * @extends {EventDispatcher}
 */
export class Style extends EventDispatcher {

    constructor(options) {
        super(options);

        this.isStyle = true;
        this.type = 'Style';
    }


    /**
     * 渐变纹理
     */
    static get steelBlueTextureOfButton() {
        var gt = new GradientTexture({
            colorStops: [

                { position: 0, color: '#00BFFF' },
                { position: 0.3, color: '#4682B4' },
                { position: 0.7, color: '#4682B4' },
                { position: 1, color: '#00BFFF' }

            ]
        });
        return gt.texture;
    }

    static get orangeTextureOfButton() {
        var gt = new GradientTexture({
            colorStops: [

                { position: 0, color: '#FFD700' },
                { position: 0.3, color: '#FFA500' },
                { position: 0.7, color: '#FFA500' },
                { position: 1, color: '#FFD700' }

            ]
        });
        return gt.texture;
    }

    static get steelBlueTextureOfDialog() {
        var gt = new GradientTexture({
            colorStops: [

                { position: 0, color: '#2d4d86' },
                { position: 1, color: '#192c4d' }

            ]
        });
        return gt.texture;
    }

    static get steelBlueTextureOfTitleBar() {
        var gt = new GradientTexture({
            colorStops: [

                { position: 0, color: '#437EB4' },
                { position: 1, color: '#19344d' }

            ]
        });
        return gt.texture;
    }
}

G.Style = Style;

/**
 * 颜色代码
 */
Style.AliceBlue = "#F0F8FF";
Style.AntiqueWhite = "#FAEBD7";
Style.Aqua = "#00FFFF";
Style.Aquamarine = "#7FFFD4";
Style.Azure = "#F0FFFF";
Style.Beige = "#F5F5DC";
Style.Bisque = "#FFE4C4";
Style.Black = "#000000";
Style.BlanchedAlmond = "#FFEBCD";
Style.Blue = "#0000FF";
Style.BlueViolet = "#8A2BE2";
Style.Brown = "#A52A2A";
Style.BurlyWood = "#DEB887";
Style.CadetBlue = "#5F9EA0";
Style.Chartreuse = "#7FFF00";
Style.Chocolate = "#D2691E";
Style.Coral = "#FF7F50";
Style.CornflowerBlue = "#6495ED";
Style.Cornsilk = "#FFF8DC";
Style.Crimson = "#DC143C";
Style.Cyan = "#00FFFF";
Style.DarkBlue = "#00008B";
Style.DarkCyan = "#008B8B";
Style.DarkGoldenRod = "#B8860B";
Style.DarkGray = "#A9A9A9";
Style.DarkGreen = "#006400";
Style.DarkKhaki = "#BDB76B";
Style.DarkMagenta = "#8B008B";
Style.DarkOliveGreen = "#556B2F";
Style.DarkOrange = "#FF8C00";
Style.DarkOrchid = "#9932CC";
Style.DarkRed = "#8B0000";
Style.DarkSalmon = "#E9967A";
Style.DarkSeaGreen = "#8FBC8F";
Style.DarkSlateBlue = "#483D8B";
Style.DarkSlateGray = "#2F4F4F";
Style.DarkTurquoise = "#00CED1";
Style.DarkViolet = "#9400D3";
Style.DeepPink = "#FF1493";
Style.DeepSkyBlue = "#00BFFF";
Style.DimGray = "#696969";
Style.DodgerBlue = "#1E90FF";
Style.FireBrick = "#B22222";
Style.FloralWhite = "#FFFAF0";
Style.ForestGreen = "#228B22";
Style.Fuchsia = "#FF00FF";
Style.Gainsboro = "#DCDCDC";
Style.GhostWhite = "#F8F8FF";
Style.Gold = "#FFD700";
Style.GoldenRod = "#DAA520";
Style.Gray = "#808080";
Style.Green = "#008000";
Style.GreenYellow = "#ADFF2F";
Style.HoneyDew = "#F0FFF0";
Style.HotPink = "#FF69B4";
Style.IndianRed = "#CD5C5C";
Style.Indigo = "#4B0082";
Style.Ivory = "#FFFFF0";
Style.Khaki = "#F0E68C";
Style.Lavender = "#E6E6FA";
Style.LavenderBlush = "#FFF0F5";
Style.LawnGreen = "#7CFC00";
Style.LemonChiffon = "#FFFACD";
Style.LightBlue = "#ADD8E6";
Style.LightCoral = "#F08080";
Style.LightCyan = "#E0FFFF";
Style.LightGoldenRodYellow = "#FAFAD2";
Style.LightGray = "#D3D3D3";
Style.LightGreen = "#90EE90";
Style.LightPink = "#FFB6C1";
Style.LightSalmon = "#FFA07A";
Style.LightSeaGreen = "#20B2AA";
Style.LightSkyBlue = "#87CEFA";
Style.LightSlateGray = "#778899";
Style.LightSteelBlue = "#B0C4DE";
Style.LightYellow = "#FFFFE0";
Style.Lime = "#00FF00";
Style.LimeGreen = "#32CD32";
Style.Linen = "#FAF0E6";
Style.Magenta = "#FF00FF";
Style.Maroon = "#800000";
Style.MediumAquaMarine = "#66CDAA";
Style.MediumBlue = "#0000CD";
Style.MediumOrchid = "#BA55D3";
Style.MediumPurple = "#9370DB";
Style.MediumSeaGreen = "#3CB371";
Style.MediumSlateBlue = "#7B68EE";
Style.MediumSpringGreen = "#00FA9A";
Style.MediumTurquoise = "#48D1CC";
Style.MediumVioletRed = "#C71585";
Style.MidnightBlue = "#191970";
Style.MintCream = "#F5FFFA";
Style.MistyRose = "#FFE4E1";
Style.Moccasin = "#FFE4B5";
Style.NavajoWhite = "#FFDEAD";
Style.Navy = "#000080";
Style.OldLace = "#FDF5E6";
Style.Olive = "#808000";
Style.OliveDrab = "#6B8E23";
Style.Orange = "#FFA500";
Style.OrangeRed = "#FF4500";
Style.Orchid = "#DA70D6";
Style.PaleGoldenRod = "#EEE8AA";
Style.PaleGreen = "#98FB98";
Style.PaleTurquoise = "#AFEEEE";
Style.PaleVioletRed = "#DB7093";
Style.PapayaWhip = "#FFEFD5";
Style.PeachPuff = "#FFDAB9";
Style.Peru = "#CD853F";
Style.Pink = "#FFC0CB";
Style.Plum = "#DDA0DD";
Style.PowderBlue = "#B0E0E6";
Style.Purple = "#800080";
Style.Red = "#FF0000";
Style.RosyBrown = "#BC8F8F";
Style.RoyalBlue = "#4169E1";
Style.SaddleBrown = "#8B4513";
Style.Salmon = "#FA8072";
Style.SandyBrown = "#F4A460";
Style.SeaGreen = "#2E8B57";
Style.SeaShell = "#FFF5EE";
Style.Sienna = "#A0522D";
Style.Silver = "#C0C0C0";
Style.SkyBlue = "#87CEEB";
Style.SlateBlue = "#6A5ACD";
Style.SlateGray = "#708090";
Style.Snow = "#FFFAFA";
Style.SpringGreen = "#00FF7F";
Style.SteelBlue = "#4682B4";
Style.Tan = "#D2B48C";
Style.Teal = "#008080";
Style.Thistle = "#D8BFD8";
Style.Tomato = "#FF6347";
Style.Turquoise = "#40E0D0";
Style.Violet = "#EE82EE";
Style.Wheat = "#F5DEB3";
Style.White = "#FFFFFF";
Style.WhiteSmoke = "#F5F5F5";
Style.Yellow = "#FFFF00";
Style.YellowGreen = "#9ACD32";

/**
 * 透明度代码
 */
Style.A100 = 'FF';
Style.A99 = 'FC';
Style.A98 = 'FA';
Style.A97 = 'F7';
Style.A96 = 'F5';
Style.A95 = 'F2';
Style.A94 = 'F0';
Style.A93 = 'ED';
Style.A92 = 'EB';
Style.A91 = 'E8';
Style.A90 = 'E6';
Style.A89 = 'E3';
Style.A88 = 'E0';
Style.A87 = 'DE';
Style.A86 = 'DB';
Style.A85 = 'D9';
Style.A84 = 'D6';
Style.A83 = 'D4';
Style.A82 = 'D1';
Style.A81 = 'CF';
Style.A80 = 'CC';
Style.A79 = 'C9';
Style.A78 = 'C7';
Style.A77 = 'C4';
Style.A76 = 'C2';
Style.A75 = 'BF';
Style.A74 = 'BD';
Style.A73 = 'BA';
Style.A72 = 'B8';
Style.A71 = 'B5';
Style.A70 = 'B3';
Style.A69 = 'B0';
Style.A68 = 'AD';
Style.A67 = 'AB';
Style.A66 = 'A8';
Style.A65 = 'A6';
Style.A64 = 'A3';
Style.A63 = 'A1';
Style.A62 = '9E';
Style.A61 = '9C';
Style.A60 = '99';
Style.A59 = '96';
Style.A58 = '94';
Style.A57 = '91';
Style.A56 = '8F';
Style.A55 = '8C';
Style.A54 = '8A';
Style.A53 = '87';
Style.A52 = '85';
Style.A51 = '82';
Style.A50 = '80';
Style.A49 = '7D';
Style.A48 = '7A';
Style.A47 = '78';
Style.A46 = '75';
Style.A45 = '73';
Style.A44 = '70';
Style.A43 = '6E';
Style.A42 = '6B';
Style.A41 = '69';
Style.A40 = '66';
Style.A39 = '63';
Style.A38 = '61';
Style.A37 = '5E';
Style.A36 = '5C';
Style.A35 = '59';
Style.A34 = '57';
Style.A33 = '54';
Style.A32 = '52';
Style.A31 = '4F';
Style.A30 = '4D';
Style.A29 = '4A';
Style.A28 = '47';
Style.A27 = '45';
Style.A26 = '42';
Style.A25 = '40';
Style.A24 = '3D';
Style.A23 = '3B';
Style.A22 = '38';
Style.A21 = '36';
Style.A20 = '33';
Style.A19 = '30';
Style.A18 = '2E';
Style.A17 = '2B';
Style.A16 = '29';
Style.A15 = '26';
Style.A14 = '24';
Style.A13 = '21';
Style.A12 = '1F';
Style.A11 = '1C';
Style.A10 = '1A';
Style.A9 = '17';
Style.A8 = '14';
Style.A7 = '12';
Style.A6 = '0F';
Style.A5 = '0D';
Style.A4 = '0A';
Style.A3 = '08';
Style.A2 = '05';
Style.A1 = '03';
Style.A0 = '00';

Style.Transparent = '#00000000';


/**
 * 缩写
 */

Style.Fill = {
    left: 0,
    right: 0,
    top: 0,
    bottom: 0
};

/**
 * 扩展版颜色，支持 opacity。
 * 暂时只支持 #HEX 颜色模式，如 #FFF 或 #FFFFFF。
 * 最后一位或两位就是透明度，FF 表示不透明，00 表示全透明。
 */
export class TrueColor extends EventDispatcher {

    constructor(options) {

        super(options);
        this.isTrueColor = true;
        this.type = "TrueColor";

        var clr = J.uc.toHEX(this.options);

        //#HEX 形式且带透明度的颜色
        if (clr.charAt(0) != '#') {
            console.warn('your pattern is not supported, use HEX string instead.')
            return;
        }

        var len = clr.length;
        //将缩写补齐为全写
        if (len == 4 || len == 5) {
            clr = '#' + clr.charAt(1) + clr.charAt(1) +
                clr.charAt(2) + clr.charAt(2) +
                clr.charAt(3) + clr.charAt(3) +
                (len == 5 ? clr.charAt(4) + clr.charAt(4) : '');
        }
        len = clr.length;

        /**
         * 是否带有透明度
         */
        this.hasAlpha = (len == 9);

        this.opacity = this.hasAlpha ? J.un.hex2dec(clr.substr(7)) / 255 : 1;

        this.color = new THREE.Color(clr.substr(0, 7));
    }

}

G.TrueColor = TrueColor;
export class UpgraderInfo extends EventDispatcher {
    constructor(options) {
        super(options);

        this.isUpgraderInfo = true;
        this.type = 'UpgraderInfo';

        /**
         * 当前等级
         */
        this.level = J.def(this.options.level, 0);

        /**
         * 达到等级所需经验值
         */
        this.exp = J.def(this.options.exp, 0);

        /**
         * 等级对应的信息
         */
        this.attackerInfo = J.def(this.options.attackerInfo, new AttackerInfo());
    }
}

G.UpgraderInfo = UpgraderInfo;
/**
 * 视频信息
 */
export class VideoInfo extends EventDispatcher {

    constructor(options) {

        super(options);

        /**
         * 名称
         */
        this._name = J.def(this.options.name, "");
        /**
         * 是否循环播放
         */
        this._loop = J.def(this.options.loop, false);
        /**
         * 声音大小
         */
        this._volume = J.def(this.options.volume, 0.5);
        /**
         * 视频地址（在 HTML5 中是视频元素 id）
         * 这里要分别对微信和H5做兼容
         */
        this._url = J.def(this.options.url, "");
        /**
         * 视频对象
         */
        this._createVideo();

    }

    /**
    * 创建视频对象
    */
    _createVideo() {

        var _t = this;

        if (J.udt.isMiniP) {
            _t.video = new Video(_t.url);
        } else {
            _t.video = document.createElement("video");
            _t.video.setAttribute("src", _t.url)
        }
        //播放完成（循环播放时不会触发）
        _t.video.onended = () => {
            _t.fire("played");
        }

    };
    /**
    * 创建视频纹理
    */
    _createVideoTexture() {

        var _t = this;

        if (J.isN(_t._videoTexture)) {
            _t._videoTexture = new THREE.VideoTexture(_t.video);
            _t._videoTexture.wrapS = _t._videoTexture.wrapT = THREE.ClampToEdgeWrapping;
            _t._videoTexture.minFilter = THREE.LinearFilter;
            _t._videoTexture.magFilter = THREE.LinearFilter;
            _t._videoTexture.format = THREE.RGBFormat;
        }

    };

    /**
    * 唯一码
    */
    get uuid() {
        return this._uuid;
    }
    /**
    * 名称
    */
    get name() {
        return this._name;
    }
    set name(v) {
        this._name = v;
        this.change('name');
    }
    /**
    * 循环播放否
    */
    get loop() {
        return this._loop;
    }
    set loop(v) {
        this._loop = v;
        if (this.video)
            this.video.setLoop(v);
        this.change('loop');
    }

    /**
    * 音量
    */
    get volume() {
        return this._volume;
    }
    set volume(v) {
        this._volume = v;
        if (this.video)
            this.video.setVolume(v);
        this.change('volume')
    }

    /**
    * 地址
    */
    get url() {
        return this._url;
    }
    set url(v) {

        var _t = this;

        _t._url = v;
        _t._createVideo();

        this.change('url');
    }

    /**
    * 视频对象
    */
    get video() {

        return this._video;

    }
    set video(v) {

        this._video = v;
        this.change('video');

    }

    /**
    * 视频纹理
    */
    get videoTexture() {

        return this._videoTexture;

    }

    /**
    * 正在播放否
    */
    get isPlaying() {

        //video 没有 isPlaying 属性， 只能通过 paused 来判断
        return this.video && !this.video.paused;

    }
    /**
    * 播放
    */
    play() {

        if (this.video) {
            this.video.play();
            this.fire("play");
        }

    }
    /**
    * 重播
    */
    replay() {

        if (this.isPlaying) {
            this.stop();
            this.play();
        } else {
            this.play();
        }
        this.fire("replay");

    }
    /**
    * 暂停
    */
    pause() {

        if (this.video) {
            this.video.pause();
            this.fire("pause");
        }
    }
    /**
    * 停止
    */
    stop() {

        if (this.video && this.isPlaying) {
            this.video.stop();
            this.fire("stop");
        }
    }
    /**
     * 附加到哪个物体上
     */
    attach(gobj) {

        var _t = this;

        //创建纹理
        _t._createVideoTexture();

        if (gobj instanceof UIObject)
            gobj.unit.material = new THREE.SpriteMaterial({ map: _t.videoTexture });
        else if (gobj instanceof GObject)
            gobj.unit.material = new THREE.MeshBasicMaterial({ map: _t.videoTexture });

        gobj.unit.material.needsUpdate = true;

    }

}

G.VideoInfo = VideoInfo;
/**
 * UI 主题基类
 */
export class Theme extends EventDispatcher {

    constructor(options) {
        super(options);

        this.isTheme = true;
        this.type = 'Theme';
    }

    get skin() {
        return new Skin(this.options.skin);
    }

    get boxSkin() {
        return new BoxSkin(this.options.boxSkin);
    }

    get hboxSkin() {
        return new HBoxSkin(this.options.hboxSkin);
    }

    get vboxSkin() {
        return new VBoxSkin(this.options.vboxSkin);
    }

    get frameSkin() {
        return new FrameSkin(this.options.frameSkin);
    }
    get pageSkin() {
        return new PageSkin(this.options.pageSkin);
    }

    /**
     * 普通按钮
     */
    get buttonSkin() {
        return new ButtonSkin(this.options.buttonSkin);
    }

    get onlyIconButtonSkin() {
        return new OnlyIconButtonSkin(this.options.onlyIconButtonSkin);
    }

    /**
     * 主按钮
     */
    get firstButtonSkin() {
        return new FirstButtonSkin(this.options.firstButtonSkin);
    }

    /**
     * 次级按钮
     */
    get secondButtonSkin() {
        return new SecondButtonSkin(this.options.secondButtonSkin);
    }

    get labelSkin() {
        return new LabelSkin(this.options.labelSkin);
    }

    get textinputSkin() {
        return new TextinputSkin(this.options.textinputSkin);
    }

    /**
     * 对话框底面
     */
    get dialogSkin() {
        return new DialogSkin(this.options.dialogSkin);
    }

}

G.Theme = Theme;
/**
 * 皮肤基类
 */
export class Skin extends EventDispatcher {

    constructor(options) {

        super(options);

        this.isSkin = true;
        this.type = "Skin";

        this.width = J.def(this.options.width, undefined);
        this.height = J.def(this.options.height, undefined);
        this.anchorX = J.def(this.options.anchorX, undefined);
        this.anchorY = J.def(this.options.anchorY, undefined);
        this.margin = J.def(this.options.margin, 0);
        this.padding = J.def(this.options.padding, 0);
        this.left = J.def(this.options.left, undefined);
        this.right = J.def(this.options.right, undefined);
        this.top = J.def(this.options.top, undefined);
        this.bottom = J.def(this.options.bottom, undefined);
        this.centerX = J.def(this.options.centerX, undefined);
        this.centerY = J.def(this.options.centerY, undefined);

    }
}

G.Skin = Skin;
/**
 * 盒子皮肤基类
 */
export class BoxSkin extends Skin {

    constructor(options) {

        super(options);

        this.isBoxSkin = true;
        this.type = "BoxSkin";


        this.backgroundColor = J.def(this.options.backgroundColor, '#ffffff00');
        this.backgroundImage = J.def(this.options.backgroundImage, "");
        this.backgroundImageRepeat = J.def(this.options.backgroundImageRepeat, {
            x: {
                type: 'stretch',
                count: 1
            },
            y: {
                type: 'stretch',
                count: 1
            }
        });
        this.backgroundImageGrid9 = J.def(this.options.backgroundImageGrid9, {
            top: 0,
            left: 0,
            right: 0,
            bottom: 0
        });
        this.overflow = J.def(this.options.overflow, 'hidden');

    }
}

G.BoxSkin = BoxSkin;

/**
 * 水平盒子皮肤
 */
export class HBoxSkin extends BoxSkin {

    constructor(options) {

        super(options);

        this.isHBoxSkin = true;
        this.type = "HBoxSkin";

        this.overflowX = J.def(this.options.overflowX, 'hidden');
        this.overflowY = J.def(this.options.overflowY, 'stretch');

    }
}

G.HBoxSkin = HBoxSkin;

/**
 * 垂直盒子皮肤
 */
export class VBoxSkin extends BoxSkin {

    constructor(options) {

        super(options);

        this.isVBoxSkin = true;
        this.type = "VBoxSkin";

        this.overflowX = J.def(this.options.overflowX, 'stretch');
        this.overflowY = J.def(this.options.overflowY, 'hidden');

    }
}

G.VBoxSkin = VBoxSkin;
/**
 * 按钮皮肤
 */
export class ButtonSkin extends BoxSkin {

    constructor(options) {

        super(options);

        this.isButtonSkin = true;
        this.type = "ButtonSkin";

        this.width = J.def(this.options.width, 120);
        this.height = J.def(this.options.height, 30);
        this.subType = J.def(this.options.subType, 'text');
        this.backgroundColor = J.def(this.options.backgroundColor, '#ffffff');
        //默认背景图是蓝色的，所以用白色的字，背景颜色已经被背景图盖住了
        this.color = J.def(this.options.color, '#ffffff');
        this.font = J.def(this.options.font, '14px sans-serif');
        this.backgroundImage = J.def(this.options.backgroundImage, '/libs/assets/button.png');
    }
}

G.ButtonSkin = ButtonSkin;

export class OnlyIconButtonSkin extends BoxSkin {

    constructor(options) {

        super(options);

        this.isOnlyIconButtonSkin = true;
        this.type = "OnlyIconButtonSkin";

        this.width = J.def(this.options.width, 32);
        this.height = J.def(this.options.height, 32);
        this.subType = J.def(this.options.subType, 'onlyIcon');
        this.margin = J.def(this.options.margin, 4);
    }
}

G.OnlyIconButtonSkin = OnlyIconButtonSkin;

export class FirstButtonSkin extends ButtonSkin {

    constructor(options) {

        super(options);

        this.isFirstButtonSkin = true;
        this.type = "FirstButtonSkin";

        this.backgroundImage = J.def(this.options.backgroundImage, MM.domain + '/libs/assets/firstButton.png');
    }
}

G.FirstButtonSkin = FirstButtonSkin;

export class SecondButtonSkin extends ButtonSkin {

    constructor(options) {

        super(options);

        this.isSecondButtonSkin = true;
        this.type = "SecondButtonSkin";

        this.backgroundImage = J.def(this.options.backgroundImage, MM.domain + '/libs/assets/secondButton.png');
    }
}

G.SecondButtonSkin = SecondButtonSkin;


/**
 * 对话框皮肤
 */
export class DialogSkin extends BoxSkin {

    constructor(options) {

        super(options);

        this.isDialogSkin = true;
        this.type = "DialogSkin";

        this.width = J.def(this.options.width, 300);
        this.height = J.def(this.options.height, 200);
        this.centerX = J.def(this.options.centerX, 0);
        this.centerY = J.def(this.options.centerY, 0);
        this.z = J.def(this.options.z, 80);
        this.backgroundColor = J.def(this.options.backgroundColor, Style.steelBlueTextureOfDialog);

        this.titleBarSkin = new BoxSkin({
            left: 0,
            right: 0,
            top: 0,
            overflow: 'hidden',
            height: 40,
            backgroundColor: '#ffffff',
            backgroundImage: Style.steelBlueTextureOfTitleBar,
            labelSkin: new LabelSkin({
                color: "#ffffff",
                left: 10,
                top: 0,
                height: 40
            }),
            closeButtonSkin: new OnlyIconButtonSkin({
                icon: "/libs/assets/close.png",
                right: 4,
                top: 4
            })
        });

        this.contentBarSkin = new HBoxSkin({
            left: 0,
            right: 0,
            top: 40,
            bottom: 40,
            labelSkin: new LabelSkin({
                color: "#ffffff"
            })
        });

        this.bottomBarSkin = new HBoxSkin({
            centerX: 0,
            bottom: 10,
            height: 40,
            overflow: 'hidden',
            labelSkin: new LabelSkin({
                color: "#ffffff"
            })
        });
    }
}

G.DialogSkin = DialogSkin;
/**
 * frame 皮肤
 */
export class FrameSkin extends BoxSkin {

    constructor(options) {

        super(options);

        this.isFrameSkin = true;
        this.type = "FrameSkin";

        this.left = J.def(this.options.left, 0);
        this.right = J.def(this.options.right, 0);
        this.top = J.def(this.options.top, 0);
        this.bottom = J.def(this.options.bottom, 0);
    }
}

G.FrameSkin = FrameSkin;

/**
 * page 皮肤
 */
export class PageSkin extends BoxSkin {

    constructor(options) {

        super(options);

        this.isPageSkin = true;
        this.type = "PageSkin";

        this.left = J.def(this.options.left, 0);
        this.right = J.def(this.options.right, 0);
        this.top = J.def(this.options.top, 0);
        this.bottom = J.def(this.options.bottom, 0);

        this.overflow = J.def(this.options.overflow, 'hidden');

        this.titleBarSkin = new BoxSkin({
            left: 0,
            right: 0,
            top: 0,
            overflow: 'hidden',
            height: 40,
            backgroundColor: '#168dc5',
            labelSkin: new LabelSkin({
                color: "#ffffff"
            })
        });

        this.contentBarSkin = new BoxSkin({
            left: 0,
            right: 0,
            top: 40,
            bottom: 60,
            overflow: 'hidden',
            backgroundColor: '#000000'
        });

        this.bottomBarSkin = new BoxSkin({
            left: 0,
            right: 0,
            bottom: 0,
            height: 60,
            overflow: 'hidden',
            backgroundColor: '#168dc5',
            labelSkin: new LabelSkin({
                color: "#ffffff"
            })
        });
    }
}

G.PageSkin = PageSkin;
/**
 * 标签皮肤
 */
export class LabelSkin extends Skin {

    constructor(options) {

        super(options);

        this.isLabelSkin = true;
        this.type = "LabelSkin";

        this.backgroundColor = J.def(this.options.backgroundColor, '#ffffff00');
        this.color = J.def(this.options.color, '#000000');
        this.font = J.def(this.options.font, '14px sans-serif');
        this.align = J.def(this.options.align, 'center');
    }
}

G.LabelSkin = LabelSkin;

export class TextInputSkin extends BoxSkin {

    constructor(options) {

        super(options);

        this.isTextInputSkin = true;
        this.type = "TextInputSkin";

        this.width = J.def(this.options.width, 120);
        this.height = J.def(this.options.height, 30);
        this.overflow = J.def(this.options.overflow, 'hidden');
        this.backgroundColor = J.def(this.options.backgroundColor, '#ffffff');
    }
}

G.TextInputSkin = TextInputSkin;



/**
 * 容器类基类。
 * 基类就是绝对定位式布局。
 * 子元素以父元素的左上角为0,0点。
 * @class
 * @param {object} options
 */
export class Box extends UIObject {

	constructor(options) {

		if (J.isN(options)) options = {};
		options.skin = J.def(options.skin, 'boxSkin');

		super(options);

		this.isBox = true;
		this.gtype = 'Box';

		this.isBackgroundImageLoaded = false;

		/**
		 * 背景色，盒子默认背景是透明的
		 */
		this._backgroundColor = J.def(this.options.backgroundColor, this.skin.backgroundColor);

		this._backgroundTrueColor = new TrueColor(this._backgroundColor);

		/**
		 * 背景图
		 */
		this._backgroundImage = J.def(this.options.backgroundImage, this.skin.backgroundImage);
		this.backgroundImageWidth = 0;
		this.backgroundImageHeight = 0;
		/**
		 * 背景图重复方式
		 */
		this._backgroundImageRepeat = J.def(this.options.backgroundImageRepeat, this.skin.backgroundImageRepeat);

		/**
		 * 背景图九格宫拉伸
		 * @type {Object}
		 */
		this.backgroundImageGrid9 = J.def(this.options.backgroundImageGrid9, this.skin.backgroundImageGrid9);

		/**
		 * 溢出处理
		 * stretch，父亲依据孩子高宽拉伸，默认
		 * hidden，超过父亲的就隐藏
		 * scroll，超过父亲产生滚动条
		 */
		this.overflow = J.def(this.options.overflow, this.skin.overflow);
		this.overflowX = J.def(this.options.overflowX, this.skin.overflowX);
		this.overflowY = J.def(this.options.overflowY, this.skin.overflowY);

		/**
		 * 总宽度（孩子宽度之和或最大宽度）。
		 * 用于测算滚动条。
		 */
		this.totalWidth = 0;
		/**
		 * 总高度（孩子高度之和或最大高度）
		 */
		this.totalHeight = 0;

		/**
		 * 用于滚动的盒子
		 */
		this.scrollBox = new UIObject();

	}

	get overflowX() {
		return this._overflowX;
	}

	set overflowX(v) {
		if (v == undefined) return;
		if (this._overflowX == v) return;
		this._overflowX = v;
		this.change('_overflowX');
	}
	get overflowY() {
		return this._overflowY;
	}

	set overflowY(v) {
		if (v == undefined) return;
		if (this._overflowY == v) return;
		this._overflowY = v;
		this.change('_overflowY');
	}

	get overflow() {
		return this._overflow;
	}

	set overflow(v) {
		if (v == undefined) return;
		if (this._overflow == v) return;

		this._overflow = v;
		this.overflowX = this.overflowY = v;
		this.change('overflow');
	}

	/**
	 * 背景色。
	 * 可以多种形式：
	 * 'rgb(250, 0,0)'
	 * 'rgb(100%,0%,0%)'
	 * 'hsl(0, 100%, 50%)'
	 * '#ff0000'
	 * '#f00'
	 * 'red'
	 * 
	 * 但如果要支持透明的话，只支持：#ff0000ff 模式，
	 * 最后两位表示透明度：ff 不透明，00 全透明。
	*/
	get backgroundColor() {
		return this._backgroundColor;
	}
	set backgroundColor(v) {
		if (this._backgroundColor == v) return;

		this._backgroundColor = v;

		if (this) {
			this.material.color = this.backgroundTrueColor.color;
			if (this.backgroundTrueColor.hasAlpha) {
				this.material.opacity = this.backgroundTrueColor.opacity;
			}
		}

		this.change('backgroundColor');
	}

	get backgroundTrueColor() {
		this._backgroundTrueColor = new TrueColor(this._backgroundColor);
		return this._backgroundTrueColor;
	}

	/**
	 * 背景图。
	 * 可以是纹理的 UUID 或 URL。
	 */
	get backgroundImage() {
		return this._backgroundImage;
	}
	set backgroundImage(v) {
		var _t = this;
		if (_t._backgroundImage == v)
			return;

		_t._backgroundImage = v;

		_t.loadBackgroundImage();

		this.change('backgroundImage');
	}

	get backgroundImageRepeat() {
		return this._backgroundImageRepeat;
	}

	set backgroundImageRepeat(v) {
		if (this._backgroundImageRepeat == v) return;

		this._backgroundImageRepeat = v;
		this.loadBackgroundImage();
		this.change('backgroundImageRepeat');
	}

	loadBackgroundImage() {
		var _t = this;

		if (J.isN(_t.backgroundImage))
			return;

		_t.isBackgroundImageLoaded = false;

		_t.loadTexture(_t.backgroundImage, (map) => {

			this.backgroundImageWidth = J.val(map, "image.width");
			this.backgroundImageHeight = J.val(map, "image.height");

			_t.setMap(map,
				_t._getRepeatType(_t.backgroundImageRepeat.x.type),
				_t._getRepeatType(_t.backgroundImageRepeat.y.type)
			);

			_t.fire('backgroundImageLoaded')
			_t.onBackgroundImageLoaded();
		});

	}

	onBackgroundImageLoaded() {
		this.isBackgroundImageLoaded = true;
	}

	_getRepeatType(t) {
		switch (t) {
			case "stretch":
				return THREE.ClampToEdgeWrapping;
			case "repeat":
				return THREE.RepeatWrapping;
			case "mirrorRepeat":
				return THREE.MirroredRepeatWrapping;
		}
	}

	/**
	 * 设置纹理
	 */
	setMap(map, wrapS, wrapT) {

		this.material.map = map;
		this.material.needsUpdate = true;

		threejsUtils.uvOffset(this.material.map, 0, 0, 1, 1, wrapS, wrapT);

	}

	onInited(args) {
		super.onInited(args);

		var _t = this;

		var mt = new THREE.SpriteMaterial({
			color: _t.backgroundTrueColor.color,
			transparent: _t.transparent,
			opacity: _t.opacity,
			depthTest: false
		});

		//如果背景色设置为带透明的颜色，则重新处理一次
		if (_t.backgroundTrueColor.hasAlpha) {
			mt.opacity = _t.backgroundTrueColor.opacity;
		}

		_t.material = mt;
		_t.material.needsUpdate = true;

		_t.loadBackgroundImage();

	}

	/**
	 * 添加子元素。，不递归。
	 * 多个孩子用逗号分隔。
	 */
	add(...args) {

		super.add(...args);
		var _t = this;
		var gobjs = J.argsToArr(arguments);

		gobjs.forEach((obj) => {

			//孩子z总比父亲z高
			obj.z += this.z;
			if (obj.z > 99) obj.z = 99;
		});
	}

	/**
	 * 盒子的计算高宽是按照 overflow 来
	 * 1. overflow 是 stretch 的时候，依据孩子的宽高最终计算自己的宽高
	 * 2. overflow 是 hidden 或 scroll 时，按 super.fixWH() 来
	 */
	stretch() {

		super.stretch();

		/*
		* 先做一次基本的宽高设置，后续的修正才会正确，
		* 比如设置了l/r/t/b的元素如果不fixWH()，
		* 则没有宽高，后面的计算就都错了。
		* 之所以在这里不分别条件，是因为儿子计算的时候需要用父亲的高宽
		*/
		this.fixWH();

		// Box 不做计算，HBox / VBox 才会有真正的拉伸。
		if (this.isHBox || this.isVBox) {

			this.width = this.stretchWidthFromChildren();
			this.height = this.stretchHeightFromChildren();

			this.stretchChildren();

		}

	}

	stretchChildren() {
		for (var i = 0; i < this.children.length; i++) {
			var child = this.children[i];
			child.stretch();
		}
	}

	/**
	 * 布局
	 */
	layout() {

		super.layout();
		//容器自己布局调整之后，自己的孩子都要重新布局
		this.layoutChildren();

	}
	/**
	 * 布局子元素
	 */
	layoutChildren() {

		for (var i = 0; i < this.children.length; i++) {
			var child = this.children[i];
			child.layout();
		}

	}

	/**
	 * 增量滚动
	 * @param {float} x 
	 * @param {float} y 
	 */
	scrollBy(x, y) {

		var offsetMax = {
			w: this.width - this.totalWidth,
			h: this.totalHeight - this.height
		};
		if (this.overflowX == 'scroll' &&
			this.totalWidth > this.width) {
			this.childrenGroup.position.x += x;

			//过界了就纠正回来
			if (this.childrenGroup.position.x > 0)
				this.childrenGroup.position.x = 0;

			if (this.childrenGroup.position.x < offsetMax.w)
				this.childrenGroup.position.x = offsetMax.w;
		}
		if (this.overflowY == 'scroll' &&
			this.totalHeight > this.height) {
			this.childrenGroup.position.y -= y;

			if (this.childrenGroup.position.y < 0)
				this.childrenGroup.position.y = 0;

			if (this.childrenGroup.position.y > offsetMax.h)
				this.childrenGroup.position.y = offsetMax.h;
		}

	}

	/**
	 * 滚动到
	 * @param {float} x 
	 * @param {float} y 
	 */
	scrollTo(x, y) {
		this.childrenGroup.position.x = 0;
		this.childrenGroup.position.y = 0;
		this.scrollBy(x, y);
	}

	onDrag(et) {
		super.onDrag(et);

		if (this.isHBox || this.isVBox) {
			this.scrollBy(et.deltaX, et.deltaY);
		}
	}
}

G.Box = Box;
/**
 * 水平容器
 * @class
 * @param {object} options
 */
export class HBox extends Box {
	constructor(options) {
		if (J.isN(options)) options = {};
		options.skin = J.def(options.skin, 'hboxSkin');
		super(options);

		this.isHBox = true;
		this.gtype = 'HBox';

	}

	/**
	 * 遍历每个孩子的宽高来计算父亲的宽高。
	 * 拉伸宽的话，就是所有孩子的宽相加，高度如果此时也拉伸，则高度就是所有孩子中最高的。
	 * 拉伸高的话，就是看看多少行，计算总高度，此时宽度如果也拉伸，那高度就是最高的孩子。
	 * 水平容器，只要宽拉伸了，高度拉伸就没有意义了，因为都在一行里面了。
	 */
	stretchWidthFromChildren() {

		var size = 0;

		var maxs = 0;
		for (var i = 0; i < this.children.length; i++) {

			var child = this.children[i];
			if (child.isHBox || child.isVBox) {
				maxs += child.stretchWidthFromChildren(type);
			} else {
				maxs += child.width + child.margin.left + child.margin.right;
			}
		}

		this.totalWidth = maxs;

		if (this.overflowX == 'stretch') {
			size = maxs;
		} else {
			size = this.width;
		}

		return size;
	}

	stretchHeightFromChildren() {

		var size = 0;
		var tsize = 0;

		//水平若拉伸了，则高度就是最高的孩子
		if (this.overflowX == 'stretch') {
			var maxs = 0;
			var s = 0;
			for (var i = 0; i < this.children.length; i++) {

				var child = this.children[i];
				if (child.isHBox || child.isVBox) {
					s = child.stretchHeightFromChildren();
				} else {
					s = child.width + child.margin.left + child.margin.right;
				}
				if (s > maxs) maxs = s;
			}
			tsize = maxs;
		}
		//水平不拉伸，那就看多少行，计算孩子总高度
		else {
			var totalW = 0;
			var totalH = 0;
			var maxh = 0;
			var w = 0;
			var h = 0;
			for (var i = 0; i < this.children.length; i++) {

				var child = this.children[i];
				if (child.isHBox || child.isVBox) {
					w = child.stretchWidthFromChildren();
					h = child.stretchHeightFromChildren();
				} else {
					w = child.width + child.margin.left + child.margin.right;
					h = child.height + child.margin.top + child.margin.bottom;
				}

				if (h > maxh) maxh = h;

				totalW += w;

				//如果宽度大于父亲，则增加一行
				if (w > this.width || totalW > this.width) {
					totalH += maxh;
					totalW = 0;
					maxh = 0;
				}
			}
			tsize = totalH + maxh;
		}

		this.totalHeight = tsize;

		if (this.overflowY == 'stretch') {
			size = tsize;
		} else {
			size = this.height;
		}

		return size;
	}

	/**
	 * 布局，依据所有子元素。
	 * 对于孩子来说，容器的左上角是 0,0，故下面的算法没有考虑到容器本身的位置的偏移，
	 * 这个动作由孩子自己回溯会比较合适，详见 UIObject/fixXY
	 */
	layoutChildren() {

		//如果是水平依据孩子宽度，那就不存在换行了
		if (this.overflowX == 'stretch')
			return;

		//当前行元素总宽度（margin+width）
		var tw = 0;
		//已有高度总和
		var th = 0;
		//当前行最大高度
		var maxh = 0;

		for (var i = 0; i < this.children.length; i++) {
			var child = this.children[i];

			var m = {
				l: child.margin.left,
				r: child.margin.right,
				t: child.margin.top,
				b: child.margin.bottom
			};

			var childW = child.width + m.l + m.r;

			var ch = child.height + m.t + m.b;
			if (ch > maxh)
				maxh = ch;

			//先加，再做是否换行的判断
			tw += childW;

			if (tw > this.width) {
				//设为 childW 是因为下面设置 left 的时候是 tw-childW。
				tw = childW;
				th += maxh;
				maxh = 0;

			}

			//tw-childW 是因为前面 +=childW，为了判断是否换行只能这样
			child.left = tw - childW + m.l;
			child.top = th + m.t;
			child.layout();

		}

	}

}

G.HBox = HBox;
/**
 * 垂直布局容器
 * @class
 * @param {object} options
 */
export class VBox extends Box {

	constructor(options) {
		if (J.isN(options)) options = {};
		options.skin = J.def(options.skin, 'vboxSkin');
		super(options);

		this.isVBox = true;
		this.gtype = 'VBox';

	}

	/**
	 * 遍历每个孩子的宽高来计算父亲的宽高。
	 * 拉伸宽的话，就是所有孩子中最宽的。
	 * 拉伸高的话，就是计算孩子总高度。
	 * 垂直容器，高宽两个都能拉伸。
	 */
	stretchWidthFromChildren() {

		var size = 0;

		var maxs = 0;
		var s = 0;
		for (var i = 0; i < this.children.length; i++) {

			var child = this.children[i];
			if (child.isHBox || child.isVBox) {
				s = child.stretchWidthFromChildren();
			} else {
				s = child.width + child.margin.left + child.margin.right;
			}
			if (s > maxs) maxs = s;
		}

		this.totalWidth = maxs;

		if (this.overflowX == 'stretch') {
			size = maxs;
		} else {
			size = this.width;
		}

		return size;
	}

	stretchHeightFromChildren() {

		var size = 0;

		var maxs = 0;
		for (var i = 0; i < this.children.length; i++) {

			var child = this.children[i];
			if (child.isHBox || child.isVBox) {
				maxs += child.stretchHeightFromChildren();
			} else {
				maxs += child.height + child.margin.top + child.margin.bottom;
			}
		}

		this.totalHeight = maxs;

		if (this.overflowY == 'stretch') {
			size = maxs;
		} else {
			size = this.height;
		}

		return size;
	}

	/**
	 * 布局，依据所有子元素
	 */
	layoutChildren() {

		if (this.overflowY == 'stretch')
			return;

		var th = 0;
		var pp = this.padding;

		for (var i = 0; i < this.children.length; i++) {

			var child = this.children[i];

			var m = {
				l: child.margin.left,
				r: child.margin.right,
				t: child.margin.top,
				b: child.margin.bottom
			};

			child.left = m.l + pp.left;
			child.top = m.t + pp.top + th;
			th += child.height + m.t + m.b + pp.top + pp.bottom;

			child.layout();
		}

	}

}

G.VBox = VBox;
/**
 * UI 元素-Button
 */
export class Button extends Box {
    constructor(options) {

        if (J.isN(options)) options = {};
        options.skin = J.def(options.skin, 'buttonSkin');

        super(options);

        this.isButton = true;
        this.gtype = 'Button';

        /**
          * 子类型。
          * all：有背景、图标、文字，textAlign 生效
          * onlyIcon：只有图标
          * onlyText：只有文字
          * icon：背景+图标
          * text：背景+文字
          */
        this.subType = J.def(this.options.subType, this.skin.subType);

        /**
         * 按钮上文本与图片的相对位置。
         * 以文字位置为参考物，且只有 subType 为 all 时生效
         * top：文字在上，图标在下，
         * bottom：文字在下，图标在上，
         * left：文字在左，图标在右，
         * right：文字在右，图标在左，
         * @type {String}
         */
        this._textAlign = J.def(this.options.textAlign, "right");

        /**
         * 文本
         * @type {string}
         */
        this._text = J.def(this.options.text, "");
        /**
         * 字体。
         * @type {string}
         */
        this._font = J.def(this.options.font, this.skin.font);

        /**
         * 字体颜色
         * @type {string}
         */
        this._color = J.def(this.options.color, this.skin.color);
        /**
         * 图标图片
         * 可以是纹理 uuid 或地址
         * @type {string}
         */
        this._icon = J.def(this.options.icon, this.skin.icon);

        //图标和背景图都是由一张图从左至右均分成4等分的图片 |normal/up|hover|down|disabled|
        //通过调整其 x 轴的位置来控制显示的是哪部分图片
        this.offsetX = {
            normal: 0 / 4,
            hover: 1 / 4,
            down: 2 / 4,
            disabled: 3 / 4
        };

        if (this.subType == 'onlyIcon' || this.subType == 'onlyText') {
            this._backgroundImage = "";
            this._transparent = true;
            this._opacity = 0;
        }

        this._enabled = J.def(this.options.enabled, true);

    }

    get enabled() { return this._enabled; }
    set enabled(v) {
        if (this._enabled == v)
            return;

        this._enabled = v;

        this.children.forEach(child => {
            child.enabled = v;
        })

        this._enabledThis(v);
        this.change('enabled');
    }

    get textAlign() { return this._textAlign; }
    set textAlign(v) {
        this._textAlign = v;
        this.label.align = v;
        this.change('textAlign');
    }
    get text() { return this._text; }
    set text(v) {
        this._text = v;
        this.label.text = v;
        this.change('text');
    }
    get font() { return this._font; }
    set font(v) {
        this._font = v;
        this.label.font = v;
        this.change('font');
    }
    get color() { return this._color; }
    set color(v) {
        this._color = v;
        this.label.color = v;
        this.change('color');
    }
    get icon() { return this._icon; }
    set icon(v) {
        this._icon = v;
        this.imgIcon.url = v;
        this.change('icon');
    }

    _enabledThis(v) {
        var _t = this;
        if (_t.subType == 'text' || _t.subType == 'icon' || _t.subType == 'all') {
            threejsUtils.uvOffset(_t.material.map, v == true ? this.offsetX.normal : this.offsetX.disabled, 0);
        }
    }

    onCreated(args) {

        super.onCreated(args);
        var _t = this;

        //处理文字
        if (_t.subType == 'text' || _t.subType == 'onlyText' || _t.subType == 'all') {

            _t.label = new Label({
                text: _t.text,
                font: _t.font,
                color: _t.color,
                width: _t.width,
                height: _t.height,
                centerX: 0,
                centerY: 0
            });

            _t.add(_t.label);

        }

        //处理图标
        if (_t.subType == 'icon' || _t.subType == 'onlyIcon' || _t.subType == 'all') {

            _t.imgIcon = new Picture({
                url: _t.icon,
                width: _t.width,
                height: _t.height,
                centerX: 0,
                centerY: 0
            });

            _t.add(_t.imgIcon);
        }
    }

    onChanged(name, value) {

        super.onChanged(name, value);

        if (this.isInited == false) return;

        if (this.subType == 'icon' || this.subType == 'onlyIcon' || this.subType == 'all') {
            this.imgIcon.width = this.width;
            this.imgIcon.height = this.height;

            this.layoutChildren();
        }
    }

    onBackgroundImageLoaded() {

        super.onBackgroundImageLoaded();

        var _t = this;
        //处理背景图
        if (_t.subType == 'text' || _t.subType == 'icon' || _t.subType == 'all') {

            //x轴偏移位置，y轴不偏移；x轴平铺范围四分之一，y轴全平铺
            threejsUtils.uvOffset(this.material.map, this.offsetX.normal, 0, 1 / 4, 1);
            this._enabledThis(_t.enabled);
        }
    }

    /**
     * 4个鼠标事件（按下、松开、悬停、移开）
     */
    onDown(et) {
        var _t = this;
        if (_t.enabled == false) return;
        if (_t.subType == 'text' || _t.subType == 'icon' || _t.subType == 'all') {
            threejsUtils.uvOffset(_t.material.map, this.offsetX.down, 0);
        }
    }

    onUp(et) {
        var _t = this;
        if (_t.enabled == false) return;
        if (_t.subType == 'text' || _t.subType == 'icon' || _t.subType == 'all') {
            threejsUtils.uvOffset(_t.material.map, this.offsetX.normal, 0);
        }
    }

    onOver(et) {
        var _t = this;
        if (_t.enabled == false) return;
        if (_t.subType == 'text' || _t.subType == 'icon' || _t.subType == 'all') {
            threejsUtils.uvOffset(_t.material.map, this.offsetX.hover, 0);
        }
    }

    onOut(et) {
        var _t = this;
        if (_t.enabled == false) return;
        if (_t.subType == 'text' || _t.subType == 'icon' || _t.subType == 'all') {
            threejsUtils.uvOffset(_t.material.map, this.offsetX.normal, 0);
        }
    }

}

G.Button = Button;

/**
 * UI 裁剪器，适用于 UIObject 子类的对象。
 * 依照父亲的矩形来裁剪自己，四周只要超过父亲的边框就裁剪掉。
 * @export
 * @class Clipper
 * @extends {UIObject}
 */
export class Clipper extends EventDispatcher {

    constructor(options) {
        super(options);

        this.isClipper = true;
        this.gtype = "Clipper";

        this.target = null;

        this.material = new THREE.ShaderMaterial({
            uniforms:
            {
                minX: {
                    value: 0.0
                },
                minY: {
                    value: 0.0
                },
                maxX: {
                    value: 1.0
                },
                maxY: {
                    value: 1.0
                },
                myColor: {
                    value: new THREE.Color(0xffffff)
                },
                myMap: {
                    value: null
                }
            },

            vertexShader: [
                'varying vec2 vUv;',
                'void main(){',
                '	vUv = uv;',
                '	gl_Position = projectionMatrix * modelViewMatrix * vec4(position, 1.0);',
                '}',
            ].join(''),

            fragmentShader: [
                'uniform float minX;',
                'uniform float minY;',
                'uniform float maxX;',
                'uniform float maxY;',
                'uniform vec3 myColor;',
                'uniform sampler2D myMap;',
                'varying vec2 vUv;',
                'void main(){',
                '   vec4 tclr = vec4(1,1,1,0);',
                '	vec4 clr = texture2D( myMap, vUv );',
                '   clr = vec4(clr.rgb * myColor,clr.a);',
                '   if(vUv.x<minX || vUv.x>maxX || vUv.y>minY || vUv.y<maxY){ clr=tclr; }',
                '	gl_FragColor = clr;',
                '}',
            ].join(''),

            side: THREE.FrontSide,
            transparent: true,
            opacity: 1
        });

    }

    clipping(name) {

        var props = ['left', 'top', 'right', 'bottom', 'width', 'height'];
        if (name == undefined || props.indexOf(name) != -1) {

            var dy1 = 0;
            var dy2 = 0;
            var dx1 = 0;
            var dx2 = 0;

            //遍历自己所有父亲，看看谁遮罩了更多，就以谁为准来进行裁剪（矩形裁剪）
            this.target.parents.forEach((p) => {

                var dy11 = p.rect.y1 - this.target.rect.y1;
                if (dy11 > dy1) dy1 = dy11;

                var dy21 = this.target.rect.y2 - p.rect.y2;
                if (dy21 > dy2) dy2 = dy21;

                var dx11 = p.rect.x1 - this.target.rect.x1;
                if (dx11 > dx1) dx1 = dx11;

                var dx21 = this.target.rect.x2 - p.rect.x2;
                if (dx21 > dx2) dx2 = dx21;

            });

            /* 
            * 特别注意，在着色器中， uv 的排列方式是：
            * (0,1)...........(1,1)
            * .....................
            * .....................
            * (0,0)...........(1,0)
            * 从左到右，x 从 0 到 1，而从上到下，y 从 1 到 0。
            */
            this.material.uniforms.minX.value = dx1 / this.target.width;
            this.material.uniforms.maxX.value = 1 - dx2 / this.target.width;
            this.material.uniforms.minY.value = 1 - dy1 / this.target.height;
            this.material.uniforms.maxY.value = dy2 / this.target.height;
        }
    }

    attach(uiobj) {

        this.target = uiobj;
        uiobj.on('changed', this.clipping, this);

        var obj = this.target;

        //传递当前纹理和颜色到着色器纹理中进行处理
        this.material.uniforms.myColor.value = obj.material.color;
        this.material.uniforms.myMap.value = obj.material.map;

        obj.material = this.material;
        obj.material.needsUpdate = true;

        this.clipping();
    }
}

G.Clipper = Clipper;
export class Dialog extends Box {

    constructor(options) {

        if (J.isN(options)) options = {};
        options.skin = J.def(options.skin, 'dialogSkin');

        super(options);

        this.isDialog = true;
        this.gtype = 'Dialog';

        this.yesText = J.def(this.options.yesText, '是');
        this.noText = J.def(this.options.noText, '否');
        this.cancelText = J.def(this.options.cancelText, '取消');
        this._title = J.def(this.options.title, '提示');

        this.allButtons = ['yes', 'no', 'cancel', 'close'];

        this.buttons = J.def(this.options.buttons, this.allButtons);
        this.parts = J.def(this.options.parts, ['title', 'content', 'bottom']);

        this.titleBar = new Box({
            skin: this.skin.titleBarSkin
        });
        this.contentBar = new Box({
            skin: this.skin.contentBarSkin
        });
        this.bottomBar = new HBox({
            skin: this.skin.bottomBarSkin
        });

        this.titleLabel = new Label({
            text: this._title,
            skin: this.titleBar.skin.labelSkin,
        });

        this.closeButton = new Button({
            skin: this.titleBar.skin.closeButtonSkin,
            onClick: (et) => {
                this.hide();
                if (this.options.onClose)
                    this.options.onClose.apply(this, et);
                this.fire('close', et);
            }
        });

        this.yesButton = new Button({
            text: this.yesText,
            skin: 'firstButtonSkin',
            width: 80,
            onClick: (et) => {
                this.hide();
                if (this.options.onYes)
                    this.options.onYes.apply(this, et);
                this.fire('yes', et);
            }
        });

        this.noButton = new Button({
            text: this.noText,
            skin: 'buttonSkin',
            width: 80,
            onClick: (et) => {
                this.hide();
                if (this.options.onNo)
                    this.options.onNo.apply(this, et);
                this.fire('no', et);
            }
        });

        this.cancelButton = new Button({
            text: this.cancelText,
            width: 80,
            skin: 'secondButtonSkin',
            width: 80,
            onClick: (et) => {
                this.hide();
                if (this.options.onCancel)
                    this.options.onCancel.apply(this, et);
                this.fire('cancel', et);
            }
        });

    }

    get title() { return this._title; }
    set title(v) {
        this._title = v;
        this.titleLabel.text = v;
    }

    onInited(args) {

        super.onInited(args);

        this.titleBar.add(this.titleLabel);

        var r = J.CE(this.allButtons, this.buttons);

        if (r.charAt(0) == '1')
            this.bottomBar.add(this.yesButton);
        if (r.charAt(1) == '1')
            this.bottomBar.add(this.noButton);
        if (r.charAt(2) == '1')
            this.bottomBar.add(this.cancelButton);
        if (r.charAt(3) == '1')
            this.titleBar.add(this.closeButton);

        var r2 = J.CNTE(['yes', 'no', 'cancel'], this.buttons);

        switch (r2) {

            case 3:
                this.bottomBar.width = 240;
                break;
            case 2:
                this.bottomBar.width = 160;
                break;
            case 1:
                this.bottomBar.width = 80;
                break;
        }

        var ps = J.CE(['title', 'content', 'bottom'], this.parts);
        if (ps[0] == 1)
            this.add(this.titleBar);
        if (ps[1] == 1)
            this.add(this.contentBar);
        if (ps[2] == 1)
            this.add(this.bottomBar);

    }
}

G.Dialog = Dialog;

/**
 * 下拉框
 */
export class DropdownList extends Button {

    constructor(options) {
        super(options);

        this.isDropdownList = true;
        this.gtype = 'DropdownList';

        this.container = new VBox({
            left: 0,
            right: 0,
            bottom: 0,
            height: 200,
            visible: false,
            z: 99
        });

        SM.add(this.container);

        this._items = J.def(this.options.items, []);

        this._selectedIndex = -1;
    }

    get items() {
        return this._items;
    }

    set items(v) {

        this._items = v;
        this.change('items');

    }

    get selectedIndex() {
        return this._selectedIndex;
    }

    set selectedIndex(v) {
        this._selectedIndex = v;
        this.change('selectedIndex');
    }

    onInited(args) {
        super.onInited(args);

        for (let i = 0; i < this.items.length; i++) {
            const item = this.items[i];

            var btn = new Button({
                text: item,
                left: 0,
                right: 0,
                data: i
            });

            btn.on('click', (et) => {
                this.selectedIndex = et.gobject.data;
                this.container.hide();
                this.fire('itemSelected', this.selectedIndex);
            });

            this.container.add(btn);
        }
    }

    onClick(et) {
        this.container.show();
    }
}

G.DropdownList = DropdownList;
/**
 * 主页面框架。
 * 填充整个舞台，溢出默认滚动。
 * 作用主要是在语义上的可读性。
 */
export class Frame extends Box {

    constructor(options) {

        if (J.isN(options)) options = {};
        options.skin = J.def(options.skin, 'frameSkin');

        super(options);
        this.isFrame = true;
        this.gtype = "Frame";

        this.pages = J.def(this.options.pages, []);
        this._page = null;
        this._pageIdx = -1;
    }

    onInited(args) {
        super.onInited(args);

        this.pages.forEach((p) => {
            if (p.isMain) {
                this.page = p;
            }
        })
    }

    add() {

        var ps = J.argsToArr(arguments);

        ps.forEach((p) => {
            p.hide();
            if (p.isMain)
                p.show();
            this.pages.push(p);

            super.add(p);
        });
    }

    remove(page) {
        J.ua.remove(this.pages, page);
        super.remove(page);
    }

    get pageIdx() {
        return this._pageIdx;
    }

    set pageIdx(v) {
        if (this._pageIdx == v) return;

        this._pageIdx = v;
        this.change('pageIdx');

        this.page = this.pages[v];
    }

    /**
     * 当前页
     */
    get page() {
        return this._page;
    }

    set page(v) {

        if (this._page == v) return;

        this._page = v;
        this.change('page');

        this._pageIdx = this.pages.indexOf(v);

        this.pages.forEach((p) => {
            p.hide();
        });

        v.show();

    }

    /**
     * 后退逻辑。
     * 采用历史记录来完成。
     */
    back() {

    }

}

G.Frame = Frame;
/**
 * 文本标签。
 * 支持多行、阴影。
 * 要获得文字整个大小，可用 this.geometry.boundingBox 拿到。
 */
export class Label extends UIObject {

	constructor(options) {

		if (J.isN(options)) options = {};
		options.skin = J.def(options.skin, 'labelSkin');

		super(options);

		this.isLabel = true;
		this.gtype = 'Label';

		/**
		 * 文本
		 * @type {string}
		 */
		this._text = J.def(this.options.text, "");
		/**
		 * 字体。
		 * 参考：http://www.w3school.com.cn/tags/canvas_font.asp。
		 * @type {string}
		 */
		this._font = J.def(this.options.font, this.skin.font);

		/**
		 * 字体颜色
		 * @type {string}
		 */
		this._color = J.def(this.options.color, this.skin.color);
		/**
		 * 背景颜色
		 * @type {int}
		 */
		this._backgroundColor = J.def(this.options.backgroundColor, this.skin.backgroundColor);

		this._backgroundTrueColor = new TrueColor(this._backgroundColor);

		this._align = J.def(this.options.align, this.skin.align);

	}

	/**
	* 文本
	*/
	get text() {
		return this._text;
	}
	set text(v) {

		if (this._text == v)
			return;

		this._text = v;
		this.drawText();
		this.change('text');

	}
	/**
	* 字体
	*/
	get font() {
		return this._font;
	}
	set font(v) {

		if (this._font == v)
			return;

		this._font = v;
		this.drawText();
		this.change('font');

	}

	/**
	* 字体颜色
	*/
	get color() {
		return this._color;
	}
	set color(v) {

		if (this._color == v)
			return;

		this._color = v;
		this.drawText();
		this.change('color');

	}

	get align() {
		return this._align;
	}
	set align(v) {

		if (this._align == v)
			return;

		this._align = v;
		this.drawText();
		this.change('align');
	}

	get backgroundColor() {
		return this._backgroundColor;
	}
	set backgroundColor(v) {
		if (this._backgroundColor == v) return;

		this._backgroundColor = v;

		this.material.color = this.backgroundTrueColor.color;
		if (this.backgroundTrueColor.hasAlpha) {
			this.material.opacity = this.backgroundTrueColor.opacity;
		}

		this.change('backgroundColor');
	}

	get backgroundTrueColor() {
		this._backgroundTrueColor = new TrueColor(this._backgroundColor);
		return this._backgroundTrueColor;
	}

	onAdded(args) {
		super.onAdded(args);
		this.drawText();
	}

	/**
	 * 重画文本
	 */
	drawText() {
		var _t = this;

		var cv = canvasUtils.createTextCanvas(_t.text, _t.font, _t.color, _t.backgroundColor, 'fill',
			_t._width, _t._height, _t.align, true);

		//如果没有设置高宽，内部会自动计算合适的高宽（当然内部为了纹理清楚，就放大了两倍）
		if (J.isN(_t.width))
			_t.width = cv.width / 2;
		if (J.isN(_t.height))
			_t.height = cv.height / 2;

		var ct = new THREE.CanvasTexture(cv);
		_t.material.map = ct;
		_t.material.needsUpdate = true;

		//修改了宽高，就要布局
		this.layout();
	}

}

G.Label = Label;
/**
 * 线条
 */
export class Line extends Box {

    constructor(options) {
        super(options);

        this.isLine = true;
        this.gtype = 'Line';

        this.overflow = 'hidden';

        this._backgroundColor = J.def(this.options.backgroundColor, "#cccccc");

        this.direction = J.def(this.options.direction, 'horizontal');

        if (this.direction == 'horizontal')
            this.height = 1;
        else
            this.width = 1;

        this._opacity = 1;
    }

}

G.Line = Line;

/**
 * UI 遮罩，适用于 UIObject 子类的对象。
 * 3D 物体请采用 alphaMap 来完成。
 * @export
 * @class Masker
 * @extends {UIObject}
 */
export class Masker extends EventDispatcher {

    constructor(options) {
        super(options);

        this.isMasker = true;
        this.gtype = "Masker";

        /**
         * 遮罩形状。
         * rect 矩形，roundrect 圆角矩形，circle 圆
         */
        this.subType = J.def(this.options.subType, {
            type: 'roundrect',
            radius: 0.04
        });

        this.subData = '';

        this.target = null;

        this.material = new THREE.ShaderMaterial({
            uniforms:
            {
                subTypeIndex: { value: this.subTypeIndex }
            },

            vertexShader: [
                'varying vec2 vUv;',
                'void main(){',
                '	vUv = uv;',
                '	gl_Position = projectionMatrix * modelViewMatrix * vec4(position, 1.0);',
                '}',
            ].join(''),

            fragmentShader: [
                'uniform int subTypeIndex;',
                'varying vec2 vUv;',
                'void main(){',
                '	vec2 uv = vUv;',
                '   vec4 tc = vec4(1,1,1,0);',
                '	vec4 clr = gl_Color;',
                '	float dis=sqrt(pow(abs(vUv.x-0.5),2)+pow(abs(vUv.y-0.5),2));',
                '	if(subTypeIndex==3){',
                '       if(dis>=0.5){ clr=tc; }',
                '   }',
                '	gl_FragColor = clr;',
                '}',
            ].join(''),

            side: THREE.FrontSide,
            transparent: true,
            opacity: 1
        });

    }

    get subTypeIndex() {
        var r = 1;
        switch (this.subType) {
            case 'rect':
                r = 1;
                break;
            case 'roundrect':
                r = 2;
                break;
            case 'circle':
                r = 3;
                break;
        }
        return r;
    }

    attach(uiobj) {

        this.target = uiobj;
        var obj = this.target;

        obj.material = this.material;
        obj.material.needsUpdate = true;
    }
}

G.Masker = Masker;
/**
 * 页面。
 * 放置在 Frame 里面的子页面。
 */
export class Page extends VBox {

    constructor(options) {

        if (J.isN(options)) options = {};
        options.skin = J.def(options.skin, 'pageSkin');

        super(options);

        this.isPage = true;
        this.gtype = "Page";

        /**
         * 是否是主页
         */
        this.isMain = J.def(this.options.isMain, false);

        this.titleBar = new HBox({
            skin: this.skin.titleBarSkin
        });

        this.contentBar = new Box({
            skin: this.skin.contentBarSkin
        });

        this.bottomBar = new Box({
            skin: this.skin.bottomBarSkin
        })

        this.backButton = new Button({
            icon: "/libs/assets/back.png",
            skin: 'onlyIconButtonSkin',
        });
        this.menuButton = new Button({
            icon: "/libs/assets/menu.png",
            skin: 'onlyIconButtonSkin',
        });
        this.searchButton = new Button({
            icon: "/libs/assets/search.png",
            skin: 'onlyIconButtonSkin',
        });

        var aicon = "/libs/assets/user.png";
        if (J.wx.isOAuth()) {
            var user = J.wx.getOAuthUser();
            aicon = user.Avatar
        }
        this.avatarButton = new Button({
            icon: aicon,
            skin: 'onlyIconButtonSkin',
        });

        this.moreButton = new Button({
            icon: "/libs/assets/more.png",
            skin: 'onlyIconButtonSkin',
        });


        this.buttons = J.def(this.options.buttons, ['back', 'avatar']);
        this.parts = J.def(this.options.parts, ['title', 'content', 'bottom']);

        /**
         * 页面标题
         */
        this._title = J.def(this.options.title, 'Page');
        this.labelContainer = new Box({
            width: 1,
            height: this.titleBar.height,
            overflow: 'hidden',
        });
        this.label = new Label({
            text: this._title,
            centerX: 0,
            centerY: 0,
            skin: this.titleBar.skin.labelSkin
        });
    }

    get title() { return this._title; }
    set title(v) {

        if (this._title == v)
            return;

        this._title = v;
        this.titleBar.text = v;
        this.change('title');
    }

    onInited(args) {
        super.onInited(args);

        var bs = J.CE(['back', 'menu', 'search', 'avatar', 'more'], this.buttons);

        if (bs[0] == '1')
            this.titleBar.add(this.backButton);
        if (bs[1] == '1')
            this.titleBar.add(this.menuButton);

        this.labelContainer.add(this.label);
        this.titleBar.add(this.labelContainer);

        if (bs[2] == '1')
            this.titleBar.add(this.searchButton);
        if (bs[3] == '1')
            this.titleBar.add(this.avatarButton);
        if (bs[4] == '1')
            this.titleBar.add(this.moreButton);

        var ps = J.CE(['title', 'content', 'bottom'], this.parts);
        if (ps[0] == 1)
            this.add(this.titleBar);
        if (ps[1] == 1)
            this.add(this.contentBar);
        if (ps[2] == 1)
            this.add(this.bottomBar);
    }

    onLayout() {
        super.onLayout();

        //将中间的标题栏拉伸
        var len = this.buttons.length;
        this.labelContainer.width = this.width - len * (this.backButton.width + this.backButton.margin.left * 2);
        this.layoutChildren();

    }
}

G.Page = Page;

/**
 * UI 元素-Picture
 * 为了不与 Image 元素冲突，特改名为 Picture
 */
export class Picture extends Box {

	constructor(options) {
		super(options);

		this.isPicture = true;
		this.gtype = 'Picture';

		/**
		 * 地址
		 * 即可是真实地址，也可是纹理的 uuid
		 */
		this._url = J.def(this.options.url, "/libs/assets/noPicture.png");

		this._opacity = 1;

		this._backgroundColor = J.def(this.options.backgroundColor, "#ffffff");
		this._backgroundImage = this._url;

		this.overflow = 'hidden';
	}

	/**
	* 图片地址，可以是uuid
	* @type {string}
	*/
	get url() {
		return this._url;
	}
	set url(v) {
		var _t = this;
		if (_t._url == v)
			return;

		_t._url = v;

		_t.backgroundImage = v;

		this.change('url');
	}

}

G.Picture = Picture;
/**
 * PictureAnimator，图片动画器。
 * 参考技术：http://localhost:7979/three.js-master/examples/?q=sprite#webgl_sprites_nodes
 * 但并没有采用 Node 这一套，而是采用 Shader 完成。
 */

export class PictureAnimator extends Picture {

    constructor(options) {
        super(options);

        this.isPictureAnimator = true;
        this.gtype = 'PictureAnimator';

        this._url = J.def(this.options.url, "/libs/assets/walkingManSpriteSheet.png");
        this._backgroundImage = this._url;

        this._repeatTypeX = "repeat";

        this.width = J.def(this.options.width, 128);
        this.height = J.def(this.options.height, 128);

        /**
         * 循环。可以是 forever|once
         */
        this.loop = J.def(this.options.loop, 'forever');

        /**
         * 帧数
         */
        this.frames = J.def(this.options.frames, 8);
        /**
         * 帧率
         */
        this.fps = J.def(this.options.fps, 10);

        this.status = 'stop';

        this._idx = 0;

        this.shaderMat = new THREE.ShaderMaterial({
            uniforms:
            {
                //当前帧序号，从 0 开始
                frameIndex: {
                    value: 0
                },
                myColor: {
                    value: new THREE.Color(0xffffff)
                },
                myMap: {
                    value: null
                },
                frames: {
                    value: this.frames
                }
            },

            vertexShader: [
                'varying vec2 vUv;',
                'void main(){',
                '	vUv = uv;',
                '	gl_Position = projectionMatrix * modelViewMatrix * vec4(position, 1.0);',
                '}',
            ].join(''),

            fragmentShader: [
                'uniform int frameIndex;',
                'uniform vec3 myColor;',
                'uniform sampler2D myMap;',
                'uniform int frames;',
                'varying vec2 vUv;',
                'void main(){',
                '   float fi=float(frameIndex);float fr=float(frames);float df=1.0/fr;',
                '	vec2 cUv = vec2( fi * df + vUv.x * df , vUv.y);',
                '	vec4 clr = texture2D( myMap, cUv );',
                '   clr = vec4(clr.rgb * myColor,clr.a);',
                '	gl_FragColor = clr;',
                '}',
            ].join(''),

            side: THREE.FrontSide,
            transparent: true,
            opacity: 1
        });
    }

    onBackgroundImageLoaded() {

        this.shaderMat.uniforms.myColor.value = this.material.color;
        this.shaderMat.uniforms.myMap.value = this.material.map;

        this.material = this.shaderMat;
        this.material.needsUpdate = true;

        super.onBackgroundImageLoaded();
    }

    play() {
        this.status = 'play';
    }

    pause() {
        this.status = 'pause';
    }

    stop() {
        this.status = 'stop';
        this._idx = 0;
    }

    onUpdated(args) {
        super.onUpdated(args);

        if (this.isBackgroundImageLoaded && this.status === 'play') {
            this._idx += this.fps * SM.delta;
            var _idx2 = Math.floor(this._idx);
            if (_idx2 >= this.frames - 1) {
                this._idx = 0;
            }
            this.shaderMat.uniforms.frameIndex.value = _idx2;
        }
    }

}

G.PictureAnimator = PictureAnimator;
/**
 * PictureRoller，图片轮播
 */
export class PictureRoller extends Box {

    constructor(options) {
        super(options);

        this.isPictureRoller = true;
        this.gtype = 'PictureRoller';

        this.overflow = 'scroll';

        /**
         * 高宽
         */
        this.width = J.def(this.options.width, 300);
        this.height = J.def(this.options.height, 200);

        this._backgroundColor = J.def(this.options.backgroundColor, "#ffffff");

        /**
         * 动画效果
         */
        this._easing = J.def(this.options.easing, TWEEN.Easing.Sinusoidal.InOut);

        /**
         * 标题集
         */
        this._titles = J.def(this.options.titles, []);
        /**
         * 标题高度
         */
        this._titleHeight = J.def(this.options.titleHeight, 40);
        /**
         * 标题颜色
         */
        this._titleColor = J.def(this.options.titleColor, "#ff0000");
        /**
         * 标题字体
         */
        this._titleFont = J.def(this.options.titleFont, '14px sans-serif');

        /**
         * 描述集
         */
        this._descs = J.def(this.options.descs, []);
        /**
         * 描述高度
         */
        this._descHeight = J.def(this.options.descHeight, 40);
        /**
         * 描述颜色
         */
        this._descColor = J.def(this.options.descColor, "#000000");
        /**
         * 描述字体
         */
        this._descFont = J.def(this.options.descFont, '12px sans-serif');

        /**
         * 图片集
         * 即可是真实地址，也可是纹理的 uuid
         */
        this._urls = J.def(this.options.urls, []);
        /**
         * 标识大小
         */
        this._signSize = J.def(this.options.signSize, 24);
        /**
         * 上一张图标
         */
        this._nextIcon = J.def(this.options.nextIcon, "/libs/assets/back.png");
        /**
         * 下一张图标
         */
        this._prevIcon = J.def(this.options.prevIcon, "/libs/assets/forward.png");

        /**
         * 自动切换
         */
        this._autoplay = J.def(this.options.autoplay, true);
        /**
         * 切换方向（left/right/up/down）
         */
        this._direction = J.def(this.options.direction, "left");
        /**
         * 切换距离
         */
        this._distance = J.def(this.options.distance, this.horizontal ? this.width : this.height);
        /**
         * 切换时长，毫秒
         */
        this._duration = J.def(this.options.duration, 500);
        /**
         * 切换间隔，毫秒
         */
        this._interval = J.def(this.options.interval, 50);
        /**
         * 首次延迟，毫秒
         */
        this._firstDelay = J.def(this.options.firstDelay, 1000);

        /**
         * 是否将子元素加为双倍
         */
        this._isDouble = J.def(this.options.isDouble, true);

        /**
         * 是否足够滚动才开始，如一个容器中只有一张图片，小于容器宽度，就没有必要滚动
         */
        this._enoughToStart = J.def(this.options.enoughToStart, true);

        /**
         * 显示部分，默认显示全部
         */
        this._parts = J.def(this.options.parts, 'picture,title,desc,number,sign');

        /**
         * 正在移动
         */
        this._isMoving = false;
        /**
         * 定时器
         */
        this._timerOfInterval = {};

    }

    /**
    * 序号
    */
    get index() {
        return this._index;
    }
    set index(v) {
        this._index = v;
        this.change('index');
    }

    /**
    * 序号
    */
    get isDouble() {
        return this._isDouble;
    }
    set isDouble(v) {
        this._isDouble = v;
        this.change('isDouble');
    }

    /**
    * 动画效果
    */
    get easing() {
        return this._easing;
    }
    set easing(v) {
        this._easing = v;
        this.change('easing');
    }


    /**
    * 自动播放
    */
    get autoplay() {
        return this._autoplay;
    }
    set autoplay(v) {
        this._autoplay = v;
        this.change('autoplay');
    }

    /**
    * 播放方向
    */
    get direction() {
        return this._direction;
    }
    set direction(v) {
        this._direction = v;
        this.change('direction');
    }

    /**
    * 切换距离
    */
    get distance() {
        return this._distance;
    }
    set distance(v) {
        this._distance = v;
        this.change('distance');
    }
    /**
    * 切换时长
    */
    get duration() {
        return this._duration;
    }
    set duration(v) {
        this._duration = v;
        this.change('duration');
    }
    /**
     * 切换间隔
     */
    get interval() {
        return this._interval;
    }
    set interval(v) {
        this._interval = v;
        this.change('interval');
    }
    /**
     * 切换间隔
     */
    get firstDelay() {
        return this._firstDelay;
    }
    set firstDelay(v) {
        this._firstDelay = v;
        this.change('firstDelay');
    }



    /**
    * 标题集
    */
    get titles() {
        return this._titles;
    }
    set titles(v) {
        this._titles = v;
        this.change('titles');
    }
    /**
    * 标题高度
    */
    get titleHeight() {
        return this._titleHeight;
    }
    set titleHeight(v) {
        this._titleHeight = v;
        this.change('titleHeight');
    }
    /**
    * 标题颜色
    */
    get titleColor() {
        return this._titleColor;
    }
    set titleColor(v) {
        this._titleColor = v;
        this.change('titleColor');
    }
    /**
    * 标题字体
    */
    get titleFont() {
        return this._titleFont;
    }
    set titleFont(v) {
        this._titleFont = v;
        this.change('titleFont');
    }

    /**
    * 描述集
    */
    get descs() {
        return this._descs;
    }
    set descs(v) {
        this._descs = v;
        this.change('descs');
    }
    /**
    * 描述高度
    */
    get descHeight() {
        return this._descHeight;
    }
    set descHeight(v) {
        this._descHeight = v;
        this.change('descHeight');
    }
    /**
    * 描述颜色
    */
    get descColor() {
        return this._descColor;
    }
    set descColor(v) {
        this._descColor = v;
        this.change('descColor');
    }
    /**
    * 描述字体
    */
    get descFont() {
        return this._descFont;
    }
    set descFont(v) {
        this._descFont = v;
        this.change('descFont');
    }

    /**
    * 标识数量
    */
    get signSize() {
        return this._signSize;
    }
    set signSize(v) {
        this._signSize = v;
        this.change('signSize');
    }
    /**
    * 左图标
    */
    get nextIcon() {
        return this._nextIcon;
    }
    set nextIcon(v) {
        this._nextIcon = v;
        this.change('nextIcon');
    }
    /**
    * 右图标
    */
    get prevIcon() {
        return this._prevIcon;
    }
    set prevIcon(v) {
        this._prevIcon = v;
        this.change('prevIcon');
    }
    /**
    * 图片集
    */
    get urls() {
        return this._urls;
    }
    set urls(v) {
        this._urls = v;
        this.change('urls');
    }

    /**
    * 数量
    */
    get count() {
        return J.isN(this.urls) ? 0 : this.urls.length;
    }
    /**
    * 水平方向否，反之垂直方向
    */
    get horizontal() {
        return this.direction == "left" || this.direction == "right";
    }

    //获取当前滚动到哪一张
    _getCurrentIndex() {

        var _t = this;

        var i = (_t.horizontal)
            ? Math.abs(_t.roller.left) / _t.width + 1
            : Math.abs(_t.roller.top) / _t.height + 1;

        return Math.round((_t.isDouble && i > _t.count) ? i - _t.count : i);

    };

    //创建滚动器，包含 roller btnNext btnPrev signContainer
    _createRoller() {
        //容器 
        //子元素：roller btnNext btnPrev signContainer
        var _t = this;

        //滚动的roller，移动的主体
        //子元素：box
        var count = _t.count * (_t.isDouble ? 2 : 1);

        //滚动容器
        var roller = (_t.horizontal)
            ? new HBox({
                name: "roller",
                width: _t.width * count,
                height: _t.height,
                top: 0,
                left: 0,
                z: 1,
                overflow: 'hidden'
            })
            : new VBox({
                name: "roller",
                width: _t.width,
                height: _t.height * count,
                top: 0,
                left: 0,
                z: 1
            });

        //标识容器
        var signContainer = new HBox({
            z: 9,
            width: (_t.signSize + 12) * _t.count,
            height: _t.signSize,
            centerX: 0,
            bottom: _t.signSize * 1.5,
            backgroundColor: "#00f"
        });

        var boxs = [];
        _t.urls.forEach((p, i) => {

            var box = new Box({
                width: _t.width,
                height: _t.height,
                top: -_t.height,
                left: _t.width * i,
                z: 2
            });

            var picture = new Picture({
                width: _t.width,
                height: _t.height,
                repeatTypeX: "stretch",
                repeatTypeY: "stretch",
                repeatX: 0,
                repeatY: 0,
                top: 0,
                left: 0,
                z: 3,
                url: _t.urls[i]
            });
            box.add(picture);
            box.picture = picture;

            var title = new Label({
                text: _t.titles[i],
                z: 4,
                color: _t.titleColor,
                width: _t.width,
                height: _t.titleHeight,
                bottom: _t.descHeight,
                font: _t.titleFont,
                left: 0
            });
            box.add(title);
            box.title = title;

            var desc = new Label({
                text: _t.descs[i],
                z: 5,
                color: _t.descColor,
                width: _t.width,
                height: _t.descHeight,
                font: _t.descFont,
                bottom: 0,
                left: 0
            });
            box.add(desc);
            box.desc = desc;

            boxs.push(box);
            roller.add(box);

            var sign = new Box({
                z: 10,
                isSign: true,
                width: _t.signSize,
                height: _t.signSize,
                margin: 6,
                backgroundColor: i == 0 ? "#ff0000" : "#006600",
                left: _t.signSize * i
            });
            signContainer.add(sign);

        });
        if (_t.isDouble) {
            //盒子复制一份
            boxs.forEach((b) => {

                var box = b.clone();
                b.children.forEach((c) => {
                    box.add(c.clone());
                });
                roller.add(box);
                box.left += this.width * _t.count;

            });
        }

        _t.roller = roller;
        _t.add(roller);

        _t.btnNext = new Button({
            z: 6,
            width: 40,
            height: 40,
            left: _t.horizontal ? 10 : (_t.width - 40) / 2,
            top: _t.horizontal ? (_t.height - 40) / 2 : 10,
            subType: "onlyIcon",
            icon: _t.nextIcon
        });
        _t.add(_t.btnNext);
        _t.btnPrev = new Button({
            z: 6,
            width: 40,
            height: 40,
            right: _t.horizontal ? 10 : (_t.width - 40) / 2,
            bottom: _t.horizontal ? (_t.height - 40) / 2 : 10,
            subType: "onlyIcon",
            icon: _t.prevIcon
        });
        _t.add(_t.btnPrev);

        _t.signContainer = signContainer;
        _t.add(signContainer);

        _t.signContainer.children.forEach((sign, i) => {
            sign.on(!_t.autoplay ? "over" : "click", () => {
                if (_t.horizontal)
                    _t._move("left", _t.roller.left, -_t.width * i);
                else
                    _t._move("top", _t.roller.top, -_t.height * i);

                console.log(i);
                return true;
            });
        });

        _t.btnNext.on("click", () => {
            _t.next();
        });
        _t.btnPrev.on("click", () => {
            _t.prev();
        });
        _t.btnNext.on("over", () => {
            return true;
        });
        _t.btnNext.on("over", () => {
            return true;
        });

    };
    /**
     * 修正小图标
     */
    focusSign(index) {

        var _t = this;

        _t.signContainer.children.forEach((sign, i) => {

            sign.material = new THREE.SpriteMaterial({ color: index == i + 1 ? "#ff0000" : "#006600" });
            sign.material.needsUpdate = true;

        });
    }
    /* 
     * 开始滚动
     */
    play(direction) {
        var _t = this;

        if (_t.enoughToStart) {
            if (!_t.horizontal && _t.roller.height <= _t.height)
                return;
            if (_t.horizontal && _t.roller.width <= _t.width)
                return;
        }

        switch (direction) {
            case "up":
                if (_t.roller.top + _t.roller.height <= _t.height) {
                    //如果已经滚到最顶部，则直接回到最底部
                    _t._move("top", _t.roller.top, 0);
                }
                else if (_t.roller.top + _t.roller.height - _t.height - _t.distance < 0) {
                    //如果向左滚动会使底部边界出现在容器中，则滚到最顶部即可
                    _t._move("top", _t.roller.top, _t.height - _t.roller.height);
                }
                else {
                    //其它情况正常滚动
                    _t._move("top", _t.roller.top, _t.roller.top - _t.distance);
                }
                break;

            case "down":
                if (_t.roller.top >= 0) {
                    //如果已经滚到最底部，则直接回到最顶部
                    _t._move("top", _t.roller.top, _t.height - _t.roller.height);
                }
                else if (_t.roller.top + _t.distance > 0) {
                    //如果向右滚动会使顶部边界出现在容器中，则滚到最底部即可
                    _t._move("top", _t.roller.top, 0);

                }
                else {
                    //其它情况正常滚动
                    _t._move("top", _t.roller.top, _t.roller.top + _t.distance);
                }
                break;
            case "left":
                if (_t.roller.left + _t.roller.width <= _t.width) {
                    //如果已经滚到最左边，则直接回到最右边
                    _t._move("left", _t.roller.left, 0);
                }
                else if (_t.roller.left + _t.roller.width - _t.width - _t.distance < 0) {
                    //如果向左滚动会使右边边界出现在容器中，则滚到最左边即可
                    _t._move("left", _t.roller.left, _t.width - _t.roller.width);
                }
                else {
                    //其它情况正常滚动
                    _t._move("left", _t.roller.left, _t.roller.left - _t.distance);
                }
                break;
            case "right":
                if (_t.roller.left >= 0) {
                    //如果已经滚到最右边，则直接回到最左边
                    _t._move("left", _t.roller.left, _t.width - _t.roller.width);
                }
                else if (_t.roller.left + _t.distance > 0) {
                    //如果向右滚动会使左边边界出现在容器中，则滚到最右边即可
                    _t._move("left", _t.roller.left, 0);

                }
                else {
                    //其它情况正常滚动
                    _t._move("left", _t.roller.left, _t.roller.left + _t.distance);
                }
                break;
        }
    };
    /* 
     * 开始移动
     */
    _move(direction, from, to) {
        var _t = this;

        if (_t._isMoving)
            return;
        _t._isMoving = true;

        J.tween({
            from: direction == "top" ? { top: from } : { left: from },
            to: direction == "top" ? { top: to } : { left: to },
            duration: _t.duration,
            easing: _t.easing,
            onUpdate: function (pos) {
                _t.roller[direction] = pos[direction];
                _t.layoutChildren();
            },
            onComplete: function () {
                _t._isMoving = false;
                var i = _t._getCurrentIndex();
                _t.focusSign(i);

                _t.fire('rolled', i);
            }
        }).start();
    };
    //上一张
    prev() {
        var _t = this;
        _t.play(_t.horizontal ? "right" : "down");
    };
    //下一张
    next() {
        var _t = this;
        _t.play(_t.horizontal ? "left" : "up");
    };
    /**
     * 初始化
     */
    onInited(args) {

        super.onInited(args);

        var _t = this;
        _t._createRoller();

        if (_t.autoplay) {

            _t._timerOfInterval = SM.interval((count) => {
                _t.play(_t.direction);
            }, _t._interval, _t.firstDelay).start();

        }
    }
    /**
     * 鼠标移上来，停止动画
     */
    onOver(et) {

        var _t = this;
        if (_t.autoplay && _t._timerOfInterval)
            _t._timerOfInterval.pause();

    };
    /**
     * 鼠标移开，继续动画
     */
    onOut(et) {

        var _t = this;
        if (_t.autoplay && _t._timerOfInterval)
            _t._timerOfInterval.resume();

    };

};

G.PictureRoller = PictureRoller;
/**
 * PictureViewer，图片轮播
 */
export class PictureViewer extends Box {

    constructor(options) {
        super(options);

        this.isPictureViewer = true;
        this.gtype = 'PictureViewer';
        this.overflow = 'scroll';

        /**
         * 充满全屏
         */
        this.z = 30;

        this.width = "100%";
        this.height = "100%";

        this.top = 0;
        this.left = 0;
        this.right = 0;
        this.bottom = 0;

        this.centerX = 0;
        this.centerY = 0;

        /**
         * 背景色
         */
        this._backgroundColor = J.def(this.options.backgroundColor, "#666666ee");
        /**
         * 动画效果
         */
        this._easing = J.def(this.options.easing, TWEEN.Easing.Sinusoidal.InOut);
        /**
         * 图片集
         * 即可是真实地址，也可是纹理的 uuid
         */
        this._urls = J.def(this.options.urls, []);
        /**
         * 标识大小
         */
        this._signSize = J.def(this.options.signSize, 12);
        /**
         * 切换时长，毫秒
         */
        this._duration = J.def(this.options.duration, 500);

        /**
         * 正在移动
         */
        this._isMoving = false;
    }

    /**
    * 序号
    */
    get index() {
        return this._index;
    }
    set index(v) {
        this._index = v;
        this.change('index');
    }

    /**
    * 动画效果
    */
    get easing() {
        return this._easing;
    }
    set easing(v) {
        this._easing = v;
        this.change('easing');
    }
    /**
    * 切换时长
    */
    get duration() {
        return this._duration;
    }
    set duration(v) {
        this._duration = v;
        this.change('duration');
    }

    /**
    * 标识大小
    */
    get signSize() {
        return this._signSize;
    }
    set signSize(v) {
        this._signSize = v;
        this.change('signSize');
    }
    /**
    * 图片集
    */
    get urls() {
        return this._urls;
    }
    set urls(v) {
        this._urls = v;
        this.change('urls');
    }

    /**
    * 数量
    */
    get count() {
        return J.isN(this.urls) ? 0 : this.urls.length;
    }

    //获取当前滚动到哪一张
    _getCurrentIndex() {

        var _t = this;
        return Math.abs(_t.viewer.left) / _t.width + 1;

    };


    //创建预览器，包含 viewer signContainer
    _createViewer() {
        //容器 
        //子元素：viewer signContainer
        var _t = this;

        //滚动的viewer，移动的主体

        //滚动容器
        var viewer = new HBox({
            name: "viewer",
            width: _t.width * _t.count,
            height: _t.height,
            top: 0,
            left: 0,
            z: _t.z + 1,
            overflow: 'hidden'
        });
        _t.viewer = viewer;
        _t.add(viewer);

        _t.urls.forEach((p, i) => {

            //子元素：box
            var box = new Box({
                width: _t.width,
                height: _t.height,
                top: 0,
                left: _t.width * i,
                z: _t.z + 2,
            });
            viewer.add(box);

            var picture = new Picture({
                width: _t.width,
                height: _t.height,
                repeatTypeX: "stretch",
                repeatTypeY: "stretch",
                repeatX: 0,
                repeatY: 0,
                centerX: 0,
                centerY: 0,
                z: _t.z + 3,
                url: _t.urls[i]
            });
            box.add(picture);
            box.picture = picture;
            //修正图片的宽高
            picture.onBackgroundImageLoaded = function () {

                //保证整张图不变形的显示在屏幕内
                picture.height = picture.backgroundImageHeight * _t.width / picture.backgroundImageWidth;
                if (picture.height > _t.height) {
                    picture.height = _t.height;
                    picture.width = picture.backgroundImageWidth * _t.height / picture.backgroundImageHeight;
                }
                _t.layoutChildren();

            };
        });

        //标识容器
        var signContainer = new HBox({
            z: _t.z + 24,
            width: (_t.signSize + 12) * _t.count,
            height: _t.signSize,
            centerX: 0,
            bottom: _t.signSize,
            backgroundColor: "#00f"
        });
        _t.signContainer = signContainer;
        _t.add(signContainer);

        _t.urls.forEach((p, i) => {
            var sign = new Box({
                z: _t.z + 25,
                isSign: true,
                width: _t.signSize,
                height: _t.signSize,
                margin: 6,
                backgroundColor: i == 0 ? "#ff0000" : "#006600",
                left: _t.signSize * i
            });
            signContainer.add(sign);
        });

        _t.signContainer.children.forEach((sign, i) => {
            sign.on("click", () => {
                _t.play(i);
                return true;
            });
        });

    };
    /**
     * 修正小图标
     */
    focusSign(index) {

        var _t = this;

        _t.signContainer.children.forEach((sign, i) => {

            sign.material = new THREE.SpriteMaterial({ color: index == i + 1 ? "#ff0000" : "#006600" });
            sign.material.needsUpdate = true;

        });
    }
    /* 
     * 开始滚动
     */
    play(index) {
        var _t = this;

        if (_t._isMoving)
            return;
        _t._isMoving = true;

        J.tween({
            from: { left: _t.viewer.left },
            to: { left: -_t.width * index },
            duration: _t.duration,
            easing: _t.easing,
            onUpdate: function (pos) {
                _t.viewer.left = pos.left;
                _t.layoutChildren();
            },
            onComplete: function () {
                _t._isMoving = false;
                var i = _t._getCurrentIndex();
                _t.focusSign(i);

                _t.fire('rolled', i);
            }
        }).start();
    };
    /**
     * 初始化
     */
    onInited(args) {

        super.onInited(args);

        var _t = this;
        _t._createViewer();

    }
};

G.PictureViewer = PictureViewer;
/**
 * 进度条(复合控件)
 */
class ProgressBar extends Box {

	constructor(options) {
		super(options);

		this.isProgressBar = true;
		this.gtype = 'ProgressBar';

		this.overflow = 'hidden';

		this.max = J.def(this.options.max, 100);
		this.min = J.def(this.options.min, 0);

		this._value = J.def(this.options.value, 0);

		this.width = J.def(this.options.width, 100);
		this.height = J.def(this.options.height, 10);

		this.showLabel = J.def(this.options.showLabel, false);

		this._backgroundColor = J.def(this.options.backgroundColor, "#ffffff");

		this.barBackgroundColor = J.def(this.options.barBackgroundColor, "#006600");

		this.label = null;
		this.bar = null;

		var _t = this;

		_t.bar = new Box({
			backgroundColor: _t.barBackgroundColor,
			z: 3,
			width: _t.width,
			height: _t.height,
			opacity: 0.9
		});

		_t.label = new Label({
			text: '0%',
			z: 4,
			width: _t.width,
			height: _t.height
		});

	}

	onInited(args) {
		super.onInited(args);

		var _t = this;
		_t.add(_t.bar);
		_t.add(_t.label);

		_t.label.visible = _t.showLabel;
	}

	get value() {
		return this._value;
	}

	set value(v) {

		if (this._value == v)
			return;

		var _t = this;

		if (v > _t.max)
			v = _t.max;
		if (v < _t.min)
			v = _t.min;
		_t._value = v;

		_t.change('value');

		if (!_t.label || !_t.bar)
			return;

		//+1 是因为 sprite 宽度不能为 0，或者说 scale 不能为 0，
		//会造成 THREE.Matrix3: .getInverse() can't invert matrix, determinant is 0 错误
		var v2 = _t.value / (_t.max - _t.min);
		_t.bar.width = v2 * _t.width + 1;
		_t.label.text = v2 * 100 + '%';
	}

}

G.ProgressBar = ProgressBar;
/**
 * 输入框
 */
export class TextInput extends Box {

    constructor(options) {

        if (J.isN(options)) options = {};
        options.skin = J.def(options.skin, 'textinputSkin');

        super(options);

        var _t = this;

        this.isTextInput = true;
        this.gtype = "TextInput";

        this._text = J.def(this.options.text, '');
        this.label = new Label({
            text: this._text,
            centerY: 0,
            left: 5,
            z: 1
        });

        this.cursor = new Line({
            direction: "vertical",
            backgroundColor: '#000',
            z: 2,
            left: 5,
            centerY: 0,
            height: 20
        });

        this.cursorTW = this.tween({
            from: { opacity: 0 },
            to: { opacity: 1 },
            repeat: Infinity,
            //光标跳动速度
            duration: 0.83 * 1000,
            onUpdate: (data) => {
                _t.cursor.material.opacity = Math.round(data.opacity);
                _t.cursor.material.needsUpdate = true;
            }
        });

        this.isCreated = false;

        //兼容小游戏
        if (J.udt.isMiniG) {
            wx.onKeyboardConfirm(function (text) {
                this.text = text.value;
            });
        } else {
            //布局完成后再设定，不然 x/y 拿不到
            SM.on('afterLayoutUI', () => {

                //只做一次
                if (this.isCreated) return;
                this.isCreated = true;

                this.textinput = $.create('input', { type: "text" });
                $('body').append(this.textinput);
                this.textinput.css({
                    left: this.x,
                    top: this.y,
                    width: this.width,
                    height: this.height,
                    position: 'absolute',
                    zIndex: 999,
                    border: 0,
                    margin: 0,
                    padding: 5,
                    boxSizing: 'border-box'
                }).on('blur', () => {
                    this.textinput.hide();
                    this.text = this.textinput.val();
                }).hide();

            }, this);
        }
    }

    get text() {
        this.textinput.blur();
        return this._text;
    }
    set text(v) {
        if (this._text == v) return;
        this._text = v;
        this.label.text = v;
        this.layout();
        this.change('text');
    }

    onInited(args) {
        super.onInited(args);
        this.add(this.label)
        this.add(this.cursor);
        this.cursor.hide();
    }

    onFocus(et) {

        super.onFocus(et);
        this.cursor.show();
        this.cursorTW.start();

        //兼容小游戏
        if (J.udt.isMiniG) {
            wx.showKeyboard({
                confirmType: 'send'
            });
        } else {
            this.textinput.show();
            this.textinput.focus();
            this.textinput.val(this._text);
        }
    }

    onBlur(et) {

        super.onBlur(et);
        this.cursor.hide();
        this.cursorTW.stop();

        //兼容小游戏
        if (J.udt.isMiniG) {
            wx.hideKeyboard();
        }
    }
}

G.TextInput = TextInput;

/**
 * 视频播放器
 */
export class VideoPlayer extends Box {

    constructor(options) {
        super(options);

        this.isVideoPlayer = true;
        this.gtype = 'VideoPlayer';

        this.overflow = 'hidden';

        this._backgroundColor = J.def(this.options.backgroundColor, "#ffffff");

        /**
         * 名称
         * @type {string}
         */
        this._name = J.def(this.options.name, "");

        this.width = J.def(this.options.width, 300);
        this.height = J.def(this.options.height, 200);
        /**
         * 自动播放
         * @type {bool}
         */
        this._autoplay = J.def(this.options.autoplay, true);
        /**
         * 是否循环播放
         */
        this._loop = J.def(this.options.loop, false);
        /**
         * 声音大小
         */
        this._volume = J.def(this.options.volume, 0.5);
        /**
         * 地址
         */
        this._url = J.def(this.options.url, "");
        /**
        * 视频信息对象
        */
        this._videoInfo = null;

    }

    /**
    * 自动播放
    */
    get autoplay() {
        return this._autoplay;
    }
    set autoplay(v) {
        this._autoplay = v;
        this.change('autoplay');
    }

    /**
    * 视频信息对象
    */
    get videoInfo() {
        return this._videoInfo;
    }
    set videoInfo(v) {
        var _t = this;

        _t._videoInfo = v;
        _t.videoInfo.attach(_t);

        this.change('videoInfo');
    }

    /**
    * 循环播放否
    */
    get loop() {
        return this._loop;
    }
    set loop(v) {
        this._loop = v;
        if (this.video)
            this.video.setLoop(v);
        this.change('loop');
    }

    /**
    * 音量
    */
    get volume() {
        return this._volume;
    }
    set volume(v) {
        this._volume = v;
        if (this.video)
            this.video.setVolume(v);
        this.change('volume')
    }

    /**
    * 图片地址，可以是uuid
    * @type {string}
    */
    get url() { return this._url; }
    set url(v) {
        var _t = this;
        if (_t._url == v)
            return;

        _t._url = v;

        if (!J.isN(_t.url))
            _t._createVideoInfo();

        this.videoInfo.attach(this);
        this.change('url');
    }

    /**
    * 视频对象
    */
    get video() {
        return this.videoInfo.video;
    }
    /**
    * 正在播放否
    */
    get isPlaying() {
        return this.videoInfo.isPlaying;
    }
    /**
     * 播放
     */
    play() {
        this.videoInfo.play();
    }
    /**
     * 重播
     */
    replay() {
        this.videoInfo.replay();
    }
    /**
     * 暂停
     */
    pause() {
        this.videoInfo.pause();
    }
    /**
     * 停止
     */
    stop() {
        this.videoInfo.stop();
    }
    /**
     * 创建按钮
     */
    _createPlayButton() {

        var _t = this;

        _t.btnPlay = new Button({
            z: 1,
            width: 40,
            height: 40,
            left: 10,
            bottom: 10,
            subType: "onlyIcon",
            icon: "/libs/assets/play.png"
        });

        _t.add(_t.btnPlay);
        _t.btnPlay.visible = !_t.autoplay;
        _t.btnPlay.children[0].visible = !_t.autoplay;

        _t.btnPlay.on("click", () => {
            if (!_t.isPlaying) {
                _t.play();
                _t.btnPlay.visible = false;
                _t.btnPlay.imgIcon.visible = false;
            }
            return false;
        });
    }

    /**
     * 创建视频信息对象
     */
    _createVideoInfo() {

        var _t = this;

        _t.videoInfo = J.us.isUUID(_t.url)
            ? RM.getItem('video', 'uuid', _t.url)
            : new VideoInfo({ url: _t.url });

        _t.video.loop = _t.loop;
        _t.video.volume = _t.volume;

    }
    /**
     * 初始化
     */
    onInited(args) {

        super.onInited(args);

        var _t = this;

        if (!J.isN(this.url)) {

            _t._createVideoInfo();

            if (_t.autoplay)
                _t.play();
        }

        _t._createPlayButton();
    }
    // 点击视频
    onClick(et) {

        var _t = this;
        // 如果视频正在播放，视频暂停，播放按钮显示
        if (_t.isPlaying) {

            _t.pause();
            _t.btnPlay.imgIcon.visible = true;

        }
    }
}

G.VideoPlayer = VideoPlayer;
/**
 * 等待提示
 */
export class Waitting extends Box {

    constructor(options) {

        super(options);
        var _t = this;

        this.isWaitting = true;
        this.gtype = 'Waitting';

        this.overflow = 'hidden';

        this.width = J.def(this.options.width, 80);
        this.height = J.def(this.options.height, 80);

        this._z = 99;
        this._backgroundColor = J.def(this.options.backgroundColor, "#ffffff");
        this._opacity = 0.5;
        this._left = this._right = this._top = this._bottom = 0;

        this.label = new Label({
            text: '请稍后...',
            color: '#ddd',
            centerX: 0,
            centerY: 0,
        });

        this.icon = new Picture({
            width: 48,
            height: 48,
            centerX: 0,
            centerY: 34,
            url: "/libs/assets/spinner.png"
        });

        this.myTween = this.tween({
            duration: 1000,
            repeat: Infinity,
            from: { angle: 0 },
            to: { angle: 2 * Math.PI },
            onUpdate: function (data) {
                //注意：sprite 的旋转是通过 material.rotation 来实现的
                _t.icon.material.rotation = -data.angle;
            }
        });

    }

    onInited(args) {
        super.onInited(args);

        this.add(this.icon);
        this.add(this.label);
    }

    show() {
        super.show();
        this.myTween.start();
    }

    hide() {
        super.hide();
        this.myTween.stop();
    }

}

G.Waitting = Waitting;
/**
 * 管理器基类。
 * @class 
 * @param {object} options 
 */
export class Manager extends FrameObject {
    constructor(options) {
        super(options);

        this.isManager = true;
        this.gtype = 'Manager';
    }

}

G.Manager = Manager;
/**
 * 地图管理器。
 * 加载地图，并引入地图中 scripts 块，进行真正的初始化。
 * 保存了 game.json 所有成员信息，
 * 其他管理器是通过成员信息形成真正的对象，
 * 比如通过舞台信息生成 THREE.Scene 对象。
 * 所有信息均采用 uuid 作为唯一标识，在游戏中共享。
 * @param {object} options
 */
export class MapManager extends Manager {

    constructor(options) {
        super(options);

        window.MM = this;

        this.gobjectScripts = [];

        /**
         * 地图地址
         * @type {string}
         */
        this.url = J.def(this.options.url, "");

    }

    /**
     * 初始化，加载地图及管理器初始化
     */
    init(args) {

        var _t = this;

        new Loader().load(_t.url, (m) => {

            //将属性都置为 MM 的成员，用起来方便
            Object.assign(_t, m);

            //如果指定了，那就按指定的来，覆盖 game.json 里面的
            if (!J.isN(_t.options.domain))
                _t.domain = _t.options.domain;

            //如果两边都没有指定，则以 url 前面作为域名。因为所有资源都是 / 开头
            if (J.isN(_t.domain))
                _t.domain = _t.url.substr(0, _t.url.lastIndexOf('/'));

            _t._requireScripts(() => {

                super.init(args);
                GM.init();

            });

        });
    }

    /**
     * 保存地图
     */
    save() { }

    /**
     * 附加脚本
     */
    _requireScripts(onComplete) {
        var _t = this;

        var _SM = new SceneManager();
        var _GM = new GameManager();
        var _LGM = new LogicManager();
        var _CPM = new ChapterManager();
        var _RM = new ResourceManager();

        var len = _t.scripts.length;
        var loaded = 0;

        //importThen 是异步的，必须等待
        var _onComplete = function () {

            loaded++;
            if (loaded == len) {
                _GM.sceneManager = _SM;
                _GM.logicManager = _LGM;
                _GM.resourceManager = _RM;
                _GM.chapterManager = _CPM;
                onComplete();
            }

        };

        if (len == 0) {
            _onComplete();
        }

        for (var i = 0; i < len; i++) {
            var s = _t.scripts[i];
            s.url = _t.domain + s.url;
            switch (s.type) {

                case "GObject":
                    J.importThen(s.url, (st) => {
                        _t.gobjectScripts.push({
                            target: s.target,
                            clazz: st.default
                        });
                        _onComplete();
                    });

                    break;

                case "SceneManager":
                    J.importThen(s.url, (clazz) => {
                        threejsUtils.mixin(_SM, new clazz.default());
                        _onComplete();
                    });

                    break;

                case "ResourceManager":
                    J.importThen(s.url, (clazz) => {
                        threejsUtils.mixin(_RM, new clazz.default());
                        _onComplete();
                    });

                    break;

                case "LogicManager":
                    J.importThen(s.url, (clazz) => {
                        threejsUtils.mixin(_LGM, new clazz.default());
                        _onComplete();
                    });

                    break;

                case "ChapterManager":
                    J.importThen(s.url, (clazz) => {
                        threejsUtils.mixin(_CPM, new clazz.default());
                        _onComplete();
                    });

                    break;

                case 'GameManager':
                    J.importThen(s.url, (clazz) => {
                        threejsUtils.mixin(_GM, new clazz.default());
                        _onComplete();
                    });

                    break;
            }
        }


    }

}

G.MapManager = MapManager;

/**
 * 游戏总控。
 * 下设多个管理器，每个持有自己所需关键对象。
 * 比如 mapManager 持有 map，不交叉持有。
 * @param {object} options
 */
let instance;

export class GameManager extends Manager {

	constructor(options) {
		super(options);

		if (instance)
			return instance;

		instance = this;

		//全局唯一实例
		window.GM = this;

		/**
		 * 游戏状态
		 * @type {String}
		 */
		this._status = 'loading';

		/**
		 * 游戏开始时间
		 * @type {Date}
		 */
		this.startTime = new Date();

		/**
		 * 当前已运行时长（秒）
		 * @type {Number}
		 */
		this.runTime = 0;

		/**
		 * 游戏结束时间
		 * @type {Date}
		 */
		this.endTime = new Date();

		this.sceneManager = this.options.sceneManager;
		this.logicManager = this.options.logicManager;
		this.chapterManager = this.options.chapterManager;
		this.resourceManager = this.options.resourceManager;

		/**
		 * 命令模式。
		 * 命令集。
		 */
		this.commands = [];

	}


	/**
	 * 仅为本客户端做命令派发。
	 * 如果是多人联机的，则要通过 WSocketManager 来管理，
	 * 管理后的结果再发给 GM。
	 * GM 仅管当前游戏，不管联机的其他客户端。
	 *
	 * @param  {type} cmd description
	 * @return {type}     description
	 */
	set command(cmd) {
		this.commands.push(cmd);
		cmd.owner.command = cmd;
	}

	init(args) {

		super.init(args);

		var _t = this;

		_t.resourceManager.on("inited", function () {
			_t.chapterManager.init();
		});

		_t.chapterManager.on('inited', function () {
			_t.sceneManager.init();
		});

		_t.sceneManager.on("inited", function () {
			_t.logicManager.init();
		});

		_t.resourceManager.init();

	}

	get status() {
		return this._status;
	}

	set status(v) {
		this._status = v;
		this.change('status');
	}

	start() {
		this.status = 'start';
		this.fire("start");
		this.startTime = new Date();
		this.runTime = 0;

		//开始游戏，进行一次全体布局
		SM.layoutUI();

	}

	pause() {
		this.status = 'pause';
		this.fire("pause");
	}

	over() {

		this.endTime = new Date();

		this.status = 'over';
		this.fire("over");
	}

	onUpdated(args) {

		super.onUpdated(args);

		this.runTime = new Date() - this.startTime;

	}

}

G.GameManager = GameManager;
/**
 * 章节管理器。
 * @class
 * @param {object} options
 */
export class ChapterManager extends Manager {

	constructor(options) {
		super(options);

		window.CPM = this;

		/**
		 * 当前章节
		 * @type {object}
		 */
		this.chapter = null;

		/**
		 * 当前舞台信息。
		 * 有时候换章节并不一定换舞台。
		 */
		this.scene = null;

		/**
		 * 全部章节
		 */
		this.chapters = [];

		/**
		 * 全部舞台
		 */
		this.scenes = [];

	}

	init(args) {

		this.chapters = MM.chapters;
		this.scenes = MM.scenes;

		MM.chapters.forEach((chapter) => {
			if (chapter.uuid == MM.chapter)
				this.chapter = chapter;
		});

		MM.scenes.forEach((scene) => {
			if (scene.uuid == this.chapter.scene)
				this.scene = scene;
		});

		//加载资源组
		if (!J.isNull(this.chapter.group)) {
			RM.loadGroup('uuid', this.chapter.group, (type, count, total) => {
				console.log(" [" + type + "] was loaded. " + count + "/" + total);
			}, () => {
				super.init(args);
			});
		} else {
			super.init(args);
		}
	}

	/**
	 * 遍历章节
	 * @param {Chapter} p  父章节
	 * @param {function} callback  回调
	 */
	traverse(p, callback) {

		if (p.children && p.children.length > 0) {
			for (let i = 0; i < p.children.length; i++) {
				const child = p.children[i];
				callback.call(child);
				this.traverse(child);
			}
		}
	}

	hasNext() { }

	/**
	 * 下一个章节
	 */
	next() {

	}

	/**
	 * 转到哪个章节
	 * @param {string} uuid
	 */
	goto(uuid) {

	}
}

G.ChapterManager = ChapterManager;
/**
 * 一个游戏中所有资源的管理
 */
export class ResourceManager extends Manager {

    constructor(options) {
        super(options);

        window.RM = this;

        /**
         * 资源分组。
         * 不同场景下或者不同章节下，有时候需要一次性加载指定的一组资源。
         */
        this.groups = [];

        /**
         * 当前分组需加载的总数量
         */
        this.total = 0;
        /**
         * 当前分组已加载的数量
         */
        this.loaded = 0;

        this.cameras = [];
        this.lights = [];

        /**
        * 声音资源，元素由名字、src、对象（如Audio对象）构成
        */
        this.fonts = [];
        this.audios = [];
        this.videos = [];
        this.images = [];
        /**
         * 对象缓冲。
         * 原型模式。
         */
        this.uiobjects = [];
        this.textures = [];
        this.materials = [];
        this.geometries = [];

        this.units = [];
        this.gameobjects = [];
    }

    /**
     * 修复 url 路径。
     * urls 可以是数组，数组中的元素可以是字符串也可以是对象，但对象一定要有 url 成员。
     * urls 可以是字符串。
     * urls 可以是对象，但对象一定要有 url 成员。
     * @param {array|string|object} urls 
     */
    fixUrl(urls) {

        var _fixUrl = function (url) {
            if (J.us.startWith(url, 'http://') ||
                J.us.startWith(url, 'https://')
                || J.us.startWith(url, 'file://')) {
                return url
            }
            else
                return MM.domain + url;
        };

        var _fixUrl2 = function (v) {
            if (typeof (v) == 'object') {
                if (v['url']) {
                    v.url = _fixUrl(v.url);
                }
            } else {
                v = _fixUrl(v);
            }
        }

        if (Array.isArray(urls)) {
            urls.forEach((v) => {
                _fixUrl2(v);
            })
        } else {
            _fixUrl2(urls);
        }
    }

    /*
        资源初始化
    */
    init(args) {

        var loader = new THREE.ObjectLoader();

        this.fixUrl(MM.fonts);
        this.fixUrl(MM.audios);
        this.fixUrl(MM.videos);
        this.fixUrl(MM.images);
        this.fixUrl(MM.units);

        //摄像头
        if (MM.cameras)
            MM.cameras.forEach((v) => { this.cameras.push(loader.parseObject(v)) });
        //灯光
        if (MM.lights)
            MM.lights.forEach((v) => { this.lights.push(loader.parseObject(v)) });

        super.init(args);
    }

    /*
     * @param { string } type 资源类型
     * @param { string } key uuid
     * @param { string } value uuid值
     * @param { function } onCompleted 加载完成后执行的函数
    */
    loadItem(type, key, value, onCompleted) {

        var _t = this;

        switch (type) {

            case "font":
                const font = J.ua.find(MM.fonts, key, value)[0];
                if (font == null) {
                    console.error("Can't find the font with uuid:" + value);
                } else {
                    var loader = new THREE.FontLoader();

                    loader.load(
                        font.url,
                        function (f) {
                            _t.fonts.push(f);
                            if (onCompleted)
                                onCompleted();
                        }
                    );
                }
                break;
            case "audio":
                if (J.udt.isMiniP) {
                    onCompleted();
                } else {
                    const audio = J.ua.find(MM.audios, key, value)[0];
                    if (audio == null) {
                        console.error("Can't find the audio with uuid:" + value);
                    } else {

                        var a = new AudioInfo(audio);
                        _t.audios.push(a);
                        a.load(a.url, () => {
                            if (onCompleted)
                                onCompleted();
                        });
                    }
                    break;
                }
            case "video":
                if (J.udt.isMiniP) {
                    onCompleted();
                } else {
                    const video = J.ua.find(MM.videos, key, value)[0];
                    if (video == null) {
                        console.error("Can't find the video with uuid:" + value);
                    } else {
                        var v = new VideoInfo(video);
                        _t.videos.push(v);
                        if (onCompleted)
                            onCompleted();
                    }
                }
                break;

            case "image":
                const img = J.ua.find(MM.images, key, value)[0];
                if (img == null) {
                    console.error("Can't find the image with uuid:" + value);
                } else {
                    var image = new THREE.ObjectLoader().parseImages([img])[value];
                    image.uuid = value;
                    _t.images.push(image);

                    if (onCompleted)
                        onCompleted();
                }
                break;
            case "texture":
                const texture = J.ua.find(MM.textures, key, value)[0];
                if (texture == null) {
                    console.error("Can't find the texture with uuid:" + value);
                } else {
                    _t.textures.push(new THREE.ObjectLoader().parseTextures([texture], threejsUtils.arrayToHash(_t.images))[value]);
                    if (onCompleted)
                        onCompleted();
                }
                break;
            case "material":
                const material = J.ua.find(MM.materials, key, value)[0];
                if (material == null) {
                    console.error("Can't find the material with uuid:" + value);
                } else {
                    _t.materials.push(new THREE.ObjectLoader().parseMaterials([material], threejsUtils.arrayToHash(_t.textures))[value]);
                    if (onCompleted)
                        onCompleted();
                }
                break;
            case "geometry":
                const geometry = J.ua.find(MM.geometries, key, value)[0];
                if (geometry == null) {
                    console.error("Can't find the geometry with uuid:" + value);
                } else {
                    _t.geometries.push(new THREE.ObjectLoader().parseGeometries([geometry])[value]);
                    if (onCompleted)
                        onCompleted();
                }
                break;
            case 'unit':

                //已经有，就不再加载
                if (_t.getItem('unit', key, value)) {
                    if (onCompleted)
                        onCompleted();
                    return;
                }

                var unit = J.ua.find(MM.units, key, value)[0];

                new THREE.ObjectLoader().parseObjectEx(
                    unit,
                    threejsUtils.arrayToHash(_t.geometries),
                    threejsUtils.arrayToHash(_t.materials),
                    function (obj) {

                        //压入缓冲
                        _t.units.push(obj);
                        if (onCompleted)
                            onCompleted();

                    }
                );

                break;
            case "gameobject":

                const gobj = J.ua.find(MM.gameobjects, key, value)[0];
                if (gobj == null) {
                    console.error("Can't find the gameObject with uuid:" + value);
                } else {

                    var unit = _t.getItem('unit', 'uuid', gobj.unit);
                    gobj.unit = unit;

                    var gobj2 = new GameObject(gobj);
                    gobj2.ouuid = gobj.uuid;

                    //这里并没有调用 assignWith 是不太对的
                    this.fillScript(gobj2);

                    _t.gameobjects.push(gobj2.attachToUnit());

                    if (onCompleted)
                        onCompleted();
                }
                break;

            case "uiobject":
                const uiobject = J.ua.find(MM.uiobjects, key, value)[0];
                if (uiobject == null) {
                    console.error("Can't find the box with uuid:" + value);
                } else {
                    var uiobj = new window[uiobject.gtype](uiobject);
                    _t.uiobjects.push(uiobj);

                    if (onCompleted)
                        onCompleted();
                }
                break;
        }
    }

    /**
     * 从 game.json 找到本对象附加的脚本
     */
    fillScript(gobj3) {

        var ss = J.ua.find(MM.gobjectScripts, "target", gobj3.ouuid);
        if (ss && ss.length > 0) {
            for (var j = 0; j < ss.length; j++) {
                var s = ss[j];
                threejsUtils.mixin(gobj3, new s.clazz.default());
            }
        }
    }

    /*
     * 加载资源集
     * @param { array } items 资源 uuid 集
     * @param { string } type 资源类型
     * @param { string } key uuid
     * @param { string } value uuid 值
     * @param { function } onCompleted 加载完成后执行的函数
    */
    loadItems(items, type, key, onProgress, onCompleted) {

        var _t = this;

        var len = items.length;
        var cnts = 0;
        if (len == 0) {

            onProgress(type, _t.loaded, _t.total);
            onCompleted();

        } else {
            for (let i = 0; i < len; i++) {

                const item = items[i];

                _t.loadItem(type, key, item, () => {
                    cnts++;
                    _t.loaded++;
                    onProgress(type, _t.loaded, _t.total);
                    if (cnts == len) {
                        onCompleted();
                    }
                });
            }
        }

    }

    /*
     * 加载资源集
     * @param { string } key uuid
     * @param { string } value uuid 值
     * @param { function } onProgress 加载完每一种资源后执行的函数
     * @param { function } onCompleted 加载完成后执行的函数
    */
    loadGroup(key, value, onProgress, onCompleted) {

        var _t = this;

        var g = J.ua.find(MM.groups, key, value)[0];

        var _onProgress = (type, loaded, total) => {
            _t.fire('progress', type, loaded, total);
            onProgress(type, _t.loaded, _t.total);
        };

        var _onCompleted = () => {
            _t.fire('completed');
            onCompleted();
        }

        if (J.isNull(g)) {
            _onCompleted();
            return;
        }

        g.fonts = J.isNull(g.fonts) ? [] : g.fonts;
        g.audios = J.isNull(g.audios) ? [] : g.audios;
        g.videos = J.isNull(g.videos) ? [] : g.videos;
        g.images = J.isNull(g.images) ? [] : g.images;

        g.textures = J.isNull(g.textures) ? [] : g.textures;
        g.uiobjects = J.isNull(g.uiobjects) ? [] : g.uiobjects;
        g.materials = J.isNull(g.materials) ? [] : g.materials;
        g.geometries = J.isNull(g.geometries) ? [] : g.geometries;

        g.units = J.isNull(g.units) ? [] : g.units;
        g.gameobjects = J.isNull(g.gameobjects) ? [] : g.gameobjects;

        var lenf = g.fonts.length;
        var lena = g.audios.length;
        var lenv = g.videos.length;
        var leni = g.images.length;

        var lent = g.textures.length;
        var lenui = g.uiobjects.length;
        var lenm = g.materials.length;
        var leng = g.geometries.length;

        var lenu = g.units.length;
        var lengo = g.gameobjects.length;

        //所有的要加载的东西都要计入
        _t.total = lenf + lena + lenv + leni
            + lent + lenui + lenm + leng
            + lenu + lengo;
        _t.loaded = 0;

        var async = new Async(_t.loadItems,
            [
                [g.fonts, 'font', key, _onProgress, () => { async.next(); }],
                [g.audios, 'audio', key, _onProgress, () => { async.next(); }],
                [g.videos, 'video', key, _onProgress, () => { async.next(); }],
                [g.images, 'image', key, _onProgress, () => { async.next(); }],
                [g.textures, 'texture', key, _onProgress, () => { async.next(); }],
                [g.uiobjects, 'uiobject', key, _onProgress, () => { async.next(); }],
                [g.materials, 'material', key, _onProgress, () => { async.next(); }],
                [g.geometries, 'geometry', key, _onProgress, () => { async.next(); }],
                [g.units, 'unit', key, _onProgress, () => { async.next(); }],
                [g.gameobjects, 'gameobject', key, _onProgress, () => { _onCompleted(); }],

            ],
            this
        );

        async.first();

    }

    /*
     * 卸载资源集
     * @param { string } key uuid
     * @param { string } value uuid 值
     * @param { function } onProgress 加载完每一种资源后执行的函数
     * @param { function } onCompleted 加载完成后执行的函数
    */
    unLoadGroup() { }

    /*
     * 加载资源集
     * @param { string } 类型 uiobject、audio 等 12 种元素
     * @param { string } key 键
     * @param { string } value 键值
    */
    getItem(type, key, value) {
        switch (type) {

            case 'font':
                for (var i = 0; i < this.fonts.length; i++) {
                    if (this.fonts[i][key] == value)
                        return this.fonts[i];
                }
                break;

            case 'audio':
                for (var i = 0; i < this.audios.length; i++) {
                    if (this.audios[i][key] == value)
                        return this.audios[i];
                }
                break;

            case 'video':
                for (var i = 0; i < this.videos.length; i++) {
                    if (this.videos[i][key] == value)
                        return this.videos[i];
                }
                break;

            case 'image':
                for (var i = 0; i < this.images.length; i++) {
                    if (this.images[i][key] == value)
                        return this.images[i];
                }
                break;

            case 'texture':
                for (var i = 0; i < this.textures.length; i++) {
                    if (this.textures[i][key] == value)
                        return this.textures[i];
                }
                break;

            case 'uiobject':
                for (var i = 0; i < this.uiobjects.length; i++) {
                    if (this.uiobjects[i][key] == value)
                        return this.uiobjects[i];
                }
                break;

            case 'material':
                for (var i = 0; i < this.materials.length; i++) {
                    if (this.materials[i][key] == value)
                        return this.materials[i];
                }
                break;

            case 'geometry':
                for (var i = 0; i < this.geometries.length; i++) {
                    if (this.geometries[i][key] == value)
                        return this.geometries[i];
                }
                break;

            case 'unit':
                for (var i = 0; i < this.units.length; i++) {
                    if (this.units[i][key] == value)
                        return this.units[i];
                }
                break;

            case 'gameobject':
                for (var i = 0; i < this.gameobjects.length; i++) {
                    if (this.gameobjects[i][key] == value)
                        return this.gameobjects[i];
                }
                break;

            case 'camera':
                for (var i = 0; i < this.cameras.length; i++) {
                    if (this.cameras[i][key] == value)
                        return this.cameras[i];
                }
                break;

            case 'light':
                for (var i = 0; i < this.lights.length; i++) {
                    if (this.lights[i][key] == value)
                        return this.lights[i];
                }
                break;
        }
        return null;
    }

}

G.ResourceManager = ResourceManager;
/**
 * 当前场景管理器。
 * 多场景切换由 GM 来控制，本管理器只管理一个场景。
 * @param {object} options
 */
export class SceneManager extends Manager {

    constructor(options) {

        super(options);

        window.SM = this;

        this.wWidth = window.innerWidth;
        this.wHeight = window.innerHeight;
        this.aspect = this.wWidth / this.wHeight;

        // UI 界面使用
        this.uiScene = new THREE.Scene();
        this.uiCamera = new THREE.OrthographicCamera(
            -this.wWidth / 2, this.wWidth / 2,
            this.wHeight / 2, -this.wHeight / 2,
            1, 100
        );

        this.uiCamera.position.z = 100;

        /**
         * 所有插件
         * @type {Array}
         */
        this.plugins = [];

        /**
         * 所有灯光
         */
        this.lights = [];


        this.oldSize = {};

        /**
         * 本场景占位符
         */
        this.placeholders = [];

        /**
         * 当前舞台信息，来自 .json的 myScene 文件
         */
        this.sceneInfo = null;
        /**
         * 当前舞台
         * @type {Scene}
         */
        this.scene = null;

        /**
         * 当前摄像头
         */
        this.camera = null;

        /**
         * 所有辅助器
         */
        this.helpers = [];

        /**
        * 声音侦听器。
        */
        this.audioListener = J.udt.isMiniP ? null : new THREE.AudioListener();

        var q = J.uls.getItem('quality');
        //画质，移动默认低画质，PC高画质
        if (q == null) {
            if (J.udt.isMobile)
                q = 'lowp';
            else
                q = 'highp';
        }

        this._quality = q;

        /**
         * 全局主题
         */
        this.theme = new Theme();

        /**
         * 舞台分区，目前仅支持二维分区
         */
        this.sectorInfo = {
            rows: 10,
            cols: 10,
            sectors: []
        };
    }

    /**
     * 当前透视摄像头看到的高度
     */
    get height() {
        return this.camera.position.distanceTo(
            new THREE.Vector3(0, 0, 0)) *
            Math.tan(J.um.getRadian(this.camera.fov / 2))
            * 2;
    }

    /**
     * 当前透视摄像头看到的宽度
     */
    get width() {
        return this.height * this.aspect;
    }

    get children() {
        return this.scene.children;
    }
    get uiChildren() {
        return this.uiScene.children;
    }

    init(args) {

        var _t = this;

        this.sceneInfo = CPM.scene;

        this.parseScene();

        /**
         * 声音侦听器。
         * 目前舞台上只有一个摄像头，故暂只支持一个摄像头绑定。
         */
        if (!J.udt.isMiniP)
            this.camera.add(this.audioListener);

        this.parseChildren(this.sceneInfo.children, this);
        this.parseUIChildren(this.sceneInfo.uiChildren, this);
        this.layoutUI();

        this.parsePlugins(() => {

            _t.animate();

            window.onorientationchange = window.onresize = function () {
                var obj = {
                    width: _t.wWidth,
                    height: _t.wHeight
                };
                _t.onResized(_t.oldSize, obj);
                _t.fire("resized", _t.oldSize, obj);

                _t.layoutUI();

            };

            this.parseHelpers();

            //舞台和物体载入完成后需刷新一次摄像头， 不刷新会有物体变形的现象
            _t.refreshCamera();

            super.init(args);

        });

    }

    /**
     * 添加对象到舞台上
     * @param {*} obj
     */
    add(obj) {

        var _t = this;

        if (obj.isGameObject) {
            obj.parent = _t.scene;
            _t.scene.add(obj);
        } else if (obj.isUIObject) {
            obj.parent = _t.uiScene;
            _t.uiScene.add(obj);
        } else {
            obj.parent = _t.scene;
            _t.scene.add(obj);
        }

        //画质选项
        if (obj.isDirectionalLight || obj.isPointLight || obj.isSpotLight) {
            switch (this.quality) {
                case 'highp':

                    obj.shadow.mapSize.width = 2048;
                    obj.shadow.mapSize.height = 2048;

                    break;

                case 'mediump':
                    obj.shadow.mapSize.width = 1024;
                    obj.shadow.mapSize.height = 1024;

                    break;

                case 'lowp':
                    obj.shadow.mapSize.width = 512;
                    obj.shadow.mapSize.height = 512;

                    break;

            }
        }

    }

    /**
     * 从舞台移除子对象
     * @param  {*} obj
     */
    remove(obj) {

        var _t = this;

        if (obj instanceof GameObject) {
            _t.scene.remove(obj);

        } else if (obj instanceof UIObject) {
            _t.uiScene.remove(obj);

        } else {
            _t.scene.remove(obj);
        }

        if (obj instanceof GObject) {
            obj.onRemoved();
            obj.fire('removed', obj);
        }
    }

    /**
     * 看向某个物体，让其最大化充满屏幕
     */
    //TODO: 这个没有看向哪个面，从哪个角度，这是不妥的
    lookOn(gobj) {

        var boundingBox = new THREE.Box3();
        boundingBox.setFromObject(gobj);

        var size = new THREE.Vector3();

        boundingBox.getSize(size);

        //求出要物体最大化的时候，摄像头应该距离物体多少米
        var dist = Math.max(size.x, size.y) / 2 / Math.tan(Math.PI / 180 * SM.camera.fov / 2);

        //物体正 Z 方向（前面）
        var positiveZ = new THREE.Vector3();
        gobj.getWorldDirection(positiveZ);

        var pos = measureUtils.getWorldPosition(gobj);

        //先把摄像头放置到物体所在的位置，并且在物体的方向上向前走一个距离
        SM.camera.position.copy(pos.clone().add(positiveZ.multiplyScalar(dist)));

        SM.camera.lookAt(pos);
    }

    /**
     * 按目前摄像头方向，往前行进多少距离
     */
    cameraTo(dist) {
        SM.camera.translateOnAxis(SM.camera.position.clone().normalize().negate(), dist);
    }

    /**
     * 摄像头左移多少距离
     */
    cameraLeft(dist) {
        SM.camera.translateOnAxis(new THREE.Vector3(-1, 0, 0), dist);
    }

    /**
     * 摄像头右移多少距离
     */
    cameraRight(dist) {
        SM.camera.translateOnAxis(new THREE.Vector3(1, 0, 0), dist);
    }

    /**
     * 摄像头后退多少距离
     */
    cameraBack(dist) {
        SM.camera.translateOnAxis(new THREE.Vector3(0, 0, 1), dist);
    }

    /**
    * 摄像头前进多少距离
    */
    cameraForward(dist) {
        SM.camera.translateOnAxis(new THREE.Vector3(0, 0, -1), dist);
    }

    /**
     * 摄像头飞行，按照点集（世界坐标）。
     * 点集采用二次贝塞尔曲线来走（控制点相等方向，画出来是个圆弧）。
     */
    cameraFly(points, duration, delay) {

        var dis = 0.1;
        J.tween({
            from: { idx: 0 },
            to: { idx: points.length - 1 },
            duration: duration,
            delay: delay,
            onUpdate: (data) => {
                SM.camera.translateOnAxis(
                    SM.camera.worldToLocal(points[data.idx]).normalize(),
                    dis
                );
            }
        }).start();
    }

    /**
     * 设定视图。
     * 顶视图 top、左视图 left、右视图 right、底视图 bottom、45度透视图 45p
     */
    set view(v) {

        switch (v) {

            case 'top':
                this.camera.position.x = this.camera.position.z = 0;
                this.camera.position.y = Math.abs(this.camera.position.y);
                this.camera.lookAt(this.scene.position);
                break;

            case 'bottom':
                this.camera.position.x = this.camera.position.z = 0;
                this.camera.position.y = -Math.abs(this.camera.position.y);
                this.camera.lookAt(this.scene.position);
                break;

            case 'left':
                this.camera.position.y = this.camera.position.z = 0;
                this.camera.position.x = Math.abs(this.camera.position.x);
                this.camera.lookAt(this.scene.position);
                break;

            case 'right':
                this.camera.position.y = this.camera.position.z = 0;
                this.camera.position.x = -Math.abs(this.camera.position.x);
                this.camera.lookAt(this.scene.position);
                break;

            case "45p":
                //将y/z设为同样的数值，并将 rotation.x 转 -45
                var max = Math.max(this.camera.position.y, this.camera.position.z);
                this.camera.position.y = this.camera.position.z = max;
                this.camera.rotation.set(J.um.getRadian(-45), 0, 0);
                this.camera.lookAt(this.scene.position);
                break;
        }
    }

    /**
     * 设置视图风格。
     * 线框 wireframe
     */
    set viewStyle(v) {

        if (v == 'wireframe') {
            this.scene.children.forEach(child => {
                child.wireframe = true;
            })
        }
    }

    get quality() {
        return this._quality;
    }
    /**
     * 设置画质。
     * 需刷新后生效。
     */
    set quality(v) {
        this._quality = v;
        J.uls.setItem('quality', v);
        this.change('quality');
    }

    /**
     * 解析辅助器
     * 可以是 "ArrowHelper","AxesHelper","BoxHelper","Box3Helper","CameraHelper","DirectionalLightHelper","FaceNormalsHelper","GridHelper","PolarGridHelper","HemisphereLightHelper","PlaneHelper","PointLightHelper","RectAreaLightHelperHelper","SkeletonHelper","SpotLightHelper","VertexNormalsHelper"，注意存储的是 Helper 对象。
     * 对应的对象是 uuid 。
     */
    parseHelpers() {

        this.sceneInfo.helpers.forEach((helper) => {

            switch (helper.type) {
                case "DirectionalLightHelper":

                    var light = this.lights.find((l) => { return l.uuid == helper.light; });
                    var helper = new THREE.DirectionalLightHelper(light, helper.size, helper.color);
                    SM.add(helper);
                    this.helpers.push(helper);

                    break;

                // TODO: 没搞完
            }
        });
    }

    /**
    * 加载所有插件，包含控制器
    */
    parsePlugins(onCompleted) {

        var _t = this;

        if (J.isN(this.sceneInfo.plugins)) {
            onCompleted();
            return;
        }

        //读取所有已经注册的插件
        //TODO: 这里未来可能有性能问题，未来应该专门搞一个 plugin 的市场 和 CDN（模仿 staticfile.org），方便统一管理
        new Loader().load(MM.domain + '/libs/plugins/config.json', (ps) => {

            //依据 uuid，先填充好信息
            ps.forEach((plugin) => {

                _t.sceneInfo.plugins.forEach((p) => {
                    if (p.uuid === plugin.uuid) {
                        p.src = MM.domain + '/libs/plugins/' + plugin.src;
                        //如果没有指定别名，则用本来的名字
                        p.varName = J.def(p.varName, plugin.varName);
                        p.type = plugin.type;
                        p.name = plugin.name;
                        //先找到配置文件，待会再解析
                        p.configFile = MM.domain + '/libs/plugins/' + plugin.config;
                    }
                });

            });


            //开始递归
            _t.parsePlugin(_t.sceneInfo.plugins[0], 0, onCompleted);

        });

    };
    /**
     * 加载控制器
     */
    parseControl(control) {
        var _t = this;

        switch (control.varName) {

            //轨道控制器
            case "orbitControls":
                var m = _t[control.varName] = new THREE.OrbitControls(_t.camera, _t.renderer.domElement);
                Object.assign(m, control.config);
                m.target = new THREE.Vector3(m.target.x, m.target.y, m.target.z);
                m.update();
                _t.plugins.push(m);
                break;

            //拖拽控制器
            case "dragControls":
                var m = _t[control.varName] = new THREE.DragControls([], _t.camera, _t.renderer.domElement);
                Object.assign(m, control.config);
                m.target = new THREE.Vector3(m.target.x, m.target.y, m.target.z);
                _t.plugins.push(m);
                break;

        }
    };

    /**
     * 加载单个插件
     */
    parsePlugin(p, idx, onCompleted) {

        var _t = this;

        //加载插件配置文件
        new Loader().load(p.configFile, (config) => {

            p.config = config;
            //加载插件脚本
            J.importThen(p.src, (clazz) => {

                switch (p.type) {
                    //控制器
                    case "control":
                        _t.parseControl(p);
                        break;

                    //对象，需加载到舞台上
                    case "object":
                        var m = new clazz.default(p.config);

                        if (m.isGameObject || m.isUIObject) {

                            _t.on("added", (args) => {
                                _t.plugins.push(args);
                                _t[p.varName] = args;
                            })

                            _t.add(m);

                        }

                        break;

                    //类，无需加载到舞台上
                    case "class":
                        var m = _t[p.varName] = clazz.default;
                        _t.plugins.push(m);
                        break;
                }


                console.log("a plugin [" + p.varName + "] was loaded. " + (idx + 1) + " /" + _t.sceneInfo.plugins.length);

                if (idx < _t.sceneInfo.plugins.length - 1) {
                    _t.parsePlugin(_t.sceneInfo.plugins[idx + 1], idx + 1, onCompleted);
                } else {
                    //最终完成
                    onCompleted();
                }

            });
        });

    };

    /**
     * 创建舞台
     */
    parseScene() {
        var _t = this;

        if (MM.dimension == "3D") {

            var loader = new THREE.ObjectLoader();
            this.scene = loader.parseObject(this.sceneInfo.scene);

            var camera = this.sceneInfo.camera;
            if (camera.position)
                camera.position = [camera.position.x, camera.position.y, camera.position.z];
            if (camera.rotation)
                camera.rotation = [camera.rotation.x, camera.rotation.y, camera.rotation.z];
            if (camera.scale)
                camera.scale = [camera.scale.x, camera.scale.y, camera.scale.z];

            this.camera = loader.parseObject(camera);
            this.camera.aspect = _t.wWidth / _t.wHeight;
            this.camera.lookAt(this.scene.position);

            switch (this.quality) {
                case 'highp':

                    this.renderer = new THREE.WebGLRenderer({
                        canvas: canvas,
                        antialias: true,
                        precision: 'highp'
                    });

                    this.renderer.shadowMap.enabled = true;
                    this.renderer.shadowMap.type = THREE.PCFSoftShadowMap;

                    break;

                case 'mediump':
                    this.renderer = new THREE.WebGLRenderer({
                        canvas: canvas,
                        antialias: true,
                        precision: 'mediump'
                    });

                    this.renderer.shadowMap.enabled = true;
                    this.renderer.shadowMap.type = THREE.PCFShadowMap;

                    break;

                case 'lowp':
                    this.renderer = new THREE.WebGLRenderer({
                        canvas: canvas,
                        antialias: false,
                        precision: 'lowp'
                    });

                    this.renderer.shadowMap.enabled = true;
                    this.renderer.shadowMap.type = THREE.BasicShadowMap;

                    break;

            }

            this.renderer.setPixelRatio(window.devicePixelRatio);
            this.renderer.setSize(_t.wWidth, _t.wHeight);
            this.renderer.autoClear = false;

            //lights
            if (this.sceneInfo.lights) {

                for (var i = 0; i < this.sceneInfo.lights.length; i++) {

                    var light = this.sceneInfo.lights[i];
                    //在 objectloader.js 源码中，对于 position/rotation/scale 等都是采用 fromArray 的方式，故先做转换
                    if (light.position)
                        light.position = [light.position.x, light.position.y, light.position.z];
                    if (light.rotation)
                        light.rotation = [light.rotation.x, light.rotation.y, light.rotation.z];
                    if (light.scale)
                        light.scale = [light.scale.x, light.scale.y, light.scale.z];

                    var rlight = loader.parseObject(light);
                    this.add(rlight);

                    this.lights.push(rlight);
                }
            }

        }
    }

    /**
     * 孵化对象
     * @param  {string} key
     * @param  {Object} value
     * @return {GameObject}
     */
    spawn(key, value) {
        var gobj = RM.getItem('gameobject', key, value).clone();
        return gobj;
    }

    /**
     * 孵化对象
     * @param  {string} key
     * @param  {Object} value
     * @return {UIObject}
     */
    spawnUI(key, value) {
        var gobj = RM.getItem('uiobject', key, value).clone();
        return gobj;
    }

    /**
     * 对UI从顶到底布局一次
     */
    layoutUI() {

        this.fire('beforeLayoutUI');

        //先做容器的拉伸
        this.uiChildren.forEach((child) => {
            child.stretch();
        });

        //再具体布局
        this.uiChildren.forEach((child) => {
            child.layout();
        });

        this.fire('afterLayoutUI');
    }

    /**
     * 解析 UI 子对象。递归。
     *
     * @param  {Array} children
     * @param  {*} parent
     */
    parseUIChildren(children, parent) {

        if (J.isNull(children))
            return;

        for (var i = 0; i < children.length; i++) {
            var cdr = children[i];

            var uiobj = new window[cdr.gtype](cdr);

            parent.add(uiobj);

            if (!J.isNull(cdr.children)) {
                this.parseUIChildren(cdr.children, uiobj);
            }

        }
    }

    /**
     * 解析子对象。
     * 不需要递归的原因，是因为舞台上就是放置 Object3D 对象，
     * 仅仅有一部分转换为 GameObject 对象，
     * 故，GameObject 对象并不是树形结构。
     *
     * @param  {Array} children
     * @param  {*} parent
     */
    parseChildren(children, parent) {

        if (J.isNull(children)) return;

        for (var i = 0; i < children.length; i++) {
            var ch = children[i];
            //多个孩子可能采用同一个 gameobject，所以要克隆一份
            var gobj = RM.getItem('gameobject', 'uuid', ch.gameobject);
            gobj = gobj.clone();

            parent.add(gobj);

            if (!J.isNull(ch.children)) {
                this.parseChildren(ch.children, gobj);
            }
        }
    }

    animate() {

        var _t = SM;
        requestAnimationFrame(_t.animate);

        //默认支持 TWEEN 更新，其他文件无需再次调用
        TWEEN.update();

        //调用物体重绘
        for (var i = 0; i < _t.children.length; i++) {
            var obj = _t.children[i];
            //虽然 Object3D 是继承 GObject，但没必要让它去重绘，浪费 CPU
            if (obj.isGameObject) {
                obj.update();
            }
        }

        //调用UI重绘
        for (var i = 0; i < _t.uiChildren.length; i++) {
            var obj = _t.uiChildren[i];
            //虽然 Object3D 是继承 GObject，但没必要让它去重绘，浪费 CPU
            if (obj.isUIObject) {
                obj.update();
            }
        }

        //调用插件重绘
        for (var i = 0; i < _t.plugins.length; i++) {
            var p = _t.plugins[i];
            if (typeof (p) == 'object' && p.update) {
                if (THREE.OrbitControls && p instanceof THREE.OrbitControls) {
                    if (p.autoRotate || p.enableDamping)
                        p.update();
                }
                else
                    p.update();
            }
        }

        _t.onUpdated();
        _t.fire("updated");

        //触发所有管理器的更新
        if (GM.isInited)
            GM.update();
        if (LGM.isInited)
            LGM.update();
        if (MM.isInited)
            MM.update();
        if (CPM.isInited)
            CPM.update();
        if (RM.isInited)
            RM.update();


        _t.renderer.clear();
        _t.renderer.render(_t.scene, _t.camera);

        //渲染 UI 层
        _t.renderer.clearDepth();
        _t.renderer.render(_t.uiScene, _t.uiCamera);
    }

    refreshCamera() {
        var _t = this;
        _t.camera.updateProjectionMatrix();
        _t.camera.lookAt(this.scene.position);
    }

    onResized() {
        var _t = this;

        this.oldSize = {
            width: _t.wWidth,
            height: _t.wHeight
        };
        _t.wWidth = window.innerWidth;
        _t.wHeight = window.innerHeight;
        _t.camera.aspect = _t.aspect = _t.wWidth / _t.wHeight
        _t.camera.updateProjectionMatrix();

        _t.uiCamera.left = -_t.wWidth / 2;
        _t.uiCamera.right = _t.wWidth / 2;
        _t.uiCamera.top = _t.wHeight / 2;
        _t.uiCamera.bottom = -_t.wHeight / 2;
        _t.uiCamera.updateProjectionMatrix();

        _t.renderer.setSize(_t.wWidth, _t.wHeight);

        //调用UI重绘
        _t.layoutUI();

    }

    /**
     * 获取某个孩子
     * @param {string} key 
     * @param {*} value 
     * @param {array} children 
     */
    _getChild(key, value, children) {

        var r = null;
        for (var i = 0; i < children.length; i++) {
            var child = children[i];

            if (child[key] == value) {
                r = child;
                break;
            }

            if (child.children.length > 0) {
                r = this._getChild(key, value, child.children);
                if (r != null)
                    break;
            }
        }

        return r;
    }

    /**
     * 遍历每个孩子
     * @param {function} callback 
     * @param {array} children 
     */
    _eachChild(callback, children) {
        for (var i = 0; i < children.length; i++) {
            var child = children[i];

            callback.call(this, child, i, children);

            if (child.children.length > 0) {
                this._eachChild(callback, child.children);
            }
        }
    }

    eachChild(callback) {
        this._eachChild(callback, this.children);
    }

    eachUIChild(callback) {
        this._eachChild(callback, this.uiChildren);
    }

    /**
     *  通过键值条件找到 GameObject 对象
     * @param {string|GameObject} key
     * @param {GameObject} value
     */
    getChild(key, value) {
        return this._getChild(key, value, this.children);
    }

    /**
     *  通过键值条件找到 UIObject 对象
     * @param {string|UIObject} key
     * @param {UIObject} value
     */
    getUIChild(key, value) {
        return this._getChild(key, value, this.uiChildren);
    }

};

G.SceneManager = SceneManager;
/**
 * 游戏逻辑管理器。
 * 是游戏过程中的碰撞、行为、摧毁等，不是关卡和波数管理，
 * 也用于控制界面逻辑。
 */
export class LogicManager extends Manager {

    constructor(options) {
        super(options);

        window.LGM = this;

        /**
         * 输入管理器
         */
        this.inputor = new Inputor();


        /**
         * 时钟
         * @type {THREE.Clock}
         */
        this.clock = new THREE.Clock();

        /**
         * 逝去的时间（帧动画更新），勿写入
         * @type {Number}
         */
        this.elapsed = 0;

        /**
         * 距离上次更新的时间，勿写入。
         * 直接可以理解为每一帧的时间跨度。
         * @type {Number}
         */
        this.delta = 0;
    }


    select(gobjs) {
        this.onSelected(gobjs);
        this.fire('selected', gobjs);
    }

    unselect(gobjs) {
        this.onUnSelected(gobjs);
        this.fire('unselected', gobjs);
    }

    onSelected(gobjs) {

    }

    onUnSelected(gobjs) {

    }

    onInited(args) {
        super.onInited(args);
        this.inputor.init();
    }

    onUpdated() {
        super.onUpdated();


        this.delta = this.clock.getDelta();
        this.elapsed = this.clock.elapsedTime;

        this.inputor.update();

    }

    /**
      * 定时器
      * @param  {function} handler 要处理的函数 必需
      * @param  {int} duration  间隔时间 可选
      * @param  {function} delay 延迟时间 可选
      */
    interval(handler, duration, delay) {

        if (J.isN(handler))
            return null;

        var it = {
            handler: handler,

            duration: J.def(duration, 1000),
            delay: J.def(delay, 1000),

            count: 0,
            status: "init"
        };

        //开始
        it.start = (() => {

            //防止反复 start
            if (it.status == "start")
                return it;

            //多加一个 twDelay 是因为 tween 会在每次的 repeat 里面都会 delay
            it.twDelay = J.tween({
                delay: it.delay,
                from: { i: -1 },
                to: { i: 1 },
                //tween 默认 3 秒时长，如果传 0 也是等同于 3 秒，故传 1 毫秒
                duration: 1,
                onComplete: () => {

                    it.twDuration = J.tween({
                        from: { i: -1 },
                        to: { i: 1 },
                        duration: it.duration,
                        repeat: Infinity,
                        onRepeat: () => {
                            it.handler(++it.count)
                        }
                    }).start();

                }
            }).start();

            it.status = "start";
            return it;

        });
        //接着开始（停止暂停）
        it.resume = (() => {
            it.status = "start";
            if (it.twDuration)
                it.twDuration.resume();
            return it;
        });
        //暂停
        it.pause = (() => {
            it.status = "pause";
            if (it.twDuration)
                it.twDuration.pause();
            return it;
        });
        //重新开始
        it.restart = (() => {
            it.stop();
            it.start();
            return it;
        });
        //停止
        it.stop = (() => {
            it.status = "stop";
            if (it.twDuration)
                it.twDuration.stop();
            it.count = 0;
            return it;
        });

        return it;
    }

}

G.LogicManager = LogicManager;
/**
 * 输入指示器。
 * 主要是显示鼠标与物体交叠的。
 */
class InputorHelper extends GObject {

    constructor(options) {
        super(options);

        this.isInputorHelper = true;
        this.gtype = 'InputorHelper';

    }
    onInited(args) {
        super.onInited(args);

        //交叠的辅助显示对象（绿色矩形）
        var geometry = new THREE.PlaneBufferGeometry(10, 10, 5, 5);
        geometry.rotateX(-Math.PI / 2);
        var material = new THREE.MeshBasicMaterial({
            color: 0x00cc00,
            wireframe: true
        });
        this.unit = new THREE.Mesh(geometry, material);
    }

    onUpdated(args) {

        super.onUpdated(args);

        if (LGM.inputor.isInited && SM.isInited) {

            if (LGM.inputor.intersects.length > 0) {
                //以找到的第一个为准放置
                var ist = LGM.inputor.intersects[0].intersect;
                if (ist.face) {
                    this.unit.position.copy(ist.point).add(ist.face.normal);
                    this.unit.position.y += 1;
                }
            }
        }
    }
}

G.InputorHelper = InputorHelper;
/**
 * 主入口文件
 * 进入游戏
 * @param {string} url game.json 所在地址
 * @param {function} callback game.json 加载完成的回调
 * @param {string} domain 覆盖 game.json 里面指定的域名，不设置就以 game.json 为准
 */
G.GO = function (url, callback, domain) {

    var mm = new MapManager({
        url: url,
        domain: domain
    });

    mm.on('inited', function () {
        if (callback) callback();
    });

    mm.init();
};

